<?php
// This file is part of the tool_certificate plugin for Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * This file contains the certificate element code's core interaction API.
 * * Modified version of the certificateelement_code by 2013 Mark Nelson <markn@moodle.com>
 *
 * @package    certificateelement_customcode
 * @copyright  2024 National Corporate Training Pty Ltd (https://nct.net.au/) <believe@nct.net.au>
 * @author     LMSACE Dev Team <lmsace.com>
 */

namespace certificateelement_customcode;

use tool_certificate\customfield\issue_handler;


/**
 * The certificate element code's core interaction API.
 * * Modified version of the certificateelement_code by 2013 Mark Nelson <markn@moodle.com>
 *
 * @package    certificateelement_customcode
 * @copyright  2024 National Corporate Training Pty Ltd (https://nct.net.au/) <believe@nct.net.au>
 * @author     LMSACE Dev Team <lmsace.com>
 */
class element extends \tool_certificate\element {

    /**
     * @var int Option to display only code
     */
    const DISPLAY_CODE = 1;

    /**
     * @var int Option to display code and a link
     */
    const DISPLAY_CODELINK = 2;

    /**
     * This function renders the form elements when adding a certificate element.
     *
     * @param \MoodleQuickForm $mform the edit_form instance
     */
    public function render_form_elements($mform) {

        // Get the possible date options.
        $options = [];
        $options[self::DISPLAY_CODE] = get_string('displaycode', 'certificateelement_code');
        $options[self::DISPLAY_CODELINK] = get_string('displaycodelink', 'certificateelement_code');

        $mform->addElement('select', 'display', get_string('display', 'certificateelement_code'), $options);
        $mform->addHelpButton('display', 'display', 'certificateelement_code');
        $mform->setDefault('display', self::DISPLAY_CODE);

        parent::render_form_elements($mform);
        $mform->setDefault('width', 60);
    }

    /**
     * Handles saving the form elements created by this element.
     * Can be overridden if more functionality is needed.
     *
     * @param \stdClass $data the form data or partial data to be updated (i.e. name, posx, etc.)
     */
    public function save_form_data(\stdClass $data) {
        $data->data = json_encode(['display' => $data->display]);
        parent::save_form_data($data);
    }

    /**
     * Formats a code according to current display value
     *
     * @param string $code
     * @return string
     */
    protected function format_code(\stdClass $issue) {
        global $SITE, $USER, $CFG;

        if (isset($issue->data) && !empty($issue->data)) {
            $decoded = json_decode($issue->data);
            $coursename = property_exists($decoded, 'courseshortname') ? $decoded->courseshortname : "";
        }

        if (!isset($coursename) || empty($coursename)) {
            $course = get_course($issue->courseid ?: $SITE->id);
            $coursename = $course->shortname;
        }

        // Year of the ceritifcate generated.
        $created = $issue->timecreated ?? time();
        $year = date('y', $created ?: time());

        // Issued unique id.
        $issueid = str_pad($issue->id, 4, '0', STR_PAD_LEFT);

        // Shortname of the site.
        $siteshortname = $SITE->shortname;

        // Custom code for the certificate in the format of,
        // "Site shortname / course shortname / year of the certificate issues / issued record id with leading 0.
        $customcode = $siteshortname . '/' . $coursename . '/' . $year . '/' . $issueid;

        $data = json_decode($this->get_data());

        // Finalize the output data based on the user selections,
        // Display the code as normal code or display the code with verification url to verify the certificate.
        switch ($data->display) {

            case self::DISPLAY_CODE:
                $display = $customcode;
                break;
            case self::DISPLAY_CODELINK:

                $code = $issue->code ?? '';
                if ($code == '') {
                    $code = \tool_certificate\certificate::generate_code($issue->userid ?: $USER->id);
                }

                $display = \html_writer::link(\tool_certificate\template::verification_url($code), $customcode);
                break;
            default:
                $display = $customcode;
        }

        return $display;
    }

    /**
     * Handles rendering the element on the pdf.
     *
     * @param \pdf $pdf the pdf object
     * @param bool $preview true if it is a preview, false otherwise
     * @param \stdClass $user the user we are rendering this for
     * @param \stdClass $issue the issue we are rendering
     */
    public function render($pdf, $preview, $user, $issue) {
        global $SITE, $USER;

        if ($preview) {
            $code = \tool_certificate\certificate::generate_code($user->id);
            $customcode = (object) ['courseid' => $SITE->id, 'id' => $user->id, 'code' => $code];
        } else {
            $customcode = $issue;
        }

        \tool_certificate\element_helper::render_content($pdf, $this, $this->format_code($customcode));
    }

    /**
     * Render the element in html.
     *
     * This function is used to render the element when we are using the
     * drag and drop interface to position it.
     *
     * @return string the html
     */
    public function render_html() {
        global $SITE, $USER;

        $code = \tool_certificate\certificate::generate_code();
        $customcode = (object) ['courseid' => $SITE->id, 'id' => $USER->id, 'code' => $code];

        $html = \tool_certificate\element_helper::render_html_content($this, $this->format_code($customcode));

        return $html;
    }

    /**
     * Returns the width.
     *
     * @return int
     */
    public function get_width(): int {
        $width = $this->persistent->get('width');
        return $width > 0 ? $width : 60;
    }
}
