<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Filters results to current course only.
 *
 * @package    auth_magic
 * @copyright  2020 bdecent gmbh <https://bdecent.de>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

namespace auth_magic\local\block_dash\filter;

use block_dash\local\data_grid\filter\condition;
use coding_exception;
use dml_exception;
use moodleform;
use MoodleQuickForm;

/**
 * Filters results to current course only.
 *
 * @package block_dash
 */
class campaign_available_cohorts_condition extends condition {

    /**
     * Get filter SQL operation.
     *
     * @return string
     */
    public function get_operation() {
        return self::OPERATION_IN_OR_EQUAL;
    }

    /**
     * Get condition label.
     *
     * @return string
     * @throws \coding_exception
     */
    public function get_label() {
        if ($label = parent::get_label()) {
            return $label;
        }

        return get_string('availableforcohort', 'auth_magic');
    }

    /**
     * Add form fields for this filter (and any settings related to this filter.)
     *
     * @param moodleform $moodleform
     * @param MoodleQuickForm $mform
     * @param string $fieldnameformat
     */
    public function build_settings_form_fields(
        moodleform $moodleform, MoodleQuickForm $mform, $fieldnameformat = 'filters[%s]'): void {

        parent::build_settings_form_fields($moodleform, $mform, $fieldnameformat); // Always call parent.

        $fieldname = sprintf($fieldnameformat, $this->get_name());

        $cohortslist = \cohort_get_all_cohorts();
        $cohorts = $cohortslist['cohorts'];
        if ($cohorts) {
            array_walk($cohorts, function(&$value) {
                $value = $value->name;
            });
        }
        $select = $mform->addElement('select', $fieldname . '[availableforcohorts]', '', $cohorts, ['class' => 'select2-form']);
        $mform->hideIf($fieldname . '[availableforcohorts]', $fieldname . '[enabled]');
        $select->setMultiple(true);
    }


    /**
     * Get values from filter based on user selection. All filters must return an array of values.
     *
     * Override in child class to add more values.
     *
     * @return array
     */
    public function get_values() {
        if (isset($this->get_preferences()['availableforcohorts']) && is_array($this->get_preferences()['availableforcohorts'])) {
            $status = $this->get_preferences()['availableforcohorts'];
            return $status;
        }
        return [];
    }

    /**
     * Return where SQL and params for placeholders.
     *
     * @return array
     * @throws \coding_exception|\dml_exception
     */
    public function get_sql_and_params() {
        global $DB;

        list($sql, $params) = parent::get_sql_and_params();

        $availableforcohorts = $this->get_preferences()['availableforcohorts'];
        $filtersql = [];
        if (!empty($availableforcohorts) && $sql) {
            foreach ($availableforcohorts as $index => $value) {
                // Create a named parameter for each value.
                $paramname = "availablecohort$index";  // Create unique param name for each iteration.
                $filtersql[] = " amc.restrictcohorts IS NOT NULL AND amc.restrictcohorts != '' AND JSON_CONTAINS(amc.restrictcohorts, :$paramname, '$')";
                // Bind the parameter value.
                $params[$paramname] = json_encode($value);
            }
            $sql = implode(" AND ", $filtersql);
        }
        return [$sql, $params];
    }
}