<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Levels additional options for activities.
 *
 * @package   format_levels
 * @copyright 2022 bdecent gmbh <https://bdecent.de>
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

namespace format_levels;

use context_course;
use moodle_url;

/**
 * Additional methods to manage the module options.
 */
class options {

    /**
     * Course format class instance
     *
     * @var \format_levels
     */
    public $format;

    /**
     * Course record.
     *
     * @var \stdclass
     */
    public $course;

    /**
     * Course completion info instance.
     *
     * @var \completion_info
     */
    public $completioninfo;

    /**
     * Module criteria for the course completion.
     *
     * @var array
     */
    public $completionmodules;

    /**
     * Setup course, completioninfo and formats.
     *
     * @param stdclass $course
     */
    public function __construct($course) {
        $this->format = course_get_format($course);
        $this->course = $this->format->get_course();
        $this->completioninfo = new \completion_info($this->course);
        $modulecriteria = $this->completioninfo->get_criteria(COMPLETION_CRITERIA_TYPE_ACTIVITY);
        $this->completionmodules = $modulecriteria ? array_column($modulecriteria, 'moduleinstance') : [];
    }

    /**
     * Find the given string is JSON format or not.
     *
     * @param string $string
     * @return bool
     */
    public static function is_json($string) {
        return (is_null(json_decode($string))) ? false : true;
    }

    /**
     * Get custom additional field value for the module.
     *
     * @param int $cmid Course module id.
     * @param string $name Module additional field name.
     * @return null|string Returns value of given module field.
     */
    public static function get_option(int $cmid, $name) {
        global $DB;
        if ($data = $DB->get_field('format_levels_options', 'value',
            ['cmid' => $cmid, 'name' => $name])) {
            return $data;
        }
        return null;
    }

    /**
     * Get levels additional fields values for the given module.
     *
     * @param int $cmid course module id.
     * @return stdclass $options List of additional field values
     */
    public static function get_options($cmid) {
        global $DB;
        $options = new \stdclass;
        if ($records = $DB->get_records('format_levels_options', ['cmid' => $cmid])) {
            foreach ($records as $key => $field) {
                $options->{$field->name} = self::is_json($field->value)
                    ? json_decode($field->value, true) : $field->value;
            }
        }
        return $options;
    }

    /**
     * Insert the additional module fields data to the table.
     *
     * @param int $cmid Course module id.
     * @param int $courseid Course id.
     * @param string $name Field name.
     * @param mixed $value value of the field.
     * @return void
     */
    public static function insert_option(int $cmid, int $courseid, $name, $value) {
        global $DB;

        $record = new \stdClass;
        $record->cmid = $cmid;
        $record->courseid = $courseid;
        $record->name = $name;
        $record->value = $value ?: '';
        $record->timemodified = time();
        if ($exitrecord = $DB->get_record('format_levels_options', [
            'cmid' => $cmid, 'courseid' => $courseid, 'name' => $name])) {
            $record->id = $exitrecord->id;
            $record->timecreated = $exitrecord->timecreated;
            $DB->update_record('format_levels_options', $record);
        } else {
            $record->timecreated = time();
            $DB->insert_record('format_levels_options', $record);
        }
    }

    /**
     * Generate the x and y initial positions for new modules.
     *
     * @param int $cmid course module id.
     * @param stdclass $section section basic values of shared module.
     * @param stdclass $course
     * @return array
     */
    public static function get_default_module_position($cmid, $section, $course) {
        $modinfo = course_get_format($course)->get_modinfo();
        $section = $modinfo->get_section_info($section);
        $sections = $modinfo->get_sections();
        if (isset($sections[$section->section])) {
            $cms = $sections[$section->section];
            $cms = array_filter($cms, function($cm) use ($modinfo) {
                $mod = $modinfo->get_cm($cm);
                return (!$mod->deletioninprogress);
            });
            if ($cmid) {
                $pos = array_search($cmid, $cms);
                $prevcm = isset($cms[$pos - 1]) ? $cms[$pos - 1] : null;
            } else {
                $prevcm = end($cms);
            }

            if ($prevcm) {
                $cmopt = self::get_options($prevcm);
                $positions = isset($cmopt->xposition) && !empty($cmopt->xposition)
                ? [(int) $cmopt->xposition + LEVELS_DEFAULT_XPOSITION, (int) $cmopt->yposition + LEVELS_DEFAULT_YPOSITION] : null;
            }
        }
        $default = [LEVELS_DEFAULT_XPOSITION, LEVELS_DEFAULT_YPOSITION];
        return ($positions) ?? $default;
    }

    /**
     * Verify the current page is section page.
     *
     * @return bool
     */
    public function is_sectionpage() {
        global $CFG, $PAGE;

        if ($CFG->branch >= 404) {
            $pageview = ($PAGE->bodyid == 'page-course-section') ? true : false;
        } else {
            $pageview = optional_param('section', false, PARAM_INT);
        }

        return $pageview;
    }

    /**
     * Fetch the Styles Rules to attach in head.
     *
     * @param int $sectionnum
     * @return void
     */
    public function get_dynamic_css_rules($sectionnum=null) {
        global $PAGE, $CFG;
        // Course body image.
        $style = '';
        $coursecontext = \context_course::instance($this->course->id);
        $coursebgimage = $this->get_course_images('coursebgimage');

        // Add section bckground images to style.
        if ($CFG->branch < 404) {
            $sectionnum = optional_param('section', $sectionnum, PARAM_INT);
        }

        if ($sectionnum !== null && $sectionnum) {
            $section = course_get_format($this->course)->get_section($sectionnum);
            $sectionoptions = $this->format->get_format_options($section->section);
            $backgroundwidth = $sectionoptions['backgroundwidth'] ?? (get_config('format_levels', 'backgroundwidth') ?: '');
            $backgroundheight = $sectionoptions['backgroundheight'] ?? (get_config('format_levels', 'backgroundheight') ?: '');
            $sectionimage = $this->get_section_bgimage($section);
            if (!empty($sectionimage)) {
                $size = ($backgroundwidth && $backgroundheight) ? $backgroundwidth.'px '.$backgroundheight.'px' : '100% 100%';
                $style .= sprintf("#page{background-image:url(%s);}", $sectionimage);
                $style .= '#page {
                    background-position: unset;
                    background-attachment: local;
                    background-repeat: no-repeat;
                    background-size: '.$size.';
                }';
            } else {
                $style .= "#page{background-color:#ccc;}";
            }
        }

        if (empty($coursebgimage)) {
            $coursebgimage = $this->get_global_fallback('courselevelbg');
            $style .= 'body#page-course-view-levels{background:#ccc;}';
        }

        // Added overflow with important due to fix the drawer moved in section page.
        $style .= (!$this->is_sectionpage())
            ? sprintf("body#page-course-view-levels{
                background-image:url(%s);background-size: cover;background-repeat: no-repeat;}
                #page{overflow-y:visible !important;}", $coursebgimage) : '';

        if ($CFG->branch >= 404 && $sectionnum == 0 && $sectionnum !== null) {
            $style .= sprintf("#page{
                background-image:url(%s);
                background-size: cover;
                background-repeat: no-repeat;}", $coursebgimage);
        }

        return $style;
    }

    /**
     * Get global fallback image for the given filearea.
     *
     * @param string $config Filearea of module/section.
     * @return string URL of fallback image.
     */
    public function get_global_fallback($config) {
        global $CFG;
        return $this->get_course_images($config, 0, \context_system::instance());
    }

    /**
     * Get level map size class.
     *
     * @param int $sectionnum
     * @return void
     */
    public function get_levelmap_sizeclass($sectionnum) {
        $sectionoptions = (object) $this->format->get_format_options($sectionnum);
        return 'mapsize-'.$sectionoptions->levelmapsize;
    }

    /**
     * Fetch section images.
     *
     * @param int $sectionid
     * @return array $levels list of section background images.
     */
    public function get_section_images($sectionid) {
        $levelimages = \format_levels::section_levels_background_images();
        $coursecontext = context_course::instance($this->course->id);
        foreach ($levelimages as $filearea) {
            $levels[$filearea] = $this->get_course_images($filearea, $sectionid);
        }
        return $levels;
    }

    /**
     * Get list of images for the given module.
     *
     * @param int $modid
     * @return array list of module images.
     */
    public function get_module_images($modid) {
        $levelimages = \format_levels::module_levels_bgimages();
        $modcontext = \context_module::instance($modid);
        foreach ($levelimages as $filearea) {
            $levels[$filearea] = $this->get_course_images($filearea, $modid, $modcontext);
        }
        return $levels;
    }

    /**
     * Get the background image for the section, It vertify the status of the section availability
     * And uses the current state bg image.
     *
     * @param stdclass $section
     * @return string Url of current image.
     */
    public function get_section_bgimage($section) {
        $sectionimages = $this->get_section_images($section->id);
        $completion = $this->get_section_completion($section->id);
        if ($completion) {
            $image = $sectionimages['completedlevel']
                ?: ($sectionimages['availablelevel'] ?: $this->get_global_fallback('sectioncompletelevelbg'));
        }

        if (!isset($image) || empty($image)) {
            $image = $sectionimages['availablelevel'] ?: $this->get_global_fallback('sectionavailablelevelbg');
        }
        return $image;
    }

    /**
     * Get module image based on the module visibility, and completion of the module for the current users.
     *
     * @param stdclass $module
     * @param stdclass $cm
     * @return string URL of the image.
     */
    public function get_activity_image($module, $cm) {
        $moduleimages = $this->get_module_images($module->id);
        $completion = $this->get_module_completion($cm);
        if (!empty($cm->availableinfo)) {
            $image = $moduleimages['unavailable']
                ?: ($moduleimages['available'] ?: $this->get_global_fallback('modunavailablelevelbg'));
        } else if ($completion) {
            $image = $moduleimages['completed']
                ?: ($moduleimages['available'] ?: $this->get_global_fallback('modcompletelevelbg'));
        } else if (isset($module->isnext) && ($module->isnext)) {
            $image = $moduleimages['nextavailable']
                ?: ($moduleimages['available'] ?: $this->get_global_fallback('modnextavailablelevelbg'));
        }
        if (!isset($image) || empty($image)) {
            $image = $moduleimages['available'] ?: $this->get_global_fallback('modavailablelevelbg');
        }

        return $image;
    }

    /**
     * Verify the module has completion state image.
     *
     * @param cm_info $module
     * @return bool
     */
    public function is_completion_stateimage($module) {
        $moduleimages = $this->get_module_images($module->id);
        $image = $moduleimages['completed']
                ?: ($moduleimages['available'] ?: $this->get_global_fallback('modcompletelevelbg'));
        return $moduleimages['completed']
            ? true : ($moduleimages['available'] ? false : (!empty($image) ? true : false));
    }

    /**
     * Verify the module has completion state image.
     *
     * @param cm_info $module
     * @return bool
     */
    public function is_unavailable_stateimage($module) {
        $moduleimages = $this->get_module_images($module->id);
        $image = $moduleimages['unavailable']
                ?: ($moduleimages['available'] ?: $this->get_global_fallback('modunavailablelevelbg'));
        return $moduleimages['unavailable']
            ? true : ($moduleimages['available'] ? false : (!empty($image) ? true : false));
    }

    /**
     * Get image for the given filearea in the format levels component, by default it uses the course context.
     * Using the context parameter, we can get other contexts images for this format.
     *
     * @param string $filearea Filearea of the image.
     * @param int $itemid Section id or Module id.
     * @param \context_instance $context Any Context.
     * @return void
     */
    public function get_course_images($filearea, $itemid=0, $context=null) {

        $context = ($context === null) ? context_course::instance($this->course->id) : $context;
        $files = get_file_storage()->get_area_files(
            $context->id, 'format_levels', $filearea,
            $itemid, 'itemid, filepath, filename', false);
        if (empty($files) ) {
            return '';
        }
        $file = current($files);
        $fileurl = moodle_url::make_pluginfile_url(
            $file->get_contextid(),
            $file->get_component(),
            $file->get_filearea(),
            $file->get_itemid(),
            $file->get_filepath(),
            $file->get_filename(), false);
        return $fileurl->out(false);
    }

    /**
     * Find all the modules inside the given sections are completed by the logged in user.
     * If result is not true it will return the progress and current completion details of section.
     *
     * @param int $sectionid Section info data.
     * @return bool|array Result of section completion or Current progress data.
     */
    public function get_section_completion($sectionid) {
        global $DB;

        $completionstatus = true;
        $completionlist = 0;
        if ($sectionid) {
            $modinfo = get_fast_modinfo($this->course);
            $section = $DB->get_record('course_sections', ['id' => $sectionid]);
            if (isset($section)) {
                if (isset($modinfo->sections[@$section->section])) {
                    foreach ($modinfo->sections[$section->section] as $modnumber) {
                        $module = $modinfo->cms[$modnumber];
                        $completiondata = $this->completioninfo->get_data($module);
                        $isactivitycriteria = in_array($modnumber, $this->completionmodules);
                        if ($module->deletioninprogress || !$this->completioninfo->is_enabled($module) || !$isactivitycriteria) {
                            continue;
                        }
                        $completionstatus = $this->get_module_completion($module);
                        if (!$completionstatus) {
                            break;
                        }
                        $completionlist += 1;
                    }
                }
            }
        }
        return ($completionlist > 0 ) ? $completionstatus : false;
    }

    /**
     * Verify the given section has course completion enabled modules. if has it returns of the available module count.
     *
     * @param int $sectionid section id not number
     * @return int Completion enabled modules count.
     */
    public function section_has_completionmodules($sectionid) {
        global $DB;

        $completionlist = 0;
        if ($sectionid) {
            $modinfo = get_fast_modinfo($this->course);
            $section = $DB->get_record('course_sections', ['id' => $sectionid]);
            if (isset($section)) {
                if (isset($modinfo->sections[@$section->section])) {
                    foreach ($modinfo->sections[$section->section] as $modnumber) {
                        $module = $modinfo->cms[$modnumber];
                        $completiondata = $this->completioninfo->get_data($module);
                        $isactivitycriteria = in_array($modnumber, $this->completionmodules);
                        if (!$this->completioninfo->is_enabled($module) || !$isactivitycriteria) {
                            continue;
                        }
                        $completionlist += 1;
                    }
                }
            }
        }

        return $completionlist;
    }

    /**
     * Find the status of given module completion for the current user.
     *
     * @param stdclass $module Module data object.
     * @return bool Status of the module completion.
     */
    public function get_module_completion($module) {

        if ($this->completioninfo->is_enabled($module)) {
            $completiondata = $this->completioninfo->get_data($module);
            switch ($completiondata->completionstate) {
                case COMPLETION_COMPLETE:
                case COMPLETION_COMPLETE_PASS:
                    $completionstatus = true;
                break;
                default:
                    $completionstatus = false;
            }
        }
        return $completionstatus ?? false;
    }

    /**
     * Find the module completion enabled.
     *
     * @param cm_info $cm
     * @return bool
     */
    public function has_completion_enabled($cm) {
        return $this->completioninfo->is_enabled($cm);
    }


    /**
     * Get activity styles.
     *
     * @param stdclass $module
     * @return void
     */
    public function get_activity_styles($module) {
        $options = $this->get_options($module->id);

        if (isset($options->xposition)) {
            $style = 'left:'. intval($options->xposition) ?: 0;
            $style .= 'px;';
            $style .= 'top:'. intval($options->yposition) ?: 0;
            $style .= 'px;';
        }

        return $style ?? '';
    }

    /**
     * Check the current user has capabilty to view the format levels student view.
     *
     * @return bool
     */
    public static function is_student_view() {
        global $PAGE;

        $setup = optional_param('setup', 0, PARAM_BOOL);
        if (has_capability('format/levels:studentview', $PAGE->context)
            && !has_capability('format/levels:managelevels', $PAGE->context)) {
            return true;
        } else if (has_capability('format/levels:managelevels', $PAGE->context) && $setup) {
            return true;
        }
        return false;
    }
}
