<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Contains the main course format out class.
 *
 * @package   format_levels
 * @copyright 2022 bdecent gmbh <https://bdecent.de>
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

namespace format_levels\output\courseformat;

use core\output\named_templatable;
use core_courseformat\base as course_format;
use course_modinfo;
use renderable;

use core_courseformat\output\local\content as content_base;

/**
 * Base class to render a course format.
 *
 * @package   format_levels
 * @copyright 2022 bdecent gmbh <https://bdecent.de>
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class content extends content_base {

    /**
     * @var bool Topic format has add section after each topic.
     *
     * The responsible for the buttons is core_courseformat\output\local\content\section.
     */
    protected $hasaddsection = true;

    /**
     * Get course format content.
     *
     * @return format_levels
     */
    public function get_format() {
        return $this->format;
    }

    /**
     * Export this data so it can be used as the context for a mustache template (core/inplace_editable).
     *
     * @param renderer_base $output typically, the renderer that's calling this function
     * @return stdClass data context for a mustache template
     */
    public function export_for_template(\renderer_base $output) {
        global $PAGE, $CFG;
        $format = $this->format;

        // Most formats uses section 0 as a separate section so we remove from the list.
        $sections = $this->export_sections($output);

        $data = (object)[
            'title' => $format->page_title(), // This method should be in the course_format class.
            'sections' => $sections,
            'format' => $format->get_format(),
            'sectionreturn' => null,
        ];

        // The single section format has extra navigation.
        if (method_exists($format, 'get_sectionnum')) {
            $singlesection = $this->format->get_sectionnum();
        } else {
            $singlesection = $this->format->get_section_number();
        }

        if ($singlesection) {
            if (!$PAGE->theme->usescourseindex) {
                $sectionnavigation = new $this->sectionnavigationclass($format, $singlesection);
                $data->sectionnavigation = $sectionnavigation->export_for_template($output);

                $sectionselector = new $this->sectionselectorclass($format, $sectionnavigation);
                $data->sectionselector = $sectionselector->export_for_template($output);
            }
            $data->hasnavigation = true;
            $data->singlesection = array_shift($data->sections);
            $data->sectionreturn = $singlesection;
            $data->title = $data->singlesection->singleheader->title;
            $data->showloader = $data->singlesection->showloader;
        } else {

            $initialsection = '';
            if (!empty($sections)) {
                $initialsection = array_shift($sections);
            }

            if ($CFG->branch >= 404 && $singlesection == 0 && $singlesection !== null) {
                $data->singlesection = array_shift($data->sections);
            }

            if ($CFG->branch >= 404) {
                // We want to remove the general section if it is empty on the course page.
                $modinfo = get_fast_modinfo($this->format->get_course());
                $section = $modinfo->get_sections();
                if (!isset($section[0])) {
                    // The general section is empty to find the navigation node for it we need to get its ID.
                    $section = $modinfo->get_section_info(0);
                    $initialsection = '';
                }
            }

            $data->initialsection = $initialsection;
            $data->sections = $sections;
            $data->showloader = (!AJAX_SCRIPT) ? true : false;
        }

        if ($this->hasaddsection) {
            $addsection = new $this->addsectionclass($format);
            $data->numsections = $addsection->export_for_template($output);
        }

        if ($CFG->branch >= 402) {
            if (method_exists($format, 'show_editor') && $format->show_editor()) {
                $bulkedittools = new $this->bulkedittoolsclass($format);
                $data->bulkedittools = $bulkedittools->export_for_template($output);
            }
        }

        $data->ajax = (AJAX_SCRIPT) ? true : false;

        $data->setup = (optional_param('setup', 0, PARAM_BOOL)
            && has_capability('format/levels:managelevels', $PAGE->context)) ? true : false;

        if ($CFG->branch >= 404) {

            $course = $this->format->get_course();
            $context = \context_course::instance($course->id);
            $displaysection = $this->format->get_sectionid();

            $ispopupactivities = isset($course->popupactivities) && $course->popupactivities;
            if ($ispopupactivities && !$PAGE->user_is_editing() && !empty($displaysection) && !$data->setup) {
                // Include popups.
                $PAGE->requires->js_call_amd('format_popups/popups', 'init', [
                    $context->id, $course->id, $displaysection,
                ]);
            }
        }

        return $data;
    }

    /**
     * Export sections array data.
     *
     * @param renderer_base $output typically, the renderer that's calling this function
     * @return array data context for a mustache template
     */
    protected function export_sections(\renderer_base $output): array {

        $format = $this->format;
        $course = $format->get_course();
        $modinfo = $this->format->get_modinfo();

        // Generate section list.
        $sections = [];
        $stealthsections = [];
        $numsections = $format->get_last_section_number();
        // Format levels data.
        $formatoptions = new \format_levels\options($this->format->get_course());
        foreach ($this->get_sections_to_display($modinfo) as $sectionnum => $thissection) {
            // The course/view.php check the section existence but the output can be called
            // from other parts so we need to check it.
            if (!$thissection) {
                throw new moodle_exception('unknowncoursesection', 'error',
                    course_get_url($course), format_string($course->fullname));
            }

            $section = new $this->sectionclass($format, $thissection);

            if ($sectionnum > $numsections) {
                // Activities inside this section are 'orphaned', this section will be printed as 'stealth' below.
                if (!empty($modinfo->sections[$sectionnum])) {
                    $stealthsections[] = $section->export_for_template($output);
                }
                continue;
            }

            if (!$format->is_section_visible($thissection)) {
                continue;
            }

            $sections[] = $section->export_for_template($output);
        }
        if (!empty($stealthsections)) {
            $sections = array_merge($sections, $stealthsections);
        }
        // Next section.
        foreach ($sections as $sectionid => $section) {
            if ($section->num == 0) {
                continue;
            }
            $completionlist = $formatoptions->section_has_completionmodules($section->id);
            $sectioninfo = $modinfo->get_section_info($section->num);
            if (!isset($currentsection)
                && $sectioninfo->uservisible && ($completionlist > 0) && !$formatoptions->get_section_completion($section->id)) {
                $isnextsection = true;
                $sections[$sectionid]->isnext = true;
                break;
            }
        }
        // Not started any section then make first section as next.
        if (!isset($isnextsection)) {
            $nextsection = current(array_keys($sections));
            $sections[$nextsection]->isnext = true;
        }
        return $sections;
    }

    /**
     * Return an array of sections to display.
     *
     * This method is used to differentiate between display a specific section
     * or a list of them.
     *
     * @param course_modinfo $modinfo the current course modinfo object
     * @return section_info[] an array of section_info to display
     */
    private function get_sections_to_display(course_modinfo $modinfo): array {
        $format = $this->format;

        if (method_exists($format, 'get_sectionnum')) {
            $singlesection = $format->get_sectionnum();
        } else {
            $singlesection = $format->get_section_number();
        }

        if ($singlesection) {
            return [
                $modinfo->get_section_info($singlesection),
            ];
        }

        return $modinfo->get_section_info_all();
    }

    /**
     * Returns the template name for render the course content.
     * If not added any other functions then remove this and use render_content funcotin.
     * @param renderer_base $renderer
     */
    public function get_template_name(\renderer_base $renderer): string {
        global $PAGE;
        return \format_levels\options::is_student_view() ? 'format_levels/courseformat/content' : 'core_courseformat/local/content';
    }
}
