<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

namespace local_kickstart_pro;

use moodle_url;
use context_course;
use format_kickstart\output\course_template_list;
use Exception;


/**
 * Hook callbacks for tool_abconfig.
 *
 * @package   tool_abconfig
 * @author    Benjamin Walker (benjaminwalker@catalyst-au.net)
 * @copyright 2024 Catalyst IT
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class hook_callbacks {

    /**
     * Runs before HTTP headers.
     *
     * @param \core\hook\output\before_http_headers $hook
     */
    public static function before_http_headers(\core\hook\output\before_http_headers $hook): void {
        global $COURSE, $USER, $PAGE, $CFG;
        require_once($CFG->dirroot . "/course/format/kickstart/lib.php");
        try {
            // First rule out conditions where a redirect should never happen.
            // Also skip is $CFG->kickstart_pro is false.
            if (AJAX_SCRIPT || is_siteadmin() || !format_kickstart_has_pro()) {
                return;
            }

            if ($PAGE->url->compare(new moodle_url('/course/view.php'), URL_MATCH_BASE) &&
                get_config('format_kickstart', 'automatictemplate')) {
                if (course_get_format($COURSE)->get_format() == 'kickstart' &&
                    has_capability('format/kickstart:import_from_template', context_course::instance($COURSE->id))) {

                    $list = new course_template_list($COURSE, $USER->id);
                    // If automatic template is enabled, and only 1 template is available, import it now.
                    if (count($list->get_templates()) === 1) {
                        $template = $list->get_templates()[0];
                        redirect(new moodle_url('/course/format/kickstart/import.php', [
                            'template_id' => $template->id,
                            'course_id' => $COURSE->id]), get_string('automatictemplate_help', 'format_kickstart'), 5,
                            \core\output\notification::NOTIFY_INFO);
                    }
                }
            } else if ($PAGE->url->compare(new moodle_url('/course/edit.php'), URL_MATCH_BASE) &&
                get_config('format_kickstart', 'coursecreatorredirect')) {
                if (!$PAGE->url->get_param('id')) { // Skip course edit page. Only redirect for new courses.
                    if (has_any_capability(['format/kickstart:import_from_template',
                        'local/kickstart_pro:import_other_courses'], $PAGE->context)) {
                        redirect(new moodle_url('/local/kickstart_pro/createcourse.php', $PAGE->url->params()));
                    }
                }
            }
        } catch (Exception $e) {
            debugging($e->getMessage()); // Prevent any issues from breaking entire site.
        }
    }
}