<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Display the default course view
 *
 * @package    local_kickstart_pro
 * @copyright  2019 bdecent gmbh <https://bdecent.de>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

namespace local_kickstart_pro\output;

use renderer_base;

/**
 * Display the default course view (Not display the import template).
 */
class default_view implements \templatable, \renderable {
    /**
     * @var string $course - course object
     */
    private $course;

    /**
     * Constructor
     * @param \stdclass $course
     */
    public function __construct($course) {
        $this->course = course_get_format($course)->get_course();
    }

    /**
     * Export for template.
     *
     * @param renderer_base $output The renderer.
     * @return array
     */
    public function export_for_template(renderer_base $output) {
        global $DB, $USER, $CFG, $PAGE;

        require_once("$CFG->dirroot/course/renderer.php");

        $roles = array_merge(get_archetype_roles('teacher'), get_archetype_roles('editingteacher'));
        $users = [];

        $context = \context_course::instance($this->course->id);
        $course = new \core_course_list_element($this->course);
        $courseimage = null;

        foreach ($course->get_course_overviewfiles() as $file) {
            $courseimage = file_encode_url("$CFG->wwwroot/pluginfile.php",
                '/' . $file->get_contextid() . '/' . $file->get_component() . '/' .
                $file->get_filearea() . $file->get_filepath() . $file->get_filename(), false);
            break;
        }

        foreach ($roles as $role) {
            foreach (get_users_from_role_on_context($role, $context) as &$assignment) {
                if (!$user = $DB->get_record('user', ['id' => $assignment->userid])) {
                    continue;
                }
                $user->fullname = fullname($user);
                $user->rolename = role_get_name($role, $context);
                $user->description = file_rewrite_pluginfile_urls($user->description, 'pluginfile.php', $context->id, 'user',
                    'profile', $user->id);
                $user->description = format_text($user->description, $user->descriptionformat);

                $picture = new \user_picture($user);
                $picture->size = 300;
                $user->profileimageurl = $picture->get_url($PAGE)->out(false);

                if (can_view_user_details_cap($user, $this->course)) {
                    $user->profileurl = new \moodle_url('/user/view.php', ['id' => $user->id, 'course' => $this->course->id]);
                }
                if (!empty($CFG->messaging) && $USER->id != $user->id && has_capability('moodle/site:sendmessage', $context)) {
                    $user->messageurl = new \moodle_url('/message/index.php', ['id' => $user->id]);
                }
                $users[] = $user;
            }
        }

        $user = null;
        if (count($users) == 1) {
            $user = $users[0];
            $users = [];
        }

        $chelper = new \coursecat_helper();

        return [
            'user' => $user,
            'users' => $users,
            'course' => $this->course,
            'coursesummary' => $chelper->get_course_formatted_summary($course),
            'courseimage' => $courseimage,
            'userinstructions' => format_text($this->course->userinstructions['text'], $this->course->userinstructions['format'])
        ];
    }
}
