<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Create a simple course with the Kickstart format as default.
 *
 * @package    local_kickstart_pro
 * @copyright  2020 bdecent gmbh <https://bdecent.de>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

use core\notification;
use local_kickstart_pro\form\course_form;

require(__DIR__.'/../../config.php');
require_once("$CFG->libdir/adminlib.php");
require_once("$CFG->dirroot/course/lib.php");
require_once(__DIR__.'/lib.php');

global $USER, $PAGE, $OUTPUT;

$categoryid = optional_param('category', null, PARAM_INT);

if (is_siteadmin()) {
    admin_externalpage_setup('kickstartcreatecourse');
} else {
    if ($categoryid) {
        $PAGE->set_context(context_coursecat::instance($categoryid));
    } else {
        $PAGE->set_context(context_system::instance());
    }
    $PAGE->set_url(new moodle_url('/local/kickstart_pro/createcourse.php'));
    require_login();

    if (!data_submitted() ||  $categoryid !== 0) {
        if (!has_any_capability(['format/kickstart:import_from_template',
            'local/kickstart_pro:import_other_courses'], $PAGE->context)) {
            throw new required_capability_exception($PAGE->context, 'format/kickstart:import_from_template', 'nopermissions', '');
        }
    }
    $PAGE->navbar->add(get_string('courses'), new moodle_url('/course'));
    $PAGE->navbar->add(get_string('createcoursefromtemplate', 'format_kickstart'));
}
$PAGE->set_title(get_string('createcoursefromtemplate', 'format_kickstart'));
$PAGE->set_heading(get_string('createcoursefromtemplate', 'format_kickstart'));
$editicon = local_kickstart_pro_get_editicon();
$PAGE->requires->js_call_amd('local_kickstart_pro/kickstartcourseformat', 'init', array($editicon));
$form = new course_form(null, ['categoryid' => $categoryid]);

if ($data = $form->get_data()) {
    $data->format = 'kickstart';
    if (empty($data->shortname)) {
        $data->shortname = $data->fullname;
    }
    try {
        $course = create_course($data);

        // Get the context of the newly created course.
        $context = context_course::instance($course->id, MUST_EXIST);

        if (!empty($CFG->creatornewroleid) && !is_viewing($context, null, 'moodle/role:assign')
            && !is_enrolled($context, null, 'moodle/role:assign')) {
            // Deal with course creators - enrol them internally with default role.
            // Note: This does not respect capabilities, the creator will be assigned the default role.
            // This is an expected behaviour. See MDL-66683 for further details.
            enrol_try_internal_enrol($course->id, $USER->id, $CFG->creatornewroleid);
        }
        redirect(new moodle_url('/course/view.php', ['id' => $course->id]));
    } catch (moodle_exception $e) {
        notification::error($e->getMessage());
    }
}

echo $OUTPUT->header();
$form->display();
echo $OUTPUT->footer();
