<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Common functions and callbacks.
 *
 * @package    local_kickstart_pro
 * @copyright  2020 bdecent gmbh <https://bdecent.de>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

defined('MOODLE_INTERNAL') || die;

global $CFG;

require_once("$CFG->dirroot/course/format/kickstart/lib.php");

use format_kickstart\output\course_template_list;

/**
 * Give plugins an opportunity touch things before the http headers are sent
 * such as adding additional headers. The return value is ignored.
 */
function local_kickstart_pro_before_http_headers() {
    global $COURSE, $USER, $PAGE;
    try {

        // First rule out conditions where a redirect should never happen.
        // Also skip is $CFG->kickstart_pro is false.
        if (AJAX_SCRIPT || is_siteadmin() || !format_kickstart_has_pro()) {
            return;
        }

        if ($PAGE->url->compare(new moodle_url('/course/view.php'), URL_MATCH_BASE) &&
            get_config('format_kickstart', 'automatictemplate')) {
            if (course_get_format($COURSE)->get_format() == 'kickstart' &&
                has_capability('format/kickstart:import_from_template', context_course::instance($COURSE->id))) {

                $list = new course_template_list($COURSE, $USER->id);
                // If automatic template is enabled, and only 1 template is available, import it now.
                if (count($list->get_templates()) === 1) {
                    $template = $list->get_templates()[0];
                    redirect(new moodle_url('/course/format/kickstart/import.php', [
                        'template_id' => $template->id,
                        'course_id' => $COURSE->id]), get_string('automatictemplate_help', 'format_kickstart'), 5,
                        \core\output\notification::NOTIFY_INFO);
                }
            }
        } else if ($PAGE->url->compare(new moodle_url('/course/edit.php'), URL_MATCH_BASE) &&
            get_config('format_kickstart', 'coursecreatorredirect')) {
            if (!$PAGE->url->get_param('id')) { // Skip course edit page. Only redirect for new courses.
                if (has_any_capability(['format/kickstart:import_from_template',
                    'local/kickstart_pro:import_other_courses'], $PAGE->context)) {
                    redirect(new moodle_url('/local/kickstart_pro/createcourse.php', $PAGE->url->params()));
                }
            }
        }
    } catch (Exception $e) {
        debugging($e->getMessage()); // Prevent any issues from breaking entire site.
    }
}
/**
 * Get a course shortname edit icon.
 * @return string edit icon html.
 */
function local_kickstart_pro_get_editicon() {

    global $OUTPUT;
    $editicon = '';
    $editicon .= html_writer::start_tag('div', array('class' => "edit-shortname-parent-kickstart"));
    $editicon .= $OUTPUT->pix_icon('i/edit', get_string('edit'), 'moodle', array('class' => "shortname-editform"));
    $editicon .= html_writer::end_div("div");
    return $editicon;
}




/**
 * Get a template images.
 * @param int $templateid
 * @return array $images
 */
function local_kickstart_pro_get_template_backimages($templateid) {
    $images = [];
    $files = get_file_storage()->get_area_files(
        context_system::instance()->id, 'local_kickstart_pro', 'templatebackimg',
        $templateid, 'itemid, filepath, filename', false);
    if (empty($files)) {
        return [];
    }
    $i = 0;
    foreach ($files as $file) {
        $fileurl = moodle_url::make_pluginfile_url(
            $file->get_contextid(),
            $file->get_component(),
            $file->get_filearea(),
            $file->get_itemid(),
            $file->get_filepath(),
            $file->get_filename(), false);
        $list['image'] = $fileurl;
        $list['active'] = ($i == 0) ? true : false;
        $list['pos'] = $i;
        $images[] = $list;
        $i++;
    }
    return $images;
}

/**
 * Serves file from.
 *
 * @param mixed $course course or id of the course
 * @param mixed $cm course module or id of the course module
 * @param context $context Context used in the file.
 * @param string $filearea Filearea the file stored
 * @param array $args Arguments
 * @param bool $forcedownload Force download the file instead of display.
 * @param array $options additional options affecting the file serving
 * @return bool false if file not found, does not return if found - just send the file
 */
function local_kickstart_pro_pluginfile($course, $cm, $context, $filearea, $args, $forcedownload, array $options = array()) {
    require_login();
    if ($context->contextlevel != CONTEXT_SYSTEM && $filearea != 'templatebackimg') {
        return false;
    }

    $fs = get_file_storage();
    $file = $fs->get_file($context->id, 'local_kickstart_pro', $filearea, $args[0], '/', $args[1]);
    if (!$file) {
        return false;
    }
    send_stored_file($file, 0, 0, 0, $options);
}
