<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Kickstart pro.
 *
 * @package    local_kickstart_pro
 * @copyright  2021 bdecent gmbh <https://bdecent.de>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
namespace local_kickstart_pro;

/**
 * Test course importing.
 *
 * @group      local_kickstart_pro
 * @group      local_kickstart_pro_test
 * @copyright  2021 bdecent gmbh <https://bdecent.de>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class local_kickstart_pro_test extends \advanced_testcase {

    /**
     * Set the admin user as User.
     *
     * @return void
     */
    public function setup(): void {
        global $DB, $CFG;
        require_once($CFG->dirroot. '/course/format/kickstart/lib.php');
        $this->resetAfterTest(true);
        $this->generator = $this->getDataGenerator();
        $this->user = $this->generator->create_user();
        $this->setUser($this->user);
        $this->category = $this->generator->create_category();
        $this->coursecreator = $DB->get_record('role', array('shortname' => 'coursecreator'));
        $this->course = $this->generator->create_course([
            'category' => $this->category->id
        ]);
        $this->categorycontext = \context_coursecat::instance($this->category->id);
        $this->generator->role_assign($this->coursecreator->id, $this->user->id, $this->categorycontext->id);
        $this->template = $this->local_kickstart_pro_template_info($this->category->id);
        $this->context = \context_system::instance();
        $this->coursecontext = \context_course::instance($this->course->id);
    }

    /**
     * Automatically create the template.
     * @param object $template template info
     * @param int $sort sort position
     * @param object $context page context
     * @param string $component
     * @return void
     */
    public function create_kickstart_template($template, $sort, $context, $component) {
        global $CFG;
        $templatesconfig = isset($CFG->kickstart_templates) ? explode(",", $CFG->kickstart_templates) : [];
        $templateid = format_kickstart_create_template($template, $sort, $context, $component);
        if (!array_search($templateid, $templatesconfig)) {
            array_push($templatesconfig, $templateid);
        }
        set_config('kickstart_templates', implode(',', $templatesconfig));
    }

    /**
     * Case to check Categories based list of the templates.
     * @covers ::import_access_templates_category()
     */
    public function test_import_access_templates_category() {
        global $DB;
        $template = $this->template;
        $template->restrictcategory = 1;
        $template->categoryids = array($this->category->id);
        $this->create_kickstart_template($template, 1, $this->context, 'local_kickstart_pro');
        $count = $DB->count_records('format_kickstart_template');
        $this->assertEquals(6, $count);
        $templatelist = new \format_kickstart\output\course_template_list($this->course, $this->user->id);
        $accesstemplates = $templatelist->get_templates();

        $checkcond = array_search($this->template->title, array_column($accesstemplates, 'title'));
        $this->assertTrue(is_numeric($checkcond));
        $wrongcheckcond = array_search('demotesttemplate', array_column($accesstemplates, 'title'));
        $this->assertNotTrue(is_numeric($wrongcheckcond));
        $otheruser = $this->generator->create_user();
        $this->setUser($otheruser);
        $othercategory = $this->generator->create_category();
        $othercourse = $this->generator->create_course([
            'category' => $othercategory->id
        ]);

        $othercategorycontext = \context_coursecat::instance($othercategory->id);
        $this->generator->role_assign($this->coursecreator->id, $otheruser->id, $othercategorycontext->id);
        $othertemplatelist = new \format_kickstart\output\course_template_list($othercourse, $otheruser->id);
        $otheraccesstemplates = $othertemplatelist->get_templates();
        $othercheckcond = array_search($template->title, array_column($otheraccesstemplates, 'title'));
        $this->assertNotTrue(is_numeric($othercheckcond));
    }

    /**
     * Case to check roles based list of the templates.
     * @covers ::import_access_templates_role()
     */
    public function test_import_access_templates_role() {
        global $DB;

        $this->resetAfterTest(true);
        $template = $this->template;
        $template->restrictrole = 1;
        $template->roleids = array($this->coursecreator->id);
        $this->create_kickstart_template($template, 1, $this->context, 'local_kickstart_pro');
        $templatelist = new \format_kickstart\output\course_template_list($this->course, $this->user->id);
        $accesstemplates = $templatelist->get_templates();

        $checkcond = array_search($this->template->title, array_column($accesstemplates, 'title'));
        $this->assertTrue(is_numeric($checkcond));

        $student = $this->generator->create_user();
        $this->setUser($student);
        $studentrole = $DB->get_record('role', array('shortname' => 'student'));
        $this->generator->role_assign($studentrole->id, $student->id, $this->coursecontext->id);
        $wrongtemplatelist = new \format_kickstart\output\course_template_list($this->course, $student->id);
        $wrongaccesstemplates = $wrongtemplatelist->get_templates();
        $wrongcheckcond = array_search($template->title, array_column($wrongaccesstemplates, 'title'));
        $this->assertNotTrue(is_numeric($wrongcheckcond));
    }

    /**
     * Case to check cohorts based list of the templates.
     * @covers ::import_access_templates_cohorts()
     */
    public function test_import_access_templates_cohorts() {
        $this->resetAfterTest(true);

        $template = $this->template;
        $cohort = $this->generator->create_cohort();
        $template->restrictcohort = 1;
        $template->cohortids = array($cohort->id);
        cohort_add_member($cohort->id, $this->user->id);
        $this->create_kickstart_template($template, 1, $this->context, 'local_kickstart_pro');
        $templatelist = new \format_kickstart\output\course_template_list($this->course, $this->user->id);
        $accesstemplates = $templatelist->get_templates();

        $checkcohortcond = array_search($this->template->title, array_column($accesstemplates, 'title'));
        $this->assertTrue(is_numeric($checkcohortcond));

        $this->resetAfterTest(true);
        $notassignuser = $this->generator->create_user();
        $this->setUser($notassignuser);
        $othercohort = $this->generator->create_cohort();
        cohort_add_member($othercohort->id, $notassignuser->id);
        $this->generator->role_assign($this->coursecreator->id, $notassignuser->id, $this->coursecontext->id);
        $wrongtemplatelist = new \format_kickstart\output\course_template_list($this->course, $notassignuser->id);
        $wrongaccesstemplates = $wrongtemplatelist->get_templates();
        $wrongcheckcond = array_search($template->title, array_column($wrongaccesstemplates, 'title'));
        $this->assertNotTrue(is_numeric($wrongcheckcond));
    }

    /**
     * Get template info.
     */
    public function local_kickstart_pro_template_info() {
        $template = array(
            'id' => 0,
            'title' => 'demoimporttemplate',
            'description' => '<p dir="ltr" style="text-align: left;">test content of the block content</p>',
            'tags' => [],
            'backupfile' => 'course-186-tiles_template.mbz',
            'preview_url' => '',
            'restrictcohort' => 0,
            'cohortids' => [],
            'restrictcategory' => 0,
            'categoryids' => [],
            'includesubcategories' => 0,
            'restrictrole' => 0,
            'roleids' => [],
            'descriptionformat' => 1,
        );
        return (object) $template;
    }

    /**
     * Get the template background images.
     * @covers ::local_kickstart_pro_get_template_backimages()
     */
    public function test_local_kickstart_pro_get_template_backimages() {
        global $DB;
        $template = $this->local_kickstart_pro_template_info();
        $template->templatebackimg = 'template1bg.jpg';
        $prevcount = $DB->count_records('format_kickstart_template');
        $context = \context_system::instance();
        $templateid = format_kickstart_create_template($template, $prevcount, $context, 'local_kickstart_pro');
        $val = local_kickstart_pro_get_template_backimages($templateid);
        $res = current($val);
        $this->assertEquals(1, $res['active']);
        $this->assertTrue(!empty($res['image']));
    }
}
