<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Local Reset - Assign module attempt reset structures defined.
 *
 * @package    local_reset
 * @copyright  2024 LMSACE DEV
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

namespace local_reset\plugins;

/**
 * Process the reset of user attempts from the assign module.
 */
class mod_assign {


    /**
     * Include the assign local lib and intiate the reset process of assign attempts.
     *
     * @param stdclass $data
     * @return void
     */
    public static function reset_user_attempt($data) {
        global $CFG;

        if (file_exists($CFG->dirroot.'/mod/assign/locallib.php')) {
            require_once($CFG->dirroot . '/mod/assign/locallib.php');

            return (new self())->reset_assign((object) $data);
        }

        return false;
    }

    /**
     * Reset the assign instance for the given user in all assignments form the course.
     *
     * @param stdclass $data
     * @return void
     */
    public function reset_assign($data) {
        global $CFG, $DB;

        require_once($CFG->dirroot.'/mod/assign/lib.php');

        $status = false;

        try {

            $params = ['courseid' => $data->courseid];
            $sql = "SELECT a.id FROM {assign} a WHERE a.course=:courseid";
            $course = $DB->get_record('course', ['id' => $data->courseid], '*', MUST_EXIST);
            if ($assigns = $DB->get_records_sql($sql, $params)) {
                foreach ($assigns as $assign) {
                    $cm = get_coursemodule_from_instance('assign',
                                                        $assign->id,
                                                        $data->courseid,
                                                        false,
                                                        MUST_EXIST);
                    $context = \context_module::instance($cm->id);
                    $assignment = new \assign($context, $cm, $course);
                    $status = $this->assign_reset_userdata($data, $assignment, $context);
                }
            }

        } catch (moodle_exception $e) {

            $transaction->rollback($e);
        }

        return $status;
    }

    /**
     * Reset the user attempt for the given user in all assign instance form the course.
     *
     * @param stdclass $data
     * @param stdclass $assign
     * @param mixed $context
     * @return bool
     */
    protected function assign_reset_userdata($data, $assign, $context) {
        global $CFG, $DB;

        require_once($CFG->dirroot.'/mod/assign/lib.php');

        $status = false;

        try {

            $transaction = $DB->start_delegated_transaction();

            $fs = get_file_storage();

            $submissionplugins = $this->load_plugins('assignsubmission', $assign);
            $feedbackplugins = $this->load_plugins('assignfeedback', $assign);

            // Delete files associated with this assignment.
            foreach ($submissionplugins as $plugin) {
                $fileareas = array();
                $plugincomponent = $plugin->get_subtype() . '_' . $plugin->get_type();

                $usersubmission = $assign->get_user_submission($data->userid, true);
                $user = $DB->get_record('user', ['id' => $data->userid]);
                $fileareas = $plugin->get_files($usersubmission, $user);
                foreach ($fileareas as $filearea => $notused) {
                    $fs->delete_area_files($context->id, $plugincomponent, $filearea);
                }

                if (!$plugin->delete_instance()) {
                    $status = true;
                }
            }

            foreach ($feedbackplugins as $plugin) {
                $fileareas = array();
                $plugincomponent = $plugin->get_subtype() . '_' . $plugin->get_type();
                $usersubmission = $assign->get_user_submission($data->userid, true);
                $user = $DB->get_record('user', ['id' => $data->userid]);
                $fileareas = $plugin->get_files($usersubmission, $user);
                foreach ($fileareas as $filearea => $notused) {
                    $fs->delete_area_files($context->id, $plugincomponent, $filearea);
                }

                if (!$plugin->delete_instance()) {
                    $status = true;
                }
            }

            $DB->delete_records_select('assign_submission',
                "userid = ? and assignment IN (SELECT id FROM {assign} WHERE course = ?)", [$data->userid, $data->courseid]);

            $DB->delete_records_select('assign_user_flags',
                "userid = ? and assignment IN (SELECT id FROM {assign} WHERE course = ?)", [$data->userid, $data->courseid]);

            // Remove assign grades.
            $DB->delete_records_select('assign_grades',
                "userid = ? and assignment IN (SELECT id FROM {assign} WHERE course = ?)", [$data->userid, $data->courseid]);

            // Remove all grades from gradebook.
            assign_reset_gradebook($data->courseid);

            // Remove user overrides.
            $DB->delete_records_select('assign_overrides',
                    'userid = ? and assignid IN (SELECT id FROM {assign} WHERE course = ?) AND userid IS NOT NULL',
                    [$data->userid, $data->courseid],
                );

            // Remove group overrides.
            $DB->delete_records_select('assign_overrides',
                    'userid = ? and assignid IN (SELECT id FROM {assign} WHERE course = ?) AND groupid IS NOT NULL',
                    [$data->userid, $data->courseid],
                );

            $transaction->allow_commit();

            \cache::make('mod_assign', 'overrides')->purge();

            $status = true;

        } catch (moodle_exception $e) {

            $transaction->rollback($e);
        }
        return $status;

    }

    /**
     * Load the plugins from the sub folders under subtype.
     *
     * @param string $subtype - either submission or feedback
     * @return array - The sorted list of plugins
     */
    public function load_plugins($subtype, $assign) {
        global $CFG;
        $result = array();

        $names = \core_component::get_plugin_list($subtype);
        foreach ($names as $name => $path) {
            if (file_exists($path . '/locallib.php')) {
                require_once($path . '/locallib.php');

                $shortsubtype = substr($subtype, strlen('assign'));
                $pluginclass = 'assign_' . $shortsubtype . '_' . $name;

                $plugin = new $pluginclass($assign, $name);

                if ($plugin instanceof \assign_plugin) {
                    $idx = $plugin->get_sort_order();
                    while (array_key_exists($idx, $result)) {
                        $idx +=1;
                    }
                    $result[$idx] = $plugin;
                }
            }
        }
        ksort($result);
        return $result;
    }

}