<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Local Reset - User lesson attempts reset structures defined.
 *
 * @package    local_reset
 * @copyright  2024 LMSACE DEV
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

namespace local_reset\plugins;

use moodle_exception;

/**
 * Process the reset of lesson attempts from the lesson module.
 */
class mod_lesson {

    /**
     * Include the lesson lib and intiate the reset process of lesson attempts.
     *
     * @param stdclass $data
     * @return void
     */
    public static function reset_user_attempt($data) {
        global $CFG;

        if (file_exists($CFG->dirroot.'/mod/lesson/lib.php')) {
            require_once($CFG->dirroot.'/mod/lesson/lib.php');

            return (new self())->reset_lesson((object) $data);
        }

        return false;
    }

    /**
     * Reset the lesson attempts for the given user in all lesson instance form the course.
     *
     * @param stdclass $data
     * @return void
     */
    protected function reset_lesson($data) {
        global $CFG, $DB;

        $status = false;

        try {

            $transaction = $DB->start_delegated_transaction();

            $lessonssql = "SELECT l.id FROM {lesson} l WHERE l.course=:course";
            $lessons = $DB->get_records_sql($lessonssql, ["course" => $data->courseid]);

            // Get rid of attempts files.
            $fs = get_file_storage();
            if ($lessons) {
                foreach ($lessons as $lessonid => $unused) {
                    if (!$cm = get_coursemodule_from_instance('lesson', $lessonid)) {
                        continue;
                    }
                    $context = \context_module::instance($cm->id);
                    $fs->delete_area_files($context->id, 'mod_lesson', 'essay_responses');
                    $fs->delete_area_files($context->id, 'mod_lesson', 'essay_answers');
                }
            }

            $DB->delete_records_select('lesson_timer',
                "userid = ? and lessonid IN (SELECT id FROM {lesson} WHERE course = ?)", [$data->userid, $data->courseid]);
            // Removes lesson grades.
            $DB->delete_records_select('lesson_grades',
                "userid = ? and lessonid IN (SELECT id FROM {lesson} WHERE course = ?)", [$data->userid, $data->courseid]);
            // Removes lesson attempts.
            $DB->delete_records_select('lesson_attempts',
                "userid = ? and lessonid IN (SELECT id FROM {lesson} WHERE course = ?)", [$data->userid, $data->courseid]);
            $DB->delete_records_select('lesson_branch',
                "userid = ? and lessonid IN (SELECT id FROM {lesson} WHERE course = ?)", [$data->userid, $data->courseid]);

            lesson_reset_gradebook($data->courseid);

            // Remove user overrides.
            $DB->delete_records_select('lesson_overrides',
                'userid = ? and lessonid IN (SELECT id FROM {lesson} WHERE course = ?) AND userid IS NOT NULL', [$data->userid, $data->courseid]);

            // Remove group overrides.
            $DB->delete_records_select('lesson_overrides',
                'userid = ? and lessonid IN (SELECT id FROM {lesson} WHERE course = ?) AND groupid IS NOT NULL', [$data->userid, $data->courseid]);

            $transaction->allow_commit();

            \cache::make('mod_lesson', 'overrides')->purge();

            $status = true;

        } catch (moodle_exception $e) {

            $transaction->rollback($e);
        }

        return $status;
    }
}
