<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Local Reset - Quiz module attempt reset structures defined.
 *
 * @package    local_reset
 * @copyright  2024 LMSACE DEV
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

namespace local_reset\plugins;

use moodle_exception;

/**
 * Process the reset of user attempts from the quiz module.
 */
class mod_quiz {

    /**
     * Include the quiz lib and intiate the reset process of quiz attempts.
     *
     * @param stdclass $data
     * @return void
     */
    public static function reset_user_attempt($data) {
        global $CFG;

        if (file_exists($CFG->dirroot.'/mod/quiz/lib.php')) {
            require_once($CFG->dirroot.'/mod/quiz/lib.php');

            return (new self())->reset_quiz((object) $data);
        }

        return false;
    }

    /**
     * Reset the user attempt for the given user in all quiz instance form the course.
     *
     * @param stdclass $data
     * @return void
     */
    protected function reset_quiz($data) {
        global $CFG, $DB;

        require_once($CFG->libdir . '/questionlib.php');

        $status = false;

        try {
            $transaction = $DB->start_delegated_transaction();

            // Delete attempts.
            \question_engine::delete_questions_usage_by_activities(new \qubaid_join(
                    '{quiz_attempts} quiza JOIN {quiz} quiz ON quiza.quiz = quiz.id',
                    'quiza.uniqueid', 'quiz.course = :quizcourseid and quiza.userid = :quizuserid',
                    ['quizcourseid' => $data->courseid, 'quizuserid' => $data->userid]));

            $DB->delete_records_select('quiz_attempts',
                    'userid = ? and quiz IN (SELECT id FROM {quiz} WHERE course = ?)', [$data->userid, $data->courseid]);

            // Remove all grades from gradebook.
            $DB->delete_records_select('quiz_grades',
                    'userid = ? and quiz IN (SELECT id FROM {quiz} WHERE course = ?)', [$data->userid, $data->courseid]);

            // Reset the gradebook of the quiz in the course.
            quiz_reset_gradebook($data->courseid);

            // Remove user overrides.
            $DB->delete_records_select('quiz_overrides',
                    'userid = ? and quiz IN (SELECT id FROM {quiz} WHERE course = ?) AND userid IS NOT NULL', [$data->userid, $data->courseid]);

            $transaction->allow_commit();

            \cache::make('mod_quiz', 'overrides')->purge();

            $status = true;

        } catch (moodle_exception $e) {

            $transaction->rollback($e);
        }

        return $status;
    }
}
