<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Local Reset - Process the reset of user attempts.
 *
 * @package    local_reset
 * @copyright  2024 LMSACE DEV
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

namespace local_reset;

use stdClass;

/**
 * Intitiate the process of user attempts for the supported modules.
 */
class reset {

    /**
     * Course to reset.
     *
     * @var stdclass
     */
    protected $course;

    /**
     * User to remove attempts.
     *
     * @var int
     */
    protected $userid;

    /**
     * Constructor
     *
     * @param stdclass $course
     */
    protected function __construct($course) {
        $this->course = $course;
    }

    /**
     * Create instance of this class.
     *
     * @param stdClass $course
     * @return self
     */
    public static function instance(stdClass $course): self {
        // Instance of the reset class, or its child classes.
        return new static($course);
    }

    /**
     * Initiate the reset process for the users.
     *
     * @param array|int $users Id of the users to reset.
     * @return array
     */
    public function reset($users) {

        if (!is_array($users)) {
            $users = [$users];
        }

        $results = false;

        foreach ($users as $userid) {
            $this->userid = $userid;

            // Get the list of supported modules namespace to reset.
            $modules = $this->supported_modules() ?: [];

            $data = ['courseid' => $this->course->id];

            foreach ($modules ?: [] as $classname => $parentnamespace) {

                $data['userid'] = $this->userid;
                // Create a instance of the module and perform the user attempts reset.
                $results = $classname::reset_user_attempt($data);
            }
        }

        return $results;
    }

    /**
     * List of modules that supports the reset of user attempts. It gets the list of available modules from the reset plugin.
     *
     * @return void
     */
    protected function supported_modules() {
        static $modules;

        if (!$modules) {
            $modules = \core_component::get_component_classes_in_namespace('local_reset', 'plugins');
        }

        return $modules;
    }
}
