<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Local Reset - Paticipants table which includes options to reset the user attempts.
 *
 * @package    local_reset
 * @copyright  2024 LMSACE DEV
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

namespace local_reset\table;

require_once($CFG->dirroot. '/enrol/locallib.php');

use pix_icon;

/**
 * Custom course participants table. Added action to perform the user attempt reset.
 */
class participants extends \core_user\table\participants {

    /**
     * Render the participants table.
     *
     * * Modified version of \core_user\table\participants "out" method by 2017 Mark Nelson <markn@moodle.com>
     * * Includes the enrolment column, removed last access column.
     *
     * @param int $pagesize Size of page for paginated displayed table.
     * @param bool $useinitialsbar Whether to use the initials bar which will only be used if there is a fullname column defined.
     * @param string $downloadhelpbutton
     */
    public function out($pagesize, $useinitialsbar, $downloadhelpbutton = '') {
        global $CFG, $OUTPUT, $PAGE, $DB;

        // Define the headers and columns.
        $headers = [];
        $columns = [];

        // At the very least, the user viewing this table will be able to use bulk actions to export it, so add 'select' column.
        /* $mastercheckbox = new \core\output\checkbox_toggleall('participants-table', true, [
            'id' => 'select-all-participants',
            'name' => 'select-all-participants',
            'label' => get_string('selectall'),
            'labelclasses' => 'sr-only',
            'classes' => 'm-1',
            'checked' => false,
        ]);
        $headers[] = $OUTPUT->render($mastercheckbox);
        $columns[] = 'select'; */

        $headers[] = get_string('fullname');
        $columns[] = 'fullname';

        $extrafields = \core_user\fields::get_identity_fields($this->context);
        foreach ($extrafields as $field) {
            $headers[] = \core_user\fields::get_display_name($field);
            $columns[] = $field;
        }

        $headers[] = get_string('roles');
        $columns[] = 'roles';

        // Get the list of fields we have to hide.
        $hiddenfields = array();
        if (!has_capability('moodle/course:viewhiddenuserfields', $this->context)) {
            $hiddenfields = array_flip(explode(',', $CFG->hiddenuserfields));
        }

        // Add column for groups if the user can view them.
        $canseegroups = !isset($hiddenfields['groups']);
        if ($canseegroups) {
            $headers[] = get_string('groups');
            $columns[] = 'groups';
        }

        // Do not show the columns if it exists in the hiddenfields array.
        // if (!isset($hiddenfields['lastaccess'])) {

        $headers[] = get_string('enrolmentmethods');
        $columns[] = 'enrolmentmethods';
        $this->no_sorting('enrolmentmethods');
        // }

        $canreviewenrol = has_capability('moodle/course:enrolreview', $this->context);
        if ($canreviewenrol && $this->courseid != SITEID) {
            $columns[] = 'status';
            $headers[] = get_string('action');
            $this->no_sorting('status');
        };

        $this->define_columns($columns);
        $this->define_headers($headers);

        // The name column is a header.
        $this->define_header_column('fullname');

        // Make this table sorted by last name by default.
        $this->sortable(true, 'lastname');

        $this->no_sorting('select');
        $this->no_sorting('roles');
        if ($canseegroups) {
            $this->no_sorting('groups');
        }

        $this->set_default_per_page(20);

        $this->set_attribute('id', 'participants');

        $this->countries = get_string_manager()->get_list_of_countries(true);
        $this->extrafields = $extrafields;
        if ($canseegroups) {
            $this->groups = groups_get_all_groups($this->courseid, 0, 0, 'g.*', true);
        }

        // If user has capability to review enrol, show them both role names.
        $allrolesnamedisplay = ($canreviewenrol ? ROLENAME_BOTH : ROLENAME_ALIAS);
        $this->allroles = role_fix_names(get_all_roles($this->context), $this->context, $allrolesnamedisplay);
        $this->assignableroles = get_assignable_roles($this->context, ROLENAME_BOTH, false);
        $this->profileroles = get_profile_roles($this->context);

        // parent::out($pagesize, $useinitialsbar, $downloadhelpbutton);

        if (!$this->columns) {
            $onerow = $DB->get_record_sql("SELECT {$this->sql->fields} FROM {$this->sql->from} WHERE {$this->sql->where}",
                $this->sql->params, IGNORE_MULTIPLE);
            //if columns is not set then define columns as the keys of the rows returned
            //from the db.
            $this->define_columns(array_keys((array)$onerow));
            $this->define_headers(array_keys((array)$onerow));
        }
        $this->pagesize = $pagesize;
        $this->setup();
        $this->query_db($pagesize, $useinitialsbar);
        $this->build_table();
        $this->close_recordset();
        $this->finish_output();

        if (has_capability('moodle/course:enrolreview', $this->context)) {
            $params = [
                'contextid' => $this->context->id,
                'uniqueid' => $this->uniqueid,
            ];
            $PAGE->requires->js_call_amd('core_user/status_field', 'init', [$params]);
        }
    }

    /**
     * Column to include the reset action.
     *
     * Get the list of configured enrolment methods,
     * Includes the action to reset if the user enrolled using any of the selected enrolment method.
     *
     * @param stdclass $row
     * @return void
     */
    public function col_status($row) {
        global $OUTPUT, $PAGE;

        if (self::has_user_enrolment($row->id, $this->course, $this->context)) {

            $url = new \moodle_url('/local/reset/reset.php', ['id' => $this->course->id, 'user' => $row->id]);
            $status = $OUTPUT->action_icon($url, new pix_icon('t/reset', get_string('reset')), null,
                //new \confirm_action(),
                ['class' => 'action-reset-user'], true
            );
        }

        return $status ?? '';
    }

    /**
     * Display the user enrolment methods for this course.
     *
     * @param stdclass $row
     * @return void
     */
    public function col_enrolmentmethods($row) {
        global $OUTPUT, $PAGE;

        $actionstoreset = get_config('local_reset', 'enrolmentmethods') ?: '';
        $actionstoreset = explode(',', $actionstoreset);

        $includereset = false;
        $manager = new \course_enrolment_manager($PAGE, $this->course);
        $userenrolments = $manager->get_user_enrolments($row->id);

        // Fetch the enrolment method shortname.
        $instances = array_column($userenrolments, 'enrolmentinstancename');

        return implode(',', $instances);
    }

    /**
     * User roles column.
     *
     * Modified version of core_user/table/participants.
     *
     * Removed the role edit option.
     *
     * @param \stdClass $data
     * @return string
     */
    public function col_roles($data) {
        $roles = isset($this->allroleassignments[$data->id]) ? $this->allroleassignments[$data->id] : [];
        $newroles = role_fix_names($roles);
        return implode(', ', array_column($newroles, 'localname'));
    }

    /**
     * Confirm the user has enrolment method to reset attempt.
     *
     * @param int $userid
     * @param stdclass $course
     * @param \context $context
     * @return bool
     */
    public static function has_user_enrolment(int $userid, $course, $context) {
        global $PAGE;

        $actionstoreset = get_config('local_reset', 'enrolmentmethods') ?: '';
        $actionstoreset = explode(',', $actionstoreset);

        $manager = new \course_enrolment_manager($PAGE, $course);
        $userenrolments = $manager->get_user_enrolments($userid);

        // Fetch the enrolment method shortname.
        $instances = array_column(array_map(fn($ue) => $ue = $ue->enrolmentinstance, $userenrolments), 'enrol');

        // Verify the user enrolled using any of the enrolments from the selected reset enrolments.
        if (array_intersect($instances, $actionstoreset) && has_capability('mod/quiz:attempt', $context, $userid)) {
            return true;
        }

        return false;
    }
}
