<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Extended class of elements for Poll.
 *
 * @package   element_poll
 * @copyright  2024 bdecent gmbh <https://bdecent.de>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
namespace element_poll;

use html_writer;
use stdClass;
use moodle_url;

defined('MOODLE_INTERNAL') || die;

require_once($CFG->libdir.'/formslib.php');

/**
 * poll element instance extended the base elements.
 */
class element extends \mod_contentdesigner\elements {

    /**
     * Shortname of the element.
     */
    const SHORTNAME = 'poll';

    /**
     * Result: hidden.
     */
    const RESULTHIDDEN = 0;

    /**
     * Result: Displaye always.
     */
    const RESULTDISPLAYALWAYS = 1;

    /**
     * Result: Displayed after their own rating .
     */
    const RESULTDISPLAYAFTERRATE = 2;

    /**
     * Update Rate: Disabled.
     */
    const DISABLED = 0;

    /**
     * Update Rate: Enabled.
     */
    const ENABLED = 1;

    /**
     * Participation is optional.
     */
    const MANDATORYNO = 0;

    /**
     * The learner must make a choice to proceed.
     */
    const MANDATORYYES = 1;

    /**
     * Element name which is visbile for the users
     *
     * @return string
     */
    public function element_name() {
        return get_string('pluginname', 'element_poll');
    }

    /**
     * Element shortname which is used as identical purpose.
     *
     * @return string
     */
    public function element_shortname() {
        return self::SHORTNAME;
    }

    /**
     * Icon of the element.
     *
     * @param renderer $output
     * @return void
     */
    public function icon($output) {
        return $output->pix_icon('i/scales', get_string('pluginname', 'element_poll'));
    }

    /**
     * Element form element definition.
     *
     * @param moodleform $mform
     * @param genreal_element_form $formobj
     * @return void
     */
    public function element_form(&$mform, $formobj) {
        global $DB;

        // Question Content.
        $mform->addElement('textarea', 'question', get_string('questioncontent', 'mod_contentdesigner'),
            ['rows' => 15, 'cols' => 40]);
        $mform->addHelpButton('question', 'questioncontent', 'mod_contentdesigner');

        // Options.
        $repeatarray = [];
        $repeatarray[] = $mform->createElement('text', 'option', get_string('optionno', 'mod_contentdesigner'));
        $repeatarray[] = $mform->createElement('hidden', 'optionid', 0);

        if ($formobj->_customdata['instanceid']) {
            $repeatno = $DB->count_records($this->tablename().'_options', ['pollid' => $formobj->_customdata['instanceid']]);
        } else {
            $repeatno = 5;
        }

        $repeateloptions = [];

        $repeateloptions['option']['helpbutton'] = ['polloptions', 'mod_contentdesigner'];
        $mform->setType('option', PARAM_CLEANHTML);

        $mform->setType('optionid', PARAM_INT);

        $formobj->repeat_elements($repeatarray, $repeatno,
                    $repeateloptions, 'option_repeats', 'option_add_fields', 3, null, true);

        // Number of selectable options.
        $mform->addElement('text', 'selectoptioncount', get_string('selectoptioncount', 'mod_contentdesigner'), []);
        $mform->setType('selectoptioncount', PARAM_INT);
        $mform->addHelpButton('selectoptioncount', 'selectoptioncount', 'mod_contentdesigner');

        // Results.
        $options = [
            self::RESULTHIDDEN => get_string('hidden', 'mod_contentdesigner'),
            self::RESULTDISPLAYALWAYS  => get_string('displayalways' , 'mod_contentdesigner'),
            self::RESULTDISPLAYAFTERRATE  => get_string('displaysubmitafter', 'mod_contentdesigner'),
        ];
        $mform->addElement('select', 'resulttype', get_string('resulttype', 'mod_contentdesigner'), $options);
        $mform->setDefault('resulttype', self::RESULTHIDDEN);
        $mform->addHelpButton('resulttype', 'resulttype', 'mod_contentdesigner');

        // Update rating.
        $options = [
            self::DISABLED => get_string('disable'),
            self::ENABLED => get_string('enable'),
        ];
        $mform->addElement('select', 'updaterating', get_string('updaterating', 'mod_contentdesigner'), $options);
        $mform->setDefault('updaterating', self::DISABLED);
        $mform->addHelpButton('updaterating', 'updaterating', 'mod_contentdesigner');

        // After submission message.
        $mform->addElement('textarea', 'submissionmessage', get_string('submissionmessage', 'mod_contentdesigner'),
            ['rows' => 15, 'cols' => 30]);
        $mform->addHelpButton('submissionmessage', 'submissionmessage', 'mod_contentdesigner');

        // Mandatory.
        $options = [
            self::MANDATORYNO => get_string('no'),
            self::MANDATORYYES => get_string('yes'),
        ];
        $mform->addElement('select', 'mandatory', get_string('mandatory', 'mod_contentdesigner'), $options);
        $mform->setDefault('mandatory', self::MANDATORYNO);
        $mform->addHelpButton('mandatory', 'mandatory', 'mod_contentdesigner');

    }

    /**
     * Get the options data
     *
     * @param int $instanceid Instance ID
     * @param array $instancedata Instance data.
     */
    public function get_optionsdata($instanceid, &$instancedata) {
        global $DB;

        if (!empty($instanceid) && ($options = $DB->get_records_menu($this->tablename().'_options',
                ['pollid' => $instanceid], 'id', 'id,text'))) {
            $choiceids = array_keys($options);
            $options = array_values($options);
            foreach (array_keys($options) as $key) {
                $instancedata['option['.$key.']'] = $options[$key];
                $instancedata['optionid['.$key.']'] = $choiceids[$key];
            }
        }

        return $instancedata;
    }

    /**
     * Prepare the form data.
     *
     * @param int $instanceid Instance ID
     * @return object $instancedata
     */
    public function prepare_formdata($instanceid) {
        $instancedata = (array) $this->get_instance($instanceid);
        $instancedata += (array) $this->get_optionsdata($instanceid, $instancedata);
        $instancedata['cmid'] = $this->cmid;
        return (object) ($instancedata);
    }

    /**
     * Update the element instance. Override the function in elements element class to add custom rules.
     *
     * @param stdclass $data
     * @return void
     */
    public function update_instance($data) {
        global $DB;

        if ($data->instanceid == false) {
            $data->timemodified = time();
            $data->timecreated = time();
            $data->id = $DB->insert_record($this->tablename, $data);

            foreach ($data->option as $key => $value) {
                $value = trim($value);
                if (isset($value) && $value <> '') {
                    $option = new stdClass();
                    $option->pollid = $data->id;
                    $option->text = $value;
                    $option->timecreated = time();
                    $option->timemodified = time();
                    $DB->insert_record("element_poll_options", $option);
                }
            }
            return $data->id;
        } else {
            $data->timecreated = time();
            $data->id = $data->instanceid;
            $data->submissionmessage = $data->submissionmessage;

            // Update, delete or insert answers.
            foreach ($data->option as $key => $value) {
                $value = trim($value);
                $option = new stdClass();
                $option->text = $value;
                $option->pollid = $data->id;
                $option->timemodified = time();
                $option->timecreated = time();
                if (isset($data->optionid[$key]) && !empty($data->optionid[$key])) { // Existing choice record.
                    $option->id = $data->optionid[$key];
                    if (isset($value) && $value <> '') {
                        $DB->update_record("element_poll_options", $option);
                    } else {
                        // Remove the empty (unused) option.
                        $DB->delete_records("element_poll_options", ["id" => $option->id]);
                        // Delete any answers associated with this option.
                        $DB->delete_records("element_poll_options", ["pollid" => $option->pollid, "optionid" => $option->id]);
                    }
                } else {
                    if (isset($value) && $value <> '') {
                        $DB->insert_record("element_poll_options", $option);
                    }
                }
            }

            if ($DB->update_record($this->tablename, $data)) {
                return $data->id;
            }
        }
    }

    /**
     * Analyze the poll is mantory to view upcoming then check the instance is attempted.
     *
     * @param stdclass $instance Instance data of the element.
     * @return bool True if need to stop the next instance Otherwise false if render of next elements.
     */
    public function prevent_nextelements($instance): bool {
        global $USER, $DB;
        if (isset($instance->mandatory) && $instance->mandatory) {
            if (empty(self::poll_get_my_response($instance))) {
                return true;
            }
        }
        return false;
    }

    /**
     * Initiate the JS.
     */
    public function initiate_js() {
        global $PAGE;
        $PAGE->requires->js_call_amd('element_poll/poll', 'init', []);
    }

    /**
     * Render the view of element instance, Which is displayed in the student view.
     *
     * @param stdclass $instance
     * @return void
     */
    public function render($instance) {
        global $DB, $USER, $OUTPUT, $PAGE;

        $html = '';
        $cm = get_coursemodule_from_id('contentdesigner', $this->cmid);
        $cm = \cm_info::create($cm);
        $course = $DB->get_record("course", ["id" => $cm->course]);
        $poll = self::get_poll($instance->id);
        $current = self::poll_get_my_response($poll);
        $allresponses = self::get_response_data($poll);
        $options = $this->prepare_options($poll, $USER, $allresponses); // Prepare the options for this poll.

        $html .= html_writer::start_div('poll-section', [
            'data-pollid' => $options['pollid'],
            'data-selectcount' => $options['selectableoptions'],
            'data-instanceid' => $this->cm->instance,
            'data-updaterating' => $poll->updaterating,
        ]);
        $html .= html_writer::tag('h5', format_string($instance->question), ['class' => "element-poll-question"]);
        $html .= $this->display_options($options, $current); // Display the poll options form.
        $html .= html_writer::end_div();

        if (self::poll_can_view_results($poll, $current)) {
            // Display the poll results.
            $results = self::prepare_poll_show_results($poll, $course, $cm, $allresponses);
            $resultarea = self::display_results($results);
        } else {
            $resultarea = '';
        }

        $html .= html_writer::tag('div', $resultarea, ['class' => 'resultarea']);

        return $html;
    }

    /**
     * Gets a full poll record
     *
     * @param int $pollid
     * @return object|bool The pool or false
     */
    public static function get_poll($pollid) {
        global $DB;

        if ($poll = $DB->get_record('element_poll', ["id" => $pollid])) {
            if ($options = $DB->get_records("element_poll_options", ["pollid" => $pollid], "id")) {
                foreach ($options as $option) {
                    $poll->option[$option->id] = $option->text;
                }
                return $poll;
            }
        }
        return false;
    }

    /**
     * Returns text string which is the answer that matches the id
     *
     * @param int $id
     * @return string
     */
    public function get_option_text($id) {
        global $DB;

        if ($result = $DB->get_record($this->tablename."_options", ["id" => $id])) {
            return $result->text;
        } else {
            return get_string("notanswered", "mod_contentdesigner");
        }
    }

    /**
     * Prepare options data.
     *
     * @param object $poll
     * @param object $user
     * @param array $allresponses
     * @return array
     */
    public function prepare_options($poll, $user, $allresponses) {
        global $DB;

        $cdisplay = ['options' => []];
        $cdisplay['pollid'] = $poll->id;
        $cdisplay['selectableoptions'] = $poll->selectoptioncount;
        $cdisplay['mandatory'] = $poll->mandatory;
        $cdisplay['updaterating'] = $poll->updaterating; // Add this to check the 'Update Rating' status.

        $selectedoptions = []; // To store the already selected options.

        // Fetch the selected options for the current user.
        $selectedoptions = $DB->get_records('element_poll_answers', ['pollid' => $poll->id, 'userid' => $user->id], '', 'optionid');
        $selectedcount = count($selectedoptions);
        $current = self::poll_get_my_response($poll);

        // Iterate through poll options.
        foreach ($poll->option as $optionid => $text) {
            if (isset($text)) { // Make sure there are no dud entries in the DB with blank text values.
                $option = new stdClass;
                $option->attributes = new stdClass;
                $option->attributes->value = $optionid;
                $option->text = format_string($text);

                // Check if the option was previously selected by the user.
                if (array_key_exists($optionid, $selectedoptions)) {
                    $option->attributes->checked = true; // Check the box for already selected options.
                }

                if ($current) {
                    // Disable unselected checkboxes.
                    if ($poll->updaterating && !array_key_exists($optionid, $selectedoptions)) {
                        // Only disable additional checkboxes if the number of selected options equals or exceeds the limit.
                        if ($selectedcount == $poll->selectoptioncount && $poll->selectoptioncount > 1) {
                            $option->attributes->disabled = true;
                        } else {
                            unset($option->attributes->disabled); // Allow selection if not at the limit.
                        }
                    }

                    if ($poll->selectoptioncount == 0) {
                        unset($option->attributes->disabled);
                    }
                }

                $cdisplay['options'][] = $option;
            }
        }

        return $cdisplay;
    }

    /**
     * Returns HTML to display poll of option
     *
     * @param object $options
     * @param rows|null $current my poll responses
     *
     * @return string
     */
    public function display_options($options, $current) {

        $attributes = ['method' => 'POST', 'action' => 'javascript:void(0);'];

        $html = html_writer::start_tag('form', $attributes);
        $html .= html_writer::start_tag('ul', ['class' => 'poll-options list-unstyled unstyled horizontal']);

        $selectableoptions = isset($options['selectableoptions']) ? $options['selectableoptions'] : 0;
        $disabled = (($options['updaterating'] == self::ENABLED || !$current)) ? false : true;
        $pollcount = 0;
        foreach ($options['options'] as $option) {
            $pollcount++;
            $html .= html_writer::start_tag('li', ['class' => 'option mr-3']);

            // Check if multiple options can be selected.
            if ($selectableoptions == 1) {
                // Only 1 option can be selected (use radio buttons).
                $option->attributes->name = 'answer';
                $option->attributes->type = 'radio';
                $option->attributes->disabled = $disabled ? 'disabled' : null;
            } else if ($selectableoptions > 1 || $selectableoptions == 0) {
                // More than 1 option or unlimited options can be selected (use checkboxes).
                $option->attributes->name = 'answer[]';
                $option->attributes->type = 'checkbox';
                if (isset($option->attributes->checked) && !isset($option->attributes->checked)) {
                    $option->attributes->disabled = $disabled ? 'disabled' : null;
                }
            }

            // Ensure unique ID by adding poll ID and option count.
            $pollid = $options['pollid'];
            $option->attributes->id = 'poll_' . $pollid . '_option_' . $pollcount;
            $option->attributes->class = 'mx-1';

            $labeltext = $option->text;

            $html .= html_writer::empty_tag('input', (array)$option->attributes);
            $html .= html_writer::tag('label', $labeltext, ['for' => $option->attributes->id]);
            $html .= html_writer::end_tag('li');
        }

        $html .= html_writer::end_tag('ul');
        $html .= html_writer::empty_tag('input', [
            'type' => 'submit',
            'value' => get_string('submit'),
            'class' => 'submit btn btn-primary',
            'disabled' => $disabled ? 'disabled' : null,
        ]);

        $html .= html_writer::end_tag('form');
        return $html;
    }

    /**
     * Get the user responses data.
     *
     * @param object $poll
     * @return array
     */
    public static function get_response_data($poll) {
        global $DB;

        $allresponses = [];
        // Get all the recorded responses for this poll.
        $rawresponses = $DB->get_records('element_poll_answers', ['pollid' => $poll->id]);
        // Use the responses to move users into the correct column.
        if ($rawresponses) {
            $answeredusers = [];
            foreach ($rawresponses as $response) {
                $allresponses[$response->optionid][$response->userid] = $response->id;
                $answeredusers[] = $response->userid;
            }
        }

        return $allresponses;
    }

    /**
     * Generate the poll result chart.
     *
     * @param stdClass $poll Poll responses object.
     * @return string the rendered chart.
     */
    public static function display_results($poll) {
        global $OUTPUT;

        $count = 0;
        $data = [];
        $numberofuser = 0;
        $totalresponses = 0;
        $percentageamount = 0;

        // First, calculate the total number of responses.
        foreach ($poll->options as $optionid => $option) {
            if (!empty($option->user)) {
                $totalresponses += count($option->user);
            }
        }

        // Now calculate the percentages for each option based on total responses.
        foreach ($poll->options as $optionid => $option) {
            if (!empty($option->user)) {
                $numberofuser = count($option->user);
            }

            // Calculate percentage based on the total number of responses.
            if ($totalresponses > 0) {
                $percentageamount = ((float)$numberofuser / (float)$totalresponses) * 100.0;
            }

            // Append the percentage to the label text to show it directly.
            $data['labels'][$count] = $option->text . ' (' . format_float($percentageamount, 1) . '%)';
            $data['series'][$count] = $numberofuser;
            $data['series_labels'][$count] = $numberofuser . ' (' . format_float($percentageamount, 1) . '%)';
            $count++;
            $numberofuser = 0;
        }

        // Create the chart and set it to doughnut style.
        $chart = new \core\chart_pie();
        $chart->set_doughnut(true);

        // Add the series to the chart.
        $series = new \core\chart_series(format_string(get_string("responses", "mod_contentdesigner")), $data['series']);
        $series->set_labels($data['series_labels']);
        $chart->add_series($series);
        $chart->set_labels($data['labels']);

        if ($poll->allresponses) {
            return $OUTPUT->notification(get_string("noresponseyet", 'mod_contentdesigner'), 'info', false);
        }

        // Render the chart.
        return $OUTPUT->render_chart($chart, false);
    }

    /**
     * Prepare the poll results.
     *
     * @param object $poll
     * @param object $course
     * @param object $cm
     * @param array $allresponses
     * @return object
     */
    public static function prepare_poll_show_results($poll, $course, $cm, $allresponses) {
        global $OUTPUT;

        $display = clone($poll);
        $display->coursemoduleid = $cm->id;
        $display->courseid = $course->id;

        $display->options = [];
        $allusers = [];
        foreach ($poll->option as $optionid => $optiontext) {
            $display->options[$optionid] = new stdClass;
            $display->options[$optionid]->text = format_string($optiontext, true,
                ['context' => \context_module::instance($cm->id)]);

            if (array_key_exists($optionid, $allresponses)) {
                $display->options[$optionid]->user = $allresponses[$optionid];
                $allusers = array_merge($allusers, array_keys($allresponses[$optionid]));
            }
        }
        unset($display->option);

        $display->numberofuser = count(array_unique($allusers));
        $display->allresponses = empty($allresponses) ? true : false;

        return $display;
    }

    /**
     * Return true if we are allowd to view the poll results.
     *
     * @param stdClass $poll Poll record
     * @param rows|null $current my poll responses
     * @return bool true if we can view the results, false otherwise.
     */
    public static function poll_can_view_results($poll, $current = null) {

        if (empty($current)) {
            $current = self::poll_get_my_response($poll);
        }

        if (($poll->resulttype == self::RESULTDISPLAYALWAYS ) ||
            ($poll->resulttype == self::RESULTDISPLAYAFTERRATE && !empty($current))) {
            return true;
        }
        return false;
    }

    /**
     * Get responses of a given user on a given poll.
     *
     * @param stdClass $poll poll record
     * @param int $userid User id
     * @return array of poll answers records
     */
    public static function poll_get_user_response($poll, $userid) {
        global $DB;
        return $DB->get_records('element_poll_answers', ['pollid' => $poll->id, 'userid' => $userid], 'optionid');
    }

    /**
     * Get my responses on a given poll.
     *
     * @param stdClass $poll Poll record
     * @return array of poll answers records
     */
    public static function poll_get_my_response($poll) {
        global $USER;
        return self::poll_get_user_response($poll, $USER->id);
    }
}
