<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Extended class of elements for Question.
 *
 * @package   element_question
 * @copyright  2022 bdecent gmbh <https://bdecent.de>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
namespace element_question;

use html_writer;
use moodle_url;

/**
 * Question element instance extended the base elements.
 */
class element extends \mod_contentdesigner\elements {

    /**
     * Shortname of the element.
     */
    const SHORTNAME = 'question';

    /**
     * Element name which is visbile for the users
     *
     * @return string
     */
    public function element_name() {
        return get_string('pluginname', 'element_question');
    }

    /**
     * Element shortname which is used as identical purpose.
     *
     * @return string
     */
    public function element_shortname() {
        return self::SHORTNAME;
    }

    /**
     * Icon of the element.
     *
     * @param renderer $output
     * @return void
     */
    public function icon($output) {
        return $output->pix_icon('e/stylequestion', get_string('pluginname', 'element_question'));
    }

    /**
     * Queestion element supports custom form. Therefore it contain all the form definitions separated.
     *
     * @return bool
     */
    public function supports_custom_form() : bool {
        return true;
    }

    /**
     * Replace the element on refersh the content. Some elements may need to update the content on refresh the elmenet.
     */
    public function supports_replace_onrefresh(): bool {
        return false;
    }

    /**
     * Element form element definition.
     *
     * @param moodle_form $mform
     * @param genreal_element_form $formobj
     * @return void
     */
    public function element_form(&$mform, $formobj) {
        return general_element_form::class;
    }

    /**
     * Initiate the JS.
     */
    public function initiate_js() {
        global $PAGE;
        // print_obj($this);exit;
    }

    /**
     * Render the view of element instance, Which is displayed in the student view.
     *
     * @param \stdClass $instance
     * @return void
     */
    public function render_attempt($instance) {
        global $CFG;
        require_once($CFG->libdir . '/questionlib.php');
        $attempt = new attempt($instance->cmid, $instance->id, $instance);

        return $attempt->render_attempt();
    }

    /**
     * Wrapper round print_question from lib/questionlib.php.
     *
     * @param int $slot the id of a question in this quiz attempt.
     * @return string HTML of the question.
     */
    public function render_attempt_for_commenting($instance) {
        global $CFG;
        require_once($CFG->libdir . '/questionlib.php');
        $attempt = new attempt($instance->cmid, $instance->id, $instance);
        return $attempt->render_attempt_for_commenting();
    }

    public function render($instance) {
        global $PAGE;

        $PAGE->requires->js_call_amd('element_question/question', 'questionSubmitted', ['instanceid' => $instance->id]);

        $url = new moodle_url('/mod/contentdesigner/element/question/startattempt.php', [
            'id' => $instance->id, 'cmid' => $instance->cmid, 'sesskey' => sesskey(),
        ]);

        return html_writer::tag('div',
            html_writer::tag('iframe', '', ['src' => $url, 'class' => 'responsive']),
            ['class' => '']
        );
    }

}
