<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Videotime element external werbservice deifintion to manage the videotime completion.
 *
 * @package   element_videotime
 * @copyright  2024 bdecent gmbh <https://bdecent.de>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

namespace element_question;

defined('MOODLE_INTERNAL') || die('No direct access');

use external_value;
require_once($CFG->libdir . '/externallib.php');
require_once($CFG->dirroot . '/question/engine/bank.php');

/**
 * Video time element external service methods.
 */
class external extends \external_api {

    /**
     * Paramters definition for the methos to get list of video time instances.
     *
     * @return external_function_parameters
     */
    public static function get_question_menu_parameters() {
        return new \external_function_parameters(
            [
                'query' => new external_value(PARAM_RAW, 'Query string'),
                'categoryid' => new external_value(PARAM_RAW_TRIMMED, 'Course module id'),
                'limitfrom' => new external_value(PARAM_INT, 'limitfrom we are fetching the records from', VALUE_DEFAULT, 0),
                'limitnum' => new external_value(PARAM_INT, 'Number of records to fetch', VALUE_DEFAULT, 25),
            ]
        );
    }

    /**
     * Fetch the list of video time instance menu.
     *
     * @param int $cmid Course module id.
     * @param int $instanceid Instance id of the H5P.
     * @param array $result Result object form the H5P data statement.
     * @return array
     */
    public static function get_question_menu($query, $categoryid, $limitfrom, $limitnum) {
        global $DB, $USER;

        $params = self::validate_parameters(self::get_question_menu_parameters(), [
            'query' => $query,
            'categoryid' => $categoryid,
            'limitfrom' => $limitfrom,
            'limitnum' => $limitnum,
        ]);


        // Look at each question in the category.
        $questionids = \question_bank::get_finder()->get_questions_from_categories(explode(',', $categoryid), null);

        if (empty($questionids)) {
            return [];
        }

        list($insql, $inparams) = $DB->get_in_or_equal(array_values($questionids), SQL_PARAMS_NAMED, 'cate');
        $sql = "SELECT q.id, q.name
                FROM {question} q
                WHERE q.id $insql";

        $list = $DB->get_records_sql($sql, ['questionids' => $questionids] + $inparams);

        return $list;
    }

    /**
     * Returns the list of instances menu.
     *
     * @return external_value True if data updated otherwise  returns false.
     */
    public static function get_question_menu_returns() {
        return new \external_multiple_structure(
            new \external_single_structure(
                [
                    'id' => new external_value(PARAM_INT, 'Video time ID'),
                    'name' => new external_value(PARAM_RAW, 'Video time name'),
                ]
            )
        );
    }


    public static function get_question_variations_parameters() {

        return new \external_function_parameters(
            array(
                'questionid' => new external_value(PARAM_INT, 'limitfrom we are fetching the records from', VALUE_REQUIRED),
                'cmid' => new external_value(PARAM_INT, 'Course module id', VALUE_REQUIRED),
            )
        );
    }

    /**
     * Get the question variations.
     *
     * @param int $questionid
     * @param int $cmid
     * @return void
     */
    public static function get_question_variations(int $questionid, int $cmid) {
        global $PAGE;

        $params = self::validate_parameters(self::get_question_variations_parameters(), [
            'questionid' => $questionid,
            'cmid' => $cmid,
        ]);

        if (empty($PAGE->context)) {
            $PAGE->set_context(\context_module::instance($cmid));
        }

        $selector = \core_question\output\question_version_selection::make_for_question('question_comment_version_dropdown', $questionid);
        $qbankrenderer = $PAGE->get_renderer('core_question', 'bank');
        $versionselection = $selector->export_for_template($qbankrenderer);

        $list[] = ['id' => 0, 'name' => get_string('alwayslatest', 'mod_contentdesigner')];
        foreach ($versionselection['options'] as $version) {
            $list[] = ['id' => $version->version, 'name' => $version->name];
        }

        return $list;
    }

    /**
     * Returns the list of instances menu.
     *
     * @return external_value True if data updated otherwise  returns false.
     */
    public static function get_question_variations_returns() {
        return new \external_multiple_structure(
            new \external_single_structure(
                array(
                    'id' => new external_value(PARAM_INT, 'Video time ID'),
                    'name' => new external_value(PARAM_RAW, 'Video time name'),
                )
            )
        );
    }

}
