<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Form for editing a general element.
 *
 * @package    mod_contentdesigner
 * @copyright  2022 bdecent gmbh <https://bdecent.de>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

namespace element_question;

defined('MOODLE_INTERNAL') || die();

use mod_contentdesigner\editor;
use stdClass;

require_once($CFG->dirroot.'/lib/formslib.php');

/**
 * General option form to create elements.
 *
 * @copyright 2022 bdecent gmbh <https://bdecent.de>
 * @license http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class general_element_form extends \moodleform {

    /**
     * Make the custom data as public varaible to access on the elements forms.
     *
     * @var array
     */
    public $_customdata;

    /**
     * Define the form.
     */
    public function definition() {
        global $USER, $CFG, $COURSE, $PAGE, $DB;

        $mform = $this->_form;
        $element = $this->_customdata['elementobj'];
        $instanceid = $this->_customdata['instanceid'];
        $cmid = $this->_customdata['cmid'];
        $courseid = $this->_customdata['courseid'];
        $questionid = $this->_customdata['questionid'] ?? '';

        $mform->addElement('header', 'elementsettings',
            get_string('elementsettings', 'mod_contentdesigner', ucfirst($element->element_name())));

        // Course id.
        $mform->addElement('hidden', 'courseid', $courseid);
        $mform->setType('courseid', PARAM_INT);

        $mform->addElement('hidden', 'cmid', $cmid);
        $mform->setType('cmid', PARAM_INT);

        $category = $DB->get_field('course', 'category', ['id' => $courseid]);
        $mform->addElement('questioncategory', 'categoryid', get_string('category', 'question'),[
            'contexts' => [
                \context_system::instance(),
                \context_coursecat::instance($category),
                \context_course::instance($courseid),
                \context_module::instance($cmid),
            ],
            'top' => true,
            'currentcat' => true,
        ]);
        /* $mform->addElement('select', 'categoryid', get_string('questioncategory', 'question'), []);
        $mform->addRule('categoryid', null, 'required', null, 'client'); */

        // Cmid.
        $options = [
            'ajax' => 'element_question/form-question-selector',
        ];
        $mform->addElement('autocomplete', 'questionid', get_string('question'), [], $options);
        $mform->addRule('questionid', null, 'required', null, 'client');
        $mform->disabledIf('questionid', 'categoryid', 'eq', '');

        $options = [
            'ajax' => 'element_question/form-questionversion-selector',
        ];
        $mform->addElement('autocomplete', 'questionversion', get_string('version'), $versions ?? [ 0 => get_string('alwayslatest', 'mod_quiz')], $options);
        $mform->addRule('questionversion', null, 'required', null, 'client');

        // $mform->addElement('select', 'testquestion', 'test', ['0' => 'name']);
        // $mform->disabledIf('questionversion', 'categoryid', 'eq', '');
        // $PAGE->requires->js_call_amd('filter_embedquestion/questionid_choice_updater', 'init');

        $behaviours = \question_engine::get_behaviour_options($this->_customdata['behaviour'] ?? '');
        $mform->addElement('select', 'preferredbehaviour',
                get_string('howquestionsbehave', 'question'), $behaviours);
        $mform->addHelpButton('preferredbehaviour', 'howquestionsbehave', 'question');

        $this->standard_element_settings($mform);

        $buttonstr = '';
        if ($instanceid) {
            $buttonstr = get_string('elementupdate', 'mod_contentdesigner');
        } else {
            $buttonstr = get_string('elementcreate', 'mod_contentdesigner');
        }

        $mform->addElement('hidden', 'chapterid', $this->_customdata['chapterid']);
        $mform->setType('chapterid', PARAM_INT);

        $mform->addElement('hidden', 'sesskey', sesskey());
        $mform->setType('sesskey', PARAM_ALPHANUMEXT);

        $mform->addElement('hidden', 'position', $this->_customdata['position']);
        $mform->setType('position', PARAM_ALPHA);

        $this->add_action_buttons(true, $buttonstr);

        // Form video time selector.
        $PAGE->requires->js_call_amd('element_question/form-question-selector', 'updateCategoryID', ['questionid', 'categoryid']);
        $PAGE->requires->js_call_amd('element_question/form-questionversion-selector', 'updateQuestionID', ['questionversion', 'questionid']);


        // $PAGE->requires->js_call_amd('element_question/form-question-selector', 'updateVersion', ['questionversion', 'questionid']);

    }

    /**
     * Defined the standard general options moodle form elements for content designer elements.
     *
     * @param moodle_form $mform Moodle quick form.
     * @return void
     */
    public function standard_element_settings(&$mform) {
        global $CFG;

        // Accessibility: "Required" is bad legend text.
        $strrequired = get_string('required');

        // Print the required moodle fields first.
        // Title for General element.
        $mform->addElement('header', 'generalsettings', get_string('generaltitle', 'mod_contentdesigner'));

        $mform->addElement('text', 'title',  get_string('elementtitle', 'mod_contentdesigner'),  'maxlength="100" size="30"');
        $mform->setType('title', PARAM_NOTAGS);

        // Visibility for General element.
        $visibleoptions = [
            1 => get_string('visible'),
            0 => get_string('hidden', 'mod_contentdesigner'),
        ];
        $mform->addElement('select', 'visible', get_string('visibility', 'mod_contentdesigner'), $visibleoptions);
        $mform->addHelpButton('visible', 'visibility', 'mod_contentdesigner');

        // Margin for General element.
        $mform->addElement('text', 'margin',  get_string('margin', 'mod_contentdesigner'), 'size="30"');
        $mform->setType('margin', PARAM_RAW);

        // Padding for General element.
        $mform->addElement('text', 'padding',  get_string('padding', 'mod_contentdesigner'), 'size="30"');
        $mform->setType('padding', PARAM_RAW);

        $mform->addElement('header', 'backgroundsettings', get_string('backgroundtitle', 'mod_contentdesigner'));

        // Background for general element.
        $mform->addElement('text', 'abovecolorbg', get_string('abovecolorbg', 'mod_contentdesigner'),
            ['placeholder' => 'linear-gradient(#e66465, #9198e5)', 'size' => "60"]);
        $mform->setType('abovecolorbg', PARAM_RAW);

        $options = [
            'accepted_types' => ['image'],
            'maxbytes' => 0,
            'maxfiles' => 1,
            'subdirs' => 0,
        ];
        $mform->addElement('filemanager', 'bgimage', get_string('elementbgimage', 'mod_contentdesigner'), null, $options);

        $mform->addElement('text', 'belowcolorbg', get_string('belowcolorbg', 'mod_contentdesigner'),
            ['placeholder' => 'linear-gradient(#e66465, #9198e5)', 'size' => "60"]);
        $mform->setType('belowcolorbg', PARAM_RAW);

        $mform->addElement('header', 'animationsettings', get_string('animationtitle', 'mod_contentdesigner'));

        // Animation for general element.
        $animationtype = [
            0 => get_string('none'),
            'fadeIn' => get_string('fadein', 'mod_contentdesigner'),
            'slideInRight' => get_string('slidefromright', 'mod_contentdesigner'),
            'slideInLeft' => get_string('slidefromleft', 'mod_contentdesigner'),
        ];
        $mform->addElement('select', 'animation', get_string('stranimation', 'mod_contentdesigner'), $animationtype);

        $durations = [
            'slow' => get_string('strslow', 'mod_contentdesigner'),
            'normal' => get_string('strnormal', 'mod_contentdesigner'),
            'fast' => get_string('strfast', 'mod_contentdesigner'),
        ];
        $mform->addElement('select', 'duration', get_string('strduration', 'mod_contentdesigner'), $durations);
        $mform->addElement('text', 'delay', get_string('strdelay', 'mod_contentdesigner'));
        $mform->setType('delay', PARAM_INT);

        $mform->addElement('header', 'scrollingsettings', get_string('scrollingeffectstitle', 'mod_contentdesigner'));

        // Animation for general element.
        $scrolldirections = [
            0 => get_string('none'),
            'left' => get_string('toleft', 'mod_contentdesigner'),
            'right' => get_string('toright', 'mod_contentdesigner'),
        ];
        $mform->addElement('select', 'direction', get_string('strdirection', 'mod_contentdesigner'), $scrolldirections);
        $mform->addElement('select', 'speed', get_string('speed', 'mod_contentdesigner'), range(0, 10));
        $mform->addElement('text', 'viewport', get_string('viewport', 'mod_contentdesigner'));
        $mform->setType('viewport', PARAM_INT);

        $mform->addElement('header', 'responsivesettings', get_string('responsivetitle', 'mod_contentdesigner'));
        // Responsive for general element.
        $mform->addElement('advcheckbox', 'hidedesktop', get_string('hideondesktop', 'mod_contentdesigner'));
        $mform->addElement('advcheckbox', 'hidetablet', get_string('hideontablet', 'mod_contentdesigner'));
        $mform->addElement('advcheckbox', 'hidemobile', get_string('hideonmobile', 'mod_contentdesigner'));
    }

    public function definition_after_data() {
        parent::definition_after_data();
        $mform = $this->_form;

        /* $categoryidnumbers = $mform->getElementValue('categoryidnumber');
        if (is_null($categoryidnumbers)) {
            return;
        }
        $categoryidnumber = $categoryidnumbers[0];
        if ($categoryidnumber === '' || $categoryidnumber === null) {
            return;
        } */

        $categoryid = $mform->getElementValue('categoryid');
        $categoryid = is_array($categoryid) ? current($categoryid) : $categoryid;
        if ($categoryid) {
            $variants = \element_question\external::get_question_menu('', $categoryid, 0, 100);
            $options = array_combine(array_column($variants, 'id'), array_column($variants, 'name'));
            $mform->getElement('questionid')->loadArray($options);
            // $options = array_merge([0 => get_string('alwayslatest', 'mod_quiz')], $options);
        }

        $questionid = $mform->getElementValue('questionid');
        $cmid = $mform->getElementValue('cmid');
        $questionid = is_array($questionid) ? current($questionid) : $questionid;
        if ($questionid) {
            $variants = \element_question\external::get_question_variations($questionid, $cmid);
            $options = array_combine(array_column($variants, 'id'), array_column($variants, 'name'));
            $mform->getElement('questionversion')->loadArray($options);
            // $options = array_merge([0 => get_string('alwayslatest', 'mod_quiz')], $options);
        }
    }


}
