<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Content designer module instance add and update form.
 *
 * @package    mod_contentdesigner
 * @copyright  2022 bdecent gmbh <https://bdecent.de>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

defined('MOODLE_INTERNAL') || die;

require_once($CFG->dirroot.'/course/moodleform_mod.php');
require_once($CFG->dirroot.'/mod/contentdesigner/lib.php');

/**
 * Content designer module form.
 */
class mod_contentdesigner_mod_form extends moodleform_mod {

    /**
     * Define the mform elements.
     * @return void
     */
    public function definition() {
        global $CFG, $DB, $OUTPUT;

        $mform =& $this->_form;

        $mform->addElement('header', 'general', get_string('general', 'form'));

        $mform->addElement('text', 'name', get_string('name'), ['size' => '48']);
        if (!empty($CFG->formatstringstriptags)) {
            $mform->setType('name', PARAM_TEXT);
        } else {
            $mform->setType('name', PARAM_CLEANHTML);
        }
        $mform->addRule('name', null, 'required', null, 'client');
        $mform->addRule('name', get_string('maximumchars', '', 255), 'maxlength', 255, 'client');

        $this->standard_intro_elements();

        $this->navigation_definition($mform);

        $this->standard_coursemodule_elements();

        $this->add_action_buttons();
    }

    /**
     * Custom completion rules definition.
     *
     * @return void
     */
    public function add_completion_rules() {
        global $CFG;
        $mform = $this->_form;

        if ($CFG->branch >= 403) {
            $suffix = $this->get_suffix();
            $completionendreach = 'completionendreach' . $suffix;
            $completionmandatory = 'completionmandatory' . $suffix;
        } else {
            $completionendreach = 'completionendreach';
            $completionmandatory = 'completionmandatory';
        }

        $mform->addElement('checkbox', $completionendreach, get_string('completionendreach', 'contentdesigner') );
        $mform->setDefault($completionendreach, 0);
        $mform->addHelpButton($completionendreach, 'completionendreach', 'contentdesigner');

        $mform->addElement('checkbox', $completionmandatory, get_string('completionmandatory', 'contentdesigner') );
        $mform->setDefault($completionmandatory, 0);
        $mform->addHelpButton($completionmandatory, 'completionmandatory', 'contentdesigner');

        return [$completionendreach, $completionmandatory];
    }

    /**
     * Navigation method form fields.
     *
     * @param moodle_form $mform
     */
    public function navigation_definition(&$mform) {

        $mform->addElement('header', 'navigationhead', get_string('navigationhead', 'contentdesigner'));
        $options = [
            CONTENTDESIGNER_NAVIGATION_SEQUENTIAL => get_string('sequential', 'contentdesigner'),
            CONTENTDESIGNER_NAVIGATION_FREE => get_string('free', 'contentdesigner'),
        ];
        $default = get_config('mod_contentdesigner', 'navigation');
        $mform->addElement('select', 'navigation', get_string('navigation', 'contentdesigner'), $options);
        $mform->addHelpButton('navigation', 'navigation', 'contentdesigner');
        $mform->setDefault('navigation', $default ?: CONTENTDESIGNER_NAVIGATION_SEQUENTIAL);
    }

    /**
     * Validate the form to check custom completion has selected conditions.
     *
     * @param array $data Input data not yet validated.
     * @return bool True if one or more rules is enabled, false if none are.
     */
    public function completion_rule_enabled($data) {
        return (!empty($data['completionendreach']) || !empty($data['completionmandatory']));
    }
}
