<?php

namespace qtype_imagechoice;

use curl;
require_once($CFG->dirroot . '/lib/adminlib.php');

/**
 * Nct license verification request handler.
 */
class nct_license {

    /**
     * Site url to verify license, WITHOUT END SLASH.
     */
    protected const SITEURL = 'https://lmsacelab.com/wp62';

    /**
     * Status of Valid license.
     */
    protected const STATUS_VALID = 'valid';

    /**
     * Status of Invalid license.
     */
    protected const STATUS_INVALID = 'invalid';

    /**
     * Status of Expired license.
     */
    protected const STATUS_EXPIRED = 'expired';

    /**
     * Status of Success of license verification.
     */
    protected const STATUS_SUCCESS = 'success';

    /**
     * Instatnce of the nct licnese.
     *
     * @var self
     */
    protected static $instance;

    /**
     * License key config name
     */
    public const CONFKEY = 'license_key';

    /**
     * Configuration name of the license status.
     */
    public const CONFSTATUS = 'license_status';

    /**
     * Verification frequency time.
     */
    public const VERIFICATIONFREQ = 60 * 60 * 24;

    /**
     * Name of the component.
     *
     * @var string
     */
    protected $component;

    /**
     * Name of the license key config
     *
     * @var string
     */
    protected $strlicensekey;

    /**
     * License verification endpoint url.
     *
     * @var string
     */
    protected $apiurl;

    /**
     * Plugin verion.
     *
     * @var int
     */
    protected $pluginversion;

    /**
     * Constructor
     */
    protected function __construct() {
        $this->component = __NAMESPACE__;
        $this->strlicensekey = 'license_key';
        $this->apiurl = self::SITEURL . '/wp-json/nct-license/v2/';
        $this->pluginversion = get_component_version($this->component);
    }

    /**
     * Set the license config as component config.
     *
     * @param string $name
     * @param mixed $value
     * @return void
     */
    protected function set_config($name, $value) {
        set_config($name, $value, $this->component);
    }

    /**
     * Get the config from component configs.
     *
     * @param string $name
     *
     * @return mixed
     */
    public function get_config($name) {
        return get_config($this->component, $name);
    }

    /**
     * Get the license key currently activated.
     *
     * @return string
     */
    public function get_key() {
        return $this->get_config($this->strlicensekey);
    }

    /**
     * Returns the nct_license class instance.
     *
     * @return nct_license
     */
    public static function instance() {
        if (self::$instance == null) {
            self::$instance = new self();
        }

        return self::$instance;
    }

    /**
     * Send a requestion to the NCT endpoint to activate or deactivate the license.
     *
     * @param string $endpoint
     * @param array $params
     *
     * @return string
     */
    private function send_request($endpoint, $params = []) {
        global $CFG;

        $url =  $this->apiurl . $endpoint;

        $componentdetails = [
            'plugin_name' => $this->component,
            'site_url' => urlencode($CFG->wwwroot),
            'plugin_version' => $this->pluginversion,
        ];

        $jsondata = json_encode($params + $componentdetails);

        $options = array(
            'RETURNTRANSFER' => 1,
            'HEADER' => 0,
            'FAILONERROR' => 0,
        );

        $header = array(
            'Content-Type: application/json',
            'Accept: application/json',
        );

        $curl = new \curl();
        $curl->setHeader($header);

        $response = $curl->post($url, $jsondata, $options);

        return json_decode($response, true);
    }

    /**
     * Activate license.
     *
     * @param string $licensekey
     * @return bool
     */
    public function activate_license($licensekey) {

        $response = $this->send_request('activate', [$this->strlicensekey => $licensekey]);

        if (isset($response['status']) && $response['status'] === self::STATUS_SUCCESS) {
            $this->set_config($this->strlicensekey, $licensekey);
            $this->set_config(self::CONFSTATUS, self::STATUS_VALID);
            return [true, $response['message'] ?? ''];
        } else {
            $this->set_config(self::CONFSTATUS, self::STATUS_INVALID);
        }

        return [false, $response['message'] ?? ''];
    }

    /**
     * Deactivate license.
     *
     * @return bool
     */
    public function deactivate_license() {

        $license_key = $this->get_config($this->strlicensekey);

        if (!$license_key) {
            return false;
        }

        $response = $this->send_request('deactivate', [$this->strlicensekey => $license_key]);

        if ($response['status'] === self::STATUS_SUCCESS) {
            $this->set_config($this->strlicensekey, null);
            $this->set_config(self::CONFSTATUS, self::STATUS_INVALID);
            return [true, $response['message'] ?? ''];
        }

        return [false, $response['message'] ?? ''];
    }

    /**
     * Check license status, is the license is activated or deactivated.
     * Once the status is verified, it will store the status and the verification time,
     * On next verification, it will use the stored results until the verification frequecy reached.
     *
     * @return void
     */
    public function check_license_status() {

        $license_key = $this->get_config($this->strlicensekey);

        if (!$license_key) {
            return self::STATUS_INVALID;
        }

        // Already verified within the current day.
        $lastchecktime = $this->get_config('nct_license_verification');
        if ($lastchecktime && time() - $lastchecktime <= self::VERIFICATIONFREQ) {
            $status = $this->get_config(self::CONFSTATUS);
            return $status;
        }

        $response = $this->send_request( 'status', [ $this->strlicensekey => $license_key ] );

        if ($response['status'] === self::STATUS_VALID) {
            // License is valid.
            $this->set_config(self::CONFSTATUS, self::STATUS_VALID);
        } elseif ($response['status'] === self::STATUS_EXPIRED) {
            // Licnese expired.
            $this->set_config(self::CONFSTATUS, self::STATUS_EXPIRED);
        } else {
            // License is invalid.
            $this->set_config(self::CONFSTATUS, self::STATUS_INVALID);
        }

        $this->set_config('nct_license_verification', time());

        return $response['status'];
    }

    /**
     * Verify the license is enabled and available.
     *
     * @return bool
     */
    public static function is_available() : bool {
        $instance = self::instance();
        $status = $instance->check_license_status();
        return $status === self::STATUS_VALID;
    }

    /**
     * Include the modal to alert the license activation on question creation.
     *
     * @return void
     */
    public static function include_alert_modal() {
        global $PAGE;

        $activatelicensetitle = get_string('activatelicense', 'qtype_imagechoice');
        $activatelicense = get_string('nct:activatlicense', 'qtype_imagechoice');
        $PAGE->requires->js_amd_inline("require(['jquery', 'core/local/modal/alert'], function($, ModalAlert) {
            ModalAlert.create({
                title: '$activatelicensetitle',
                body: '$activatelicense',
                show: true,
            })
        })");
    }
}
