<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Defines the editing form for the image choice question type.
 *
 * @package    qtype_imagechoice
 * @copyright  2024 LMSACE <lmsace.com>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

use qtype_imagechoice\nct_license;

defined('MOODLE_INTERNAL') || die();

/**
 * Image choice editing form definition.
 */
class qtype_imagechoice_edit_form extends question_edit_form {

    public $fileoptions = array(
        'subdirs' => true,
        'maxfiles' => 1,
        'maxbytes' => 0,
    );

    /**
     * Number of minimum answers for a question.
     */
    const QUESTION_NUMANS_START = 2;

    /**
     * Force the user to listen the audio.
     */
    public const FORCEAUDIO = 1;

    /**
     * Force the user to listern the seconds.
     */
    public const FORCESECONDS = 2;

    /**
     * Disable the next button to force the learner to listen the feedback.
     */
    const NEXTDISABLE = 0;

    /**
     * Hideen the next button to force the learner to listen the feedback.
     */
    const NEXTHIDDEN = 1;

    /**
     * Add question-type specific form fields.
     *
     * @param object $mform the form being built.
     */
    protected function definition_inner($mform) {

        $menu = array(
            get_string('answersingleno', 'qtype_multichoice'),
            get_string('answersingleyes', 'qtype_multichoice'),
        );
        $mform->addElement('select', 'single',
                get_string('answerhowmany', 'qtype_multichoice'), $menu);
        $mform->setDefault('single', $this->get_default_value('single',
            get_config('qtype_imagechoice', 'answerhowmany')));

        // Shuffle answers.
        $mform->addElement('advcheckbox', 'shuffleanswers',
                get_string('shuffleanswers', 'qtype_multichoice'), null, null, array(0, 1));
        $mform->addHelpButton('shuffleanswers', 'shuffleanswers', 'qtype_multichoice');
        $mform->setDefault('shuffleanswers', $this->get_default_value('shuffleanswers',
            get_config('qtype_imagechoice', 'shuffleanswers')));

        // Force listen the feedback.
        $options = [
            0 => get_string('none'),
            self::FORCEAUDIO => get_string('forceaudio', 'qtype_imagechoice'),
            self::FORCESECONDS => get_string('forceduration', 'qtype_imagechoice'),
        ];
        $mform->addElement('select', 'forcefeedback', get_string('forcefeedback', 'qtype_imagechoice'), $options);

        // Force seconds to listner the feedback.
        $mform->addElement('text', 'feedbackduration', get_string('forcesecondsconfig', 'qtype_imagechoice'), 60);
        $mform->setType('feedbackduration', PARAM_INT);
        $mform->addRule('feedbackduration', get_string('error'), 'numeric', null, 'client');
        $mform->hideIf('feedbackduration', 'forcefeedback', 'neq', self::FORCESECONDS);

        // Force the next to hide or disable until the user listen the feedback.
        $options = [
            self::NEXTDISABLE => get_string('disable'),
            self::NEXTHIDDEN => get_string('hide'),
        ];

        $mform->addElement('select', 'forcenext', get_string('forcenextconfig', 'qtype_imagechoice'), $options);
        $mform->hideIf('forcenext', 'forcefeedback', 'eq', 0);

        // Animation.
        $options = [
            'fadeIn' => get_string('fadein', 'qtype_imagechoice'),
            'fadeInUp' => get_string('fadeinup', 'qtype_imagechoice'),
            'fadeInDown' => get_string('fadeindown', 'qtype_imagechoice'),
            'fadeInLeft' => get_string('fadeinleft', 'qtype_imagechoice'),
            'fadeInRight' => get_string('fadeinright', 'qtype_imagechoice'),
            'backInUp' => get_string('backinup', 'qtype_imagechoice'),
            'slideInUp' => get_string('slideinup', 'qtype_imagechoice'),
            'slideInRight' => get_string('slideinright', 'qtype_imagechoice'),
            'slideInLeft' => get_string('slideinleft', 'qtype_imagechoice'),
            'slideInDown' => get_string('slideindown', 'qtype_imagechoice'),
            'zoomIn' => get_string('zoomin', 'qtype_imagechoice'),
            'zoomInUp' => get_string('zoominup', 'qtype_imagechoice'),
            'bounceIn' => get_string('bouncein', 'qtype_imagechoice'),
            'bounceInDown' => get_string('bounceindown', 'qtype_imagechoice'),
            'bounceInLeft' => get_string('bounceinleft', 'qtype_imagechoice'),
            'bounceInRight' => get_string('bounceinright', 'qtype_imagechoice'),
            'bounceInUp' => get_string('bounceinup', 'qtype_imagechoice'),
        ];
        $mform->addElement('select', 'animation', get_string('animation', 'qtype_imagechoice'), $options);


        // Verify the nct license is available.
        if (!nct_license::is_available()) {
            $mform->addElement('static', 'activatelicense', '', get_string('nct:activatlicense', 'qtype_imagechoice'));
            nct_license::include_alert_modal();
        } else {
            $this->add_per_answer_fields($mform, get_string('choiceno', 'qtype_multichoice', '{no}'),
                question_bank::fraction_options_full(), min(2, QUESTION_NUMANS_START), 2);
        }

    }

    /**
     * Remove the action button after definition.
     */
    public function definition_after_data() {
        $mform = $this->_form;

        if (!nct_license::is_available()) {
            // print_pre($mform);exit;
            $mform->removeElement('buttonar');
            $mform->removeElement('updatebuttonar');
        }
    }

    /**
     * Answer fields.
     *
     * @param moodle_form $mform
     * @param string $label
     * @param array $gradeoptions
     * @param array $repeatedoptions
     * @param string $answersoption
     * @return array
     */
    protected function get_per_answer_fields($mform, $label, $gradeoptions,
            &$repeatedoptions, &$answersoption) {

        $repeated = array();

        $repeated[] = $mform->createElement('static', 'answerheader', html_writer::tag('p', $label, ['class' => 'qtype-imagechoice-choice-head']));
        // Answer text.
        $repeated[] = $mform->createElement('editor', 'answer', $label, ['rows' => 2], $this->editoroptions);
        // Answer score.
        $repeated[] = $mform->createElement('select', 'fraction', get_string('gradenoun'), $gradeoptions);
        // Answer feedbacks.
        $repeated[] = $mform->createElement('editor', 'feedback',
            get_string('feedback', 'question'), ['rows' => 4], $this->editoroptions);
        $repeatedoptions['answer']['type'] = PARAM_RAW;
        $repeatedoptions['fraction']['default'] = 0;
        $answersoption = 'answers';

        $options = [
            'subdirs' => 0,
            'areamaxbytes' => 10485760,
            'maxfiles' => 1,
            'accepted_types' => ['web_image'],
            'return_types' => FILE_INTERNAL | FILE_EXTERNAL,
        ];

        // Answer image field.
        $repeated[] = $mform->createElement('filemanager', 'answerimage', get_string('answerimage', 'qtype_imagechoice'), 'answer', $options);


        return $repeated;
    }

    /**
     * Data preprocessing to set the answers.
     *
     * @param stdclass $question
     * @return void
     */
    protected function data_preprocessing($question) {

        $question = $this->data_preprocessing_answers($question, true);

        if (!empty($question->options)) {

            $question->single = $question->options->single;
            $question->shuffleanswers = $question->options->shuffleanswers;
            $question->animation = $question->options->animation;

            $question->forcefeedback = $question->options->forcefeedback;
            $question->feedbackduration = $question->options->feedbackduration;
            $question->forcenext = $question->options->forcenext;
        }

        $key = 0;

        if (optional_param('_qf__qtype_imagechoice_edit_form', null, PARAM_INT) == null) {
            foreach ($question->options->answers ?? [] as $answer) {
                $draftitemid = file_get_submitted_draft_itemid('answerimage');
                file_prepare_draft_area(
                    $draftitemid,  $this->context->id, 'qtype_imagechoice',
                    'answerimage', !empty($answer->id) ? (int) $answer->id : null, $this->fileoptions,
                );
                $question->answerimage[$key] = $draftitemid;
                $key++;
            }
        }
        return $question;
    }

    /**
     * Shortname of the image choice question type
     *
     * @return string
     */
    public function qtype() {
        return 'imagechoice';
    }
}
