<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Image choice - Libarary methods to process the pluginfiles and genreate the output of fragments.
 *
 * @package    qtype_multichoice by  Dongsheng Cai <dongsheng@moodle.com>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

defined('MOODLE_INTERNAL') || die();

use mod_quiz\output\navigation_panel_attempt;

/**
 * Checks file access for multiple choice questions.
 *
 * @package  qtype_multichoice
 * @category files
 * @param stdClass $course course object
 * @param stdClass $cm course module object
 * @param stdClass $context context object
 * @param string $filearea file area
 * @param array $args extra arguments
 * @param bool $forcedownload whether or not force download
 * @param array $options additional options affecting the file serving
 * @return bool
 */
function qtype_imagechoice_pluginfile($course, $cm, $context, $filearea, $args, $forcedownload, array $options=array()) {
    global $CFG;
    require_once($CFG->libdir . '/questionlib.php');

    if ($filearea == 'answerimage') {

        // Make sure the user is logged in and has access to the module (plugins that are not course modules should leave out the 'cm' part).
        require_login($course, true, $cm);

        // The args is an array containing [itemid, path].
        // Fetch the itemid from the path.
        $itemid = array_shift($args);

        // Extract the filename / filepath from the $args array.
        $filename = array_pop($args); // The last item in the $args array.
        if (empty($args)) {
            // $args is empty => the path is '/'.
            $filepath = '/';
        } else {
            // $args contains the remaining elements of the filepath.
            $filepath = '/' . implode('/', $args) . '/';
        }

        // Retrieve the file from the Files API.
        $fs = get_file_storage();
        $file = $fs->get_file($context->id, 'qtype_imagechoice', $filearea, $itemid, $filepath, $filename);
        if (!$file) {
            // The file does not exist.
            return false;
        }

        // We can now send the file back to the browser - in this case with a cache lifetime of 1 day and no filtering.
        send_stored_file($file, DAYSECS * 90, 0, $forcedownload, $options);
    }
    question_pluginfile($course, $context, 'qtype_imagechoice', $filearea, $args, $forcedownload, $options);
}

/**
 * Fragment to verify the question is answered complete.
 *
 * @param array $args
 * @return void
 */
function qtype_imagechoice_output_fragment_processattempt(array $args) {
    global $CFG, $PAGE;

    require_once($CFG->dirroot. '/mod/quiz/locallib.php');

    $postdata = [];
    parse_str($args['formdata'], $postdata);

    $prefix = $args['prefix'];

    // Added the submit flag to make the answers saved by the behaviour plugins.
    $postdata[$prefix."-submit"] = true;

    // Attempt id.
    $attemptid = $postdata['attempt'] ?? false;
    $cmid = null;

    $renderer = $PAGE->get_renderer('mod_quiz'); // Quiz render to generate the questions layout with feedback.

    // Create quiz attemt instance for this attemptid.
    if ($attemptid) {
        $attemptobj = quiz_create_attempt_handling_errors($attemptid, $cmid);
        $qubaid = $attemptobj->get_uniqueid();
        $quba = question_engine::load_questions_usage_by_activity($qubaid);
        foreach ($postdata as $key => $value) {
            if (!str_contains($key, $prefix) && str_contains($key, 'q'.$qubaid)) {
                unset($postdata[$key]);
            }
        }

        // Process submitted actions.
        $timenow = time();
        $attemptobj->process_submitted_actions($timenow, false, $postdata);
        foreach (qtype_imagechoice_get_slots_in_request($quba, $postdata) as $slot) {
            $qa = $quba->get_question_attempt($slot);
            if ($qa->get_field_prefix() == $prefix) {
                $result = $attemptobj->render_question($slot, false, $renderer);
            }
        }
    }

    return $result ?? '';
}

/**
 * Update the review navigation page of the quiz.
 *
 * @param array $args
 * @return void
 */
function qtype_imagechoice_output_fragment_update_review_navigation(array $args) {
    global $PAGE, $CFG;

    require_once($CFG->dirroot. '/mod/quiz/locallib.php');

    $attempt = $args['attempt'];
    $nopage = $args['nopage'] ?: false;
    $attemptobj = quiz_create_attempt_handling_errors($attempt);
    $output = $PAGE->get_renderer('mod_quiz');
    $navbc = $attemptobj->get_navigation_panel($output, navigation_panel_attempt::class, !$nopage ? $attemptobj->get_currentpage() : -1);
    return $navbc->content;
}

/**
 * Get the list of slot numbers that should be processed as part of processing
 * the current request.
 * @param array $postdata optional, only intended for testing. Use this data
 * instead of the data from $_POST.
 * @return array of slot numbers.
 */
function qtype_imagechoice_get_slots_in_request($quba, $postdata = null) {
    // Note: we must not use "question_attempt::get_submitted_var()" because there is no attempt instance!!!
    if (is_null($postdata)) {
        $slots = optional_param('slots', null, PARAM_SEQUENCE);
    } else if (array_key_exists('slots', $postdata)) {
        $slots = clean_param($postdata['slots'], PARAM_SEQUENCE);
    } else {
        $slots = null;
    }
    if (is_null($slots)) {
        $slots = $quba->get_slots();
    } else if (!$slots) {
        $slots = array();
    } else {
        $slots = explode(',', $slots);
    }
    return $slots;
}




