<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * The questiontype class for the image choice question type.
 *
 * @package    qtype_imagechoice
 * @copyright  2024 LMSACE Dev Team <lmsace.com>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

defined('MOODLE_INTERNAL') || die();

// Multichoice question type question class.
require_once($CFG->dirroot . '/question/type/multichoice/question.php');

/**
 * Image choice question types options and method to fetch the answer images.
 * Used in the qtype_imagechoice question instance.
 */
trait qtype_imagechoice_question {

    /**
     * Image choice option to supports single or multiple choice question.
     *
     * @var int
     */
    public $single;

    /**
     * Image choice option for shuffle the answers.
     *
     * @var int
     */
    public $shuffleanswers;

    /**
     * Image choice option of entry animations.
     *
     * @var string
     */
    public $animation;

    /**
     * Image choice option to force the feedback
     *
     * @var int
     */
    public $forcefeedback;

    /**
     * Image choice option for duration of the feedback to force the users to listner.
     *
     * @var int
     */
    public $feedbackduration;

    /**
     * Option to hide or disable the next button.
     *
     * @var bool
     */
    public $forcenext;

    /**
     * Answer images.
     *
     * @var string
     */
    public $answerimage;

    /**
     * Fetch the answer image.
     *
     * @param integer $ansid
     * @return void
     */
    public function get_answer_image(int $ansid) {
        $fs = get_file_storage();

        $files = $fs->get_area_files($this->contextid, 'qtype_imagechoice', 'answerimage', $ansid, "", false);

        if (!empty($files)) {
            $file = reset($files);

            $url = \moodle_url::make_pluginfile_url(
                $file->get_contextid(), $file->get_component(),
                $file->get_filearea(),
                $file->get_itemid(),
                $file->get_filepath(),
                $file->get_filename()
            );
        }

        return $url ?? false;
    }

    /**
     * Use the immediatefeedback behaviour as default feedback.
     *
     * @param question_attempt $qa
     * @param [type] $preferredbehaviour
     * @return void
     */
    public function make_behaviour(question_attempt $qa, $preferredbehaviour) {
        return question_engine::make_behaviour('immediatefeedback', $qa, $preferredbehaviour);
    }
}

/**
 * Represents a image choice question where single choices can be selected.
 *
 * @copyright  2024 LMSACE <lmsace.com>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class qtype_imagechoice_single_question extends qtype_multichoice_single_question {

    use qtype_imagechoice_question;

    /**
     * Renderer of the question type image choice.
     *
     * @param moodle_page $page
     * @return qtype_imagechoice_renderer_base
     */
    public function get_renderer(moodle_page $page) {
        return $page->get_renderer('qtype_imagechoice', 'single');
    }

}

/**
 * Represents a image choice question where multiple choices can be selected.
 *
 * @copyright  2024 LMSACE <lmsace.com>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class qtype_imagechoice_multi_question extends qtype_multichoice_multi_question {

    use qtype_imagechoice_question;

    /**
     * Renderer of the question type image choice.
     *
     * @param moodle_page $page
     * @return qtype_imagechoice_renderer_base
     */
    public function get_renderer(moodle_page $page) {
        return $page->get_renderer('qtype_imagechoice', 'multi');
    }

}
