<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * The questiontype class for the image choice question type.
 *
 * @package    qtype_imagechoice
 * @copyright  2024 LMSACE <lmsace.com>
 * @copyright  Modifed version of qtype_multicoice by 1999 onwards Martin Dougiamas  {@link http://moodle.com}
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */


defined('MOODLE_INTERNAL') || die();

global $CFG;

require_once($CFG->libdir . '/questionlib.php');
require_once($CFG->dirroot . '/question/type/multichoice/questiontype.php');

/**
 * The Image choice question type.
 */
class qtype_imagechoice extends question_type {

    /**
     * Extra question fields.
     *
     * @return void
     */
    public function extra_question_fields() {
        return ['qtype_imagechoice_options', 'single', 'shuffleanswers', 'animation', 'forcefeedback', 'feedbackduration', 'forcenext'];
    }

    /**
     * Return $answer, filling necessary fields for the question_answers table.
     *
     * The questions using question_answers table may want to overload this. Default code will work
     * for shortanswer and similar question types.
     * @param stdClass $answer Object to save data.
     * @param object $questiondata This holds the information from the question editing form or import.
     * @param int $key A key of the answer in question.
     * @param object $context needed for working with files.
     * @return $answer answer with filled data.
     */
    public function fill_answer_fields($answer, $questiondata, $key, $context) {

        $answerdata = $questiondata->answer[$key];

        $answer->answer   = $this->import_or_save_files($answerdata, $context, 'question', 'answer', $answer->id);
        $answer->answerformat = $answerdata['format'];
        $answer->fraction = $questiondata->fraction[$key];
        $answer->feedback = $this->import_or_save_files($questiondata->feedback[$key],
                $context, 'question', 'answerfeedback', $answer->id);
        $answer->feedbackformat = $questiondata->feedback[$key]['format'];

        // Store the answer image.
        $draftitemid = $questiondata->answerimage[$key];

        if (!empty($draftitemid)) {
            // This is the normal case. We are safing the questions editing form.
            file_save_draft_area_files($draftitemid, $context->id, 'qtype_imagechoice',
                    'answerimage', $answer->id, $this->fileoptions);

        }

        return $answer;
    }

    /**
     * Returns true is answer with the $key is empty in the question data and should not be saved in DB.
     *
     * The questions using question_answers table may want to overload this. Default code will work
     * for shortanswer and similar question types.
     * @param object $questiondata This holds the information from the question editing form or import.
     * @param int $key A key of the answer in question.
     * @return bool True if answer shouldn't be saved in DB.
     */
    protected function is_answer_empty($questiondata, $key) {
        return trim($questiondata->answer[$key]['text']) == '' && $questiondata->fraction[$key] == 0 &&
                    html_is_blank($questiondata->feedback[$key]['text']);
    }

    /**
     * Saves question-type specific options
     *
     * This is called by {@see save_question()} to save the question-type specific data
     *
     * @param object $question This holds the information from the editing form,
     *      it is not a standard question object.
     * @return bool|stdClass $result->error or $result->notice
     */
    public function save_question_options($question) {
        // Save the questions additional options related to the type specific.
        parent::save_question_options($question);

        // Save the answers data.
        $this->save_question_answers($question);
    }

    /**
     * Get question options.
     *
     * @param [type] $question
     * @return void
     */
    public function get_question_options($question) {
        global $DB, $OUTPUT;

        $question->options = $DB->get_record('qtype_imagechoice_options', ['questionid' => $question->id]);

        if ($question->options === false) {
            // If this has happened, then we have a problem.
            // For the user to be able to edit or delete this question, we need options.
            debugging("Question ID {$question->id} was missing an options record. Using default.", DEBUG_DEVELOPER);

            $question->options = $this->create_default_options($question);
        }

        parent::get_question_options($question);
    }


    /**
     * Create a default options object for the provided question.
     *
     * @param object $question The queston we are working with.
     * @return object The options object.
     */
    protected function create_default_options($question) {
        // Create a default question options record.
        $options = new stdClass();
        $options->questionid = $question->id;

        $config = get_config('qtype_imagechoice');
        $options->single = $config->answerhowmany;
        $options->shuffleanswers = $config->shuffleanswers;
        $options->animation = $config->animation;
        $options->forcefeedback = $config->forcefeedback ?? 0;
        $options->feedbackduration =  $config->feedbackduration ?? '';
        $options->forcenext =  $config->forcenext ?? 0;

        return $options;
    }

    /**
     * Creating the question instance based on the single or multiple answer type.
     *
     * @param [type] $questiondata
     * @return void
     */
    protected function make_question_instance($questiondata) {
        question_bank::load_question_definition_classes($this->name());
        if ($questiondata->options->single) {
            $class = 'qtype_imagechoice_single_question';
        } else {
            $class = 'qtype_imagechoice_multi_question';
        }
        return new $class();
    }

    /**
     * Update the questions options to the question.
     *
     * @param question_definition $question
     * @param [type] $questiondata
     * @return void
     */
    protected function initialise_question_instance(question_definition $question, $questiondata) {

        parent::initialise_question_instance($question, $questiondata);

        $question->shuffleanswers = $questiondata->options->shuffleanswers;
        $question->animation = $questiondata->options->animation ?? '';
        $question->forcefeedback = $questiondata->options->forcefeedback ?? '';
        $question->feedbackduration = $questiondata->options->feedbackduration ?? '';
        $question->forcenext = $questiondata->options->forcenext ?? '';

        $this->initialise_question_answers($question, $questiondata, false);
    }

    /**
     * Modifed version of qtype_multichoice.
     * not originally needed in imagechoice, But it cause issue using features of multichoice.
     *
     * @param [type] $answer
     * @return void
     */
    public function make_answer($answer) {
        // Overridden just so we can make it public for use by question.php.
        return parent::make_answer($answer);
    }

    /**
     * Delete the image choice options data when the question is deleted.
     *
     * @param int $questionid
     * @param int $contextid
     * @return void
     */
    public function delete_question($questionid, $contextid) {
        global $DB;

        $DB->delete_records('qtype_imagechoice_options', array('questionid' => $questionid));

        parent::delete_question($questionid, $contextid);
    }

    /**
     * Generate the random scores.
     *  * Modified verion of qtype_multichoice
     *
     * @param [type] $questiondata
     * @return void
     */
    public function get_random_guess_score($questiondata) {
        if (!$questiondata->options->single) {
            // Pretty much impossible to compute for _multi questions. Don't try.
            return null;
        }

        if (empty($questiondata->options->answers)) {
            // A multi-choice question with no choices is senseless,
            // but, seemingly, it can happen (presumably as a side-effect of bugs).
            // Therefore, ensure it does not lead to errors here.
            return null;
        }

        // Single choice questions - average choice fraction.
        $totalfraction = 0;
        foreach ($questiondata->options->answers as $answer) {
            $totalfraction += $answer->fraction;
        }
        return $totalfraction / count($questiondata->options->answers);
    }

    /**
     * Extended form qtype_multichoice.
     *
     * @param [type] $questiondata
     * @return void
     */
    public function get_possible_responses($questiondata) {
        if ($questiondata->options->single) {
            $responses = array();

            foreach ($questiondata->options->answers as $aid => $answer) {
                $responses[$aid] = new question_possible_response(
                        question_utils::to_plain_text($answer->answer, $answer->answerformat),
                        $answer->fraction);
            }

            $responses[null] = question_possible_response::no_response();
            return array($questiondata->id => $responses);
        } else {
            $parts = array();

            foreach ($questiondata->options->answers as $aid => $answer) {
                $parts[$aid] = array($aid => new question_possible_response(
                        question_utils::to_plain_text($answer->answer, $answer->answerformat),
                        $answer->fraction));
            }

            return $parts;
        }
    }

    public function move_files($questionid, $oldcontextid, $newcontextid) {
        parent::move_files($questionid, $oldcontextid, $newcontextid);
        $this->move_files_in_answers($questionid, $oldcontextid, $newcontextid, true);
    }

     /**
     * Move all the files belonging to this question's answers when the question
     * is moved from one context to another.
     * @param int $questionid the question being moved.
     * @param int $oldcontextid the context it is moving from.
     * @param int $newcontextid the context it is moving to.
     * @param bool $answerstoo whether there is an 'answer' question area,
     *      as well as an 'answerfeedback' one. Default false.
     */
    protected function move_files_in_answers($questionid, $oldcontextid,
            $newcontextid, $answerstoo = false) {
        global $DB;
        $fs = get_file_storage();

        $answerids = $DB->get_records_menu('question_answers',
                array('question' => $questionid), 'id', 'id,1');
        foreach ($answerids as $answerid => $notused) {
            if ($answerstoo) {
                $fs->move_area_files_to_new_context($oldcontextid,
                        $newcontextid, 'question', 'answer', $answerid);
                // Move the answer image files.
                $fs->move_area_files_to_new_context($oldcontextid,
                        $newcontextid, 'qtype_imagechoice', 'answerimage', $answerid);
            }
            $fs->move_area_files_to_new_context($oldcontextid,
                    $newcontextid, 'question', 'answerfeedback', $answerid);
        }
    }

    /**
     * Delete the files used in the question and answers.
     *
     * @param [type] $questionid
     * @param [type] $contextid
     * @return void
     */
    protected function delete_files($questionid, $contextid) {
        parent::delete_files($questionid, $contextid);

        $this->delete_files_in_answers($questionid, $contextid, true);
    }

    /**
     * Delete all the files belonging to this question's answers.
     * @param int $questionid the question being deleted.
     * @param int $contextid the context the question is in.
     * @param bool $answerstoo whether there is an 'answer' question area,
     *      as well as an 'answerfeedback' one. Default false.
     */
    protected function delete_files_in_answers($questionid, $contextid, $answerstoo = false) {
        global $DB;
        $fs = get_file_storage();

        $answerids = $DB->get_records_menu('question_answers',
                array('question' => $questionid), 'id', 'id,1');
        foreach ($answerids as $answerid => $notused) {
            if ($answerstoo) {
                $fs->delete_area_files($contextid, 'question', 'answer', $answerid);
                // Delete the image choice answer images.
                $fs->delete_area_files($contextid, 'qtype_imagechoice', 'answerimage', $answerid);
            }
            $fs->delete_area_files($contextid, 'question', 'answerfeedback', $answerid);
        }
    }
}
