<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Multiple choice question renderer classes.
 *
 * @package    qtype_imagechoice
 * @copyright  2024 LMSACE Dev Team <lmsace.com>
 * @copyright  Modified version of qtype_multichoice by 2009 The Open University
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

use mod_quiz\quiz_attempt;
use qtype_imagechoice\nct_license;

defined('MOODLE_INTERNAL') || die();

require_once($CFG->dirroot. "/question/type/multichoice/questiontype.php");

/**
 * Base class for generating the bits of output common to image choice
 * single and multiple questions.
 */
abstract class qtype_imagechoice_renderer_base extends qtype_with_combined_feedback_renderer {

    /**
     * Method to generating the bits of output after question choices.
     *
     * @param question_attempt $qa The question attempt object.
     * @param question_display_options $options controls what should and should not be displayed.
     *
     * @return string HTML output.
     */
    abstract protected function after_choices(question_attempt $qa, question_display_options $options);

    abstract protected function get_input_type();

    abstract protected function get_input_name(question_attempt $qa, $value);

    abstract protected function get_input_value($value);

    abstract protected function get_input_id(question_attempt $qa, $value);

    /**
     * Whether a choice should be considered right, wrong or partially right.
     * @param question_answer $ans representing one of the choices.
     * @return float 1.0, 0.0 or something in between, respectively.
     */
    abstract protected function is_right(question_answer $ans);

    abstract protected function prompt();

    /**
     * Return any HTML that needs to be included in the page's <head> when this
     * question is used.
     * @param $qa the question attempt that will be displayed on the page.
     * @return string HTML fragment.
     */
    public function head_code(question_attempt $qa) {
        global $PAGE, $CFG;

        $PAGE->add_body_class("qtype-imagechoice-question-page");

        $PAGE->requires->css('/question/type/imagechoice/animate.css');
    }

    /**
     * Formulation of the question text with answers.
     *
     * Prepare answers list and render from the template.
     *
     * @param question_attempt $qa
     * @param question_display_options $options
     * @return void
     */
    public function formulation_and_controls(question_attempt $qa, question_display_options $options) {
        global $SESSION;

        if (!nct_license::is_available()) {
            $data = [
                'notactivated' => get_string('nct:activatlicense', 'qtype_imagechoice')
            ];
            return $this->render_from_template('qtype_imagechoice/question', $data);
        }

        $ispreview = str_contains($this->page->url->get_path(), 'preview');

        $question = $qa->get_question();
        $response = $question->get_response($qa);

        $usageid = $qa->get_usage_id();

        try {
            $attempt = quiz_attempt::create_from_usage_id($usageid);
        } catch (Exception $e) {
            $attempt = false;
        }

        $inputname = $qa->get_qt_field_name('answer');
        $inputattributes = array(
            'type' => $this->get_input_type(),
            'name' => $inputname,
        );

        if ($options->readonly) {
            $inputattributes['disabled'] = 'disabled';
        }

        $choices = [];

        // Animation to reveal the answers and reveal the feedback.
        $entryanimation = $question->animation ?: 'fadeIn';
        $animation = $qa->get_state() != question_state::$todo ? 'animate__flipInY' : 'animate__'.$entryanimation;

        // Overrides the config of review options.
        if ($qa->get_behaviour_name() == 'immediatefeedback' && $options->readonly == 1) {
           // TODO: If needed override the display options here.
        }

        foreach ($question->get_order($qa) as $value => $ansid) {

            $ans = $question->answers[$ansid];

            $inputattributes['name'] = $this->get_input_name($qa, $value);
            $inputattributes['value'] = $this->get_input_value($value);
            $inputattributes['id'] = $this->get_input_id($qa, $value);
            $inputattributes['aria-labelledby'] = $inputattributes['id'] . '_label';
            $inputattributes['class'] = 'qtype-imagechoice-answer-radio'; // Custom class to use for Js.
            $isselected = $question->is_choice_selected($response, $value);
            if ($isselected) {
                $inputattributes['checked'] = 'checked';
            } else {
                unset($inputattributes['checked']);
            }

            // If the question is answered and this answer is not selected one then no need to add animation.
            $noanimation = !$isselected && $qa->get_state() != question_state::$todo;

            $hidden = '';
            $feedback = '';
            $feedbackimg = '';

            if (!$options->readonly && $this->get_input_type() == 'checkbox') {

                $hidden .= html_writer::empty_tag('input', array(
                    'type' => 'hidden',
                    'name' => $inputattributes['name'],
                    'value' => 0,
                ));
            }

            $choicetext = $question->format_text($ans->answer, $ans->answerformat, $qa, 'question', 'answer', $ansid);

            $choiceimage = $question->get_answer_image($ansid);

            // Check the support_legacy_review_options_hack method exisits.
            if(method_exists('qtype_multichoice', 'support_legacy_review_options_hack')) {
                qtype_multichoice::support_legacy_review_options_hack($options);
            };

            if (defined('qtype_multichoice::COMBINED_BUT_NOT_CHOICE_FEEDBACK')) {
                $choicefeedback = $options->feedback !== qtype_multichoice::COMBINED_BUT_NOT_CHOICE_FEEDBACK;
            } else {
                $choicefeedback = property_exists($options, 'suppresschoicefeedback') ? $options->suppresschoicefeedback : true;
            }

            if ($options->feedback && $choicefeedback && trim($ans->feedback)) {

                $feedback = html_writer::tag('div',
                    $question->make_html_inline(
                        $question->format_text($ans->feedback, $ans->feedbackformat, $qa, 'question', 'answerfeedback', $ansid)
                    ),
                    array('class' => 'specificfeedback')
                );
            }

            $class = 'r' . ($value % 2);
            if ($options->correctness) {
                // Feedback images will be rendered using Font awesome.
                // Font awesome icons are actually characters(text) with special glyphs,
                // so the icons cannot be aligned correctly even if the parent div wrapper is using align-items: flex-start.
                // To make the Font awesome icons follow align-items: flex-start, we need to wrap them inside a span tag.
                $feedbackimg = html_writer::span($this->feedback_image($this->is_right($ans)), 'ml-1');
                $feedbackclass = $this->feedback_class($this->is_right($ans));
                $class .= ' ' . $feedbackclass;
            }

            $choices[] = [
                'hiddeninput' => $hidden,
                'choiceinput' => html_writer::empty_tag('input', $inputattributes),
                'choicetext' => $choicetext,
                'choiceimage' => $choiceimage,
                'text' => $ans->answer,
                'inputattrs' => $inputattributes,
                'class' => $class,
                'feedback' => $feedback,
                'feedbackstr' => isset($feedbackclass) ? get_string($feedbackclass, 'question') : '',
                'feedbackimg' => $feedbackimg,
                'ansid' => $ansid,
                'animation' => !$attempt || (!$attempt->is_finished() && !$noanimation) ? $animation : 'noanimation',
            ];
        }

        if ($question->showstandardinstruction == 1) {
            $legendclass = '';
            $questionnumber = $options->add_question_identifier_to_label($this->prompt(), true, true);
        } else {
            $questionnumber = $options->add_question_identifier_to_label(get_string('answer'), true, true);
            $legendclass = 'sr-only';
        }
        $legendattrs = [
            'class' => 'prompt h6 font-weight-normal ' . $legendclass,
        ];

        $data = [
            'questiontext' => $question->format_questiontext($qa),
            'legend' => html_writer::tag('legend', $questionnumber, $legendattrs),
            'choices' => $choices,
            'afterchoices' => $this->after_choices($qa, $options),
            'width' => (100 / count($choices)),
            'fieldprefix' => $qa->get_field_prefix(),
            'revealfeedback' => $qa->get_state() != question_state::$todo && $options->feedback,
            'isfinished' => $attempt && $attempt->is_finished(),
            'multianswer' => !$question->single,
            'isanswered' => $qa->get_state() != question_state::$todo,
            'ispreview' => $ispreview,
            'feedbackaudio' => $question->forcefeedback == 1,
        ];

        // JS options.
        $options = [
            'multianswer' => !$question->single,
            'forcefeedback' => $question->forcefeedback,
            'feedbackduration' => $question->feedbackduration,
            'forcenext' => $question->forcenext,
        ];

        // Load JS module for the question answers.
        $this->page->requires->js_call_amd('qtype_multichoice/answers', 'init',
            [$qa->get_outer_question_div_unique_id()]);


        $this->page->requires->js_call_amd('qtype_imagechoice/imagechoice', 'init',
            [$qa->get_outer_question_div_unique_id(), $question->contextid, $options]);

        if ($qa->get_state() == question_state::$invalid) {
            $data['validationerror'] = html_writer::nonempty_tag('div',
                    $question->get_validation_error($qa->get_last_qt_data()),
                    array('class' => 'validationerror'));
        }

        return $this->render_from_template('qtype_imagechoice/question', $data);
    }

    public function specific_feedback(question_attempt $qa) {
        return $this->combined_feedback($qa);
    }

    /**
     * Function returns string based on number of correct answers
     * @param array $right An Array of correct responses to the current question
     * @return string based on number of correct responses
     */
    protected function correct_choices(array $right) {
        // Return appropriate string for single/multiple correct answer(s).
        if (count($right) == 1) {
                return get_string('correctansweris', 'qtype_multichoice',
                        implode(', ', $right));
        } else if (count($right) > 1) {
                return get_string('correctanswersare', 'qtype_multichoice',
                        implode(', ', $right));
        } else {
                return "";
        }
    }
}


/**
 * Subclass for generating the bits of output specific to multiple choice
 * single questions.
 *
 * @copyright  2024 LMSACE Dev Team <lmsace.com>
 * @copyright  Modified verion of qtype_multichoice by 2009 The Open University
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class qtype_imagechoice_single_renderer extends qtype_imagechoice_renderer_base {

    protected function get_input_type() {
        return 'radio';
    }

    protected function get_input_name(question_attempt $qa, $value) {
        return $qa->get_qt_field_name('answer');
    }

    protected function get_input_value($value) {
        return $value;
    }

    protected function get_input_id(question_attempt $qa, $value) {
        return $qa->get_qt_field_name('answer' . $value);
    }

    protected function is_right(question_answer $ans) {
        return $ans->fraction;
    }

    protected function prompt() {
        return get_string('selectone', 'qtype_multichoice');
    }

    public function correct_response(question_attempt $qa) {
        $question = $qa->get_question();

        // Put all correct answers (100% grade) into $right.
        $right = array();
        foreach ($question->answers as $ansid => $ans) {
            if (question_state::graded_state_for_fraction($ans->fraction) ==
                    question_state::$gradedright) {
                $right[] = $question->make_html_inline($question->format_text($ans->answer, $ans->answerformat,
                        $qa, 'question', 'answer', $ansid));
            }
        }
        return $this->correct_choices($right);
    }

    public function after_choices(question_attempt $qa, question_display_options $options) {
        // Only load the clear choice feature if it's not read only.
        if ($options->readonly) {
            return '';
        }

        $question = $qa->get_question();
        $response = $question->get_response($qa);
        $hascheckedchoice = false;
        foreach ($question->get_order($qa) as $value => $ansid) {
            if ($question->is_choice_selected($response, $value)) {
                $hascheckedchoice = true;
                break;
            }
        }

        $clearchoiceid = $this->get_input_id($qa, -1);
        $clearchoicefieldname = $qa->get_qt_field_name('clearchoice');
        $clearchoiceradioattrs = [
            'type' => $this->get_input_type(),
            'name' => $qa->get_qt_field_name('answer'),
            'id' => $clearchoiceid,
            'value' => -1,
            'class' => 'sr-only',
            'aria-hidden' => 'true'
        ];
        $clearchoicewrapperattrs = [
            'id' => $clearchoicefieldname,
            'class' => 'qtype_multichoice_clearchoice',
        ];

        // When no choice selected during rendering, then hide the clear choice option.
        // We are using .sr-only and aria-hidden together so while the element is hidden
        // from both the monitor and the screen-reader, it is still tabbable.
        $linktabindex = 0;
        if (!$hascheckedchoice && $response == -1) {
            $clearchoicewrapperattrs['class'] .= ' sr-only';
            $clearchoicewrapperattrs['aria-hidden'] = 'true';
            $clearchoiceradioattrs['checked'] = 'checked';
            $linktabindex = -1;
        }
        // Adds an hidden radio that will be checked to give the impression the choice has been cleared.
        $clearchoiceradio = html_writer::empty_tag('input', $clearchoiceradioattrs);
        $clearchoice = html_writer::link('#', get_string('clearchoice', 'qtype_multichoice'),
            ['tabindex' => $linktabindex, 'role' => 'button', 'class' => 'btn btn-link ml-3 mt-n1']);
        $clearchoiceradio .= html_writer::label($clearchoice, $clearchoiceid);

        // Now wrap the radio and label inside a div.
        $result = html_writer::tag('div', $clearchoiceradio, $clearchoicewrapperattrs);

        // Load required clearchoice AMD module.
        $this->page->requires->js_call_amd('qtype_multichoice/clearchoice', 'init',
            [$qa->get_outer_question_div_unique_id(), $clearchoicefieldname]);

        return $result;
    }

}


/**
 * Subclass for generating the bits of output specific to image choice
 * multi=select questions.
 *
 * @copyright  2024 LMSACE Dev Team <lmsace.com>
 * @copyright  Modified verion of qtype_multichoice by 2009 The Open University
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class qtype_imagechoice_multi_renderer extends qtype_imagechoice_renderer_base {

    protected function after_choices(question_attempt $qa, question_display_options $options) {
        return '';
    }

    protected function get_input_type() {
        return 'checkbox';
    }

    protected function get_input_name(question_attempt $qa, $value) {
        return $qa->get_qt_field_name('choice' . $value);
    }

    protected function get_input_value($value) {
        return 1;
    }

    protected function get_input_id(question_attempt $qa, $value) {
        return $this->get_input_name($qa, $value);
    }

    protected function is_right(question_answer $ans) {
        if ($ans->fraction > 0) {
            return 1;
        } else {
            return 0;
        }
    }

    protected function prompt() {
        return get_string('selectmulti', 'qtype_multichoice');
    }

    public function correct_response(question_attempt $qa) {
        $question = $qa->get_question();

        $right = array();
        foreach ($question->answers as $ansid => $ans) {
            if ($ans->fraction > 0) {
                $right[] = $question->make_html_inline($question->format_text($ans->answer, $ans->answerformat,
                        $qa, 'question', 'answer', $ansid));
            }
        }
        return $this->correct_choices($right);
    }

    protected function num_parts_correct(question_attempt $qa) {
        if ($qa->get_question()->get_num_selected_choices($qa->get_last_qt_data()) >
                $qa->get_question()->get_num_correct_choices()) {
            return get_string('toomanyselected', 'qtype_multichoice');
        }

        return parent::num_parts_correct($qa);
    }
}
