<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Defines the editing form for the image timeline question type.
 *
 * @package    qtype_imagetimeline
 * @copyright  2024 LMSACE <lmsace.com>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

defined('MOODLE_INTERNAL') || die();

/**
 * Image timeline editing form definition.
 */
class qtype_imagetimeline_edit_form extends question_edit_form {

    public $fileoptions = array(
        'subdirs' => true,
        'maxfiles' => 1,
        'maxbytes' => 0,
    );

    /**
     * Number of minimum answers for a question.
     */
    const QUESTION_NUMANS_START = 2;

    /**
     * The default starting number of drop zones.
     */
    const START_NUM_ITEMS = 2;

    protected $answerrepeats = 0;


    /**
     * Add question-type specific form fields.
     *
     * @param object $mform the form being built.
     */
    protected function definition_inner($mform) {
        global $PAGE;

        // Shuffle answers.
        $mform->addElement('advcheckbox', 'shuffleanswers',
            get_string('shuffleanswers', 'qtype_imagetimeline'), null, null, array(0, 1));

        $mform->addHelpButton('shuffleanswers', 'shuffleanswers', 'qtype_imagetimeline');
        $mform->setDefault('shuffleanswers', $this->get_default_value('shuffleanswers', get_config('qtype_imagetimeline', 'shuffleanswers')));

        // Animation.
        $options = [
            'fadeIn' => get_string('fadein', 'qtype_imagetimeline'),
            'fadeInUp' => get_string('fadeinup', 'qtype_imagetimeline'),
            'fadeInDown' => get_string('fadeindown', 'qtype_imagetimeline'),
            'fadeInLeft' => get_string('fadeinleft', 'qtype_imagetimeline'),
            'fadeInRight' => get_string('fadeinright', 'qtype_imagetimeline'),
            'backInUp' => get_string('backinup', 'qtype_imagetimeline'),
            'slideInUp' => get_string('slideinup', 'qtype_imagetimeline'),
            'slideInRight' => get_string('slideinright', 'qtype_imagetimeline'),
            'slideInLeft' => get_string('slideinleft', 'qtype_imagetimeline'),
            'slideInDown' => get_string('slideindown', 'qtype_imagetimeline'),
            'zoomIn' => get_string('zoomin', 'qtype_imagetimeline'),
            'zoomInUp' => get_string('zoominup', 'qtype_imagetimeline'),
            'bounceIn' => get_string('bouncein', 'qtype_imagetimeline'),
            'bounceInDown' => get_string('bounceindown', 'qtype_imagetimeline'),
            'bounceInLeft' => get_string('bounceinleft', 'qtype_imagetimeline'),
            'bounceInRight' => get_string('bounceinright', 'qtype_imagetimeline'),
            'bounceInUp' => get_string('bounceinup', 'qtype_imagetimeline'),
        ];
        $mform->addElement('select', 'animation', get_string('animation', 'qtype_imagetimeline'), $options);

        $mform->addElement('selectyesno', 'showstandardinstruction',
            get_string('showstandardinstruction', 'qtype_imagetimeline'), null, null, [0, 1]);
        $mform->addHelpButton('showstandardinstruction', 'showstandardinstruction', 'qtype_imagetimeline');
        $mform->setDefault('showstandardinstruction', $this->get_default_value('showstandardinstruction',
                get_config('qtype_timeline', 'showstandardinstruction')));

        $this->add_per_answer_fields($mform, get_string('choiceno', 'qtype_imagetimeline', '{no}'),
                question_bank::fraction_options_full(), min(2, QUESTION_NUMANS_START), 2);

        $repeats = $this->optional_param('noanswers', min(2, QUESTION_NUMANS_START), PARAM_INT);

        $this->answerrepeats = $repeats;
        list($itemrepeatsatstart, $imagerepeats) = $this->get_drag_item_repeats();
        $this->definition_drop_zones($mform, $imagerepeats);

        $this->add_combined_feedback_fields(true);
        $this->add_interactive_settings(true, true);

        $PAGE->requires->js_call_amd('qtype_imagetimeline/question', 'form', []);
    }

    /**
     * Returns an array of starting number of repeats, and the total number of repeats.
     *
     * @return array
     */
    protected function get_drag_item_repeats() {
        $countimages = 0;
        if (isset($this->question->id)) {
            // print_pre($this->question->options->answers);exit;
            foreach ($this->question->options->answers as $drag) {
                $countimages = max($countimages, $drag->no);
            }
        }

        if (!$countimages) {
            $countimages = $this->answerrepeats ?: self::START_NUM_ITEMS;
        }
        $itemrepeatsatstart = $countimages;

        $imagerepeats = optional_param('noitems', $itemrepeatsatstart, PARAM_INT);
        $addfields = optional_param('additems', false, PARAM_BOOL);
        if ($addfields) {
            $imagerepeats += self::QUESTION_NUMANS_START;
        }
        return array($itemrepeatsatstart, $imagerepeats);
    }

    /**
     * Make and add drop zones to the form.
     *
     * @param object $mform The Moodle form object.
     * @param int $imagerepeats The initial number of repeat elements.
     */
    protected function definition_drop_zones($mform, $imagerepeats) {
        $mform->addElement('header', 'dropzoneheader', get_string('dropzoneheader', 'qtype_'.$this->qtype()));

        $countdropzones = 0;
        if (isset($this->question->id)) {
            foreach ($this->question->options->answers as $drop) {
                $countdropzones = max($countdropzones, $drop->no);
            }
        }

        if (!$countdropzones) {
            $countdropzones = $this->answerrepeats ?: self::START_NUM_ITEMS;
        }
        $dropzonerepeatsatstart = $countdropzones;

        $this->repeat_elements($this->drop_zone($mform, $imagerepeats), $dropzonerepeatsatstart,
                $this->drop_zones_repeated_options(),
                'nodropzone', 'adddropzone', self::QUESTION_NUMANS_START,
                get_string('addmoredropzones', 'qtype_ddimageortext'), true);
    }

    /**
     * Drop zone to place timeline images
     *
     * @param [type] $mform
     * @param [type] $imagerepeats
     * @return void
     */
    protected function drop_zone($mform, $imagerepeats) {

        $grouparray = [];
        $options = [];
        $options[0] = '';
        for ($i = 1; $i <= $imagerepeats; $i += 1) {
            $options[$i] = $i;
        }
        $grouparray[] = $mform->createElement('select', 'choice', get_string('imagechoice', 'qtype_imagetimeline'), $options);
        $grouparray[] = $mform->createElement(
            'text', 'timeline', get_string('label', 'qtype_imagetimeline'), array('size' => 20, 'class' => 'tweakcss'));
        $mform->setType('timeline', PARAM_NOTAGS);
        $dropzone = $mform->createElement('group', 'drops', get_string('droptimeline', 'qtype_imagetimeline', '{no}'), $grouparray);

        return array($dropzone);
    }

    /**
     * Repeated options.
     *
     * @return void
     */
    protected function drop_zones_repeated_options() {
        $repeatedoptions = array();
        // The next two are PARAM_RAW becuase we need to distinguish 0 and ''.
        // We do the necessary validation in the validation method.
        $repeatedoptions['drops[timeline]']['type'] = PARAM_RAW;
        $repeatedoptions['choice']['default'] = '0';
        return $repeatedoptions;
    }

    /**
     * Answer fields.
     *
     * @param moodle_form $mform
     * @param string $label
     * @param array $gradeoptions
     * @param array $repeatedoptions
     * @param string $answersoption
     * @return array
     */
    protected function get_per_answer_fields($mform, $label, $gradeoptions,
            &$repeatedoptions, &$answersoption) {

        $options = [
            'subdirs' => 0,
            'areamaxbytes' => 10485760,
            'maxfiles' => 1,
            'accepted_types' => ['web_image'],
            'return_types' => FILE_INTERNAL | FILE_EXTERNAL,
        ];

        $repeated = array();

        $repeated[] = $mform->createElement('static', 'answerheader', html_writer::tag('p', $label, ['class' => 'qtype-imagetimeline-choice-head']));

        // Answer image field.
        $repeated[] = $mform->createElement('filemanager', 'answerimage', get_string('answerimage', 'qtype_imagetimeline'), 'answer', $options);
        // Answer text.
        $repeated[] = $mform->createElement('text', 'answer', $label, ['rows' => 2]);

        // Answer type.
        $repeatedoptions['answer']['type'] = PARAM_RAW;
        $repeatedoptions['imagename']['type'] = PARAM_RAW;
        $answersoption = 'answers';

        return $repeated;
    }

    /**
     * Data preprocessing to set the answers.
     *
     * @param stdclass $question
     * @return void
     */
    protected function data_preprocessing($question) {

        $question = $this->data_preprocessing_combined_feedback($question, true);
        $question = $this->data_preprocessing_hints($question, true, true);

        $question = $this->data_preprocessing_answers($question, false);

        if (!empty($question->options)) {
            $question->shuffleanswers = $question->options->shuffleanswers;
            $question->drops = array();
            foreach ($question->options->drops as $drop) {
                $question->drops[$drop->no - 1] = array();
                $question->drops[$drop->no - 1]['choice'] = $drop->choice;
                $question->drops[$drop->no - 1]['timeline'] = $drop->timeline;
            }
        }

        $key = 0;
        // In the question.php edit_form set_data method called earlier without procession the submission data.
        // For this reason, the itemid missing error caused by the draft_itemid.
        // To prevent the missing itemid issue added verification.
        if (optional_param('_qf__qtype_imagetimeline_edit_form', null, PARAM_INT) == null) {
            foreach ($question->options->answers ?? [] as $answer) {
                $draftitemid = file_get_submitted_draft_itemid('answerimage');
                file_prepare_draft_area(
                    $draftitemid,  $this->context->id, 'qtype_imagetimeline',
                    'answerimage', !empty($answer->id) ? (int) $answer->id : null, $this->fileoptions,
                );
                $question->answerimage[$key] = $draftitemid;
                $key++;
            }
        }

        return $question;
    }

    /**
     * Shortname of the image timeline question type
     *
     * @return string
     */
    public function qtype() {
        return 'imagetimeline';
    }

    public function validation($data, $files) {
        $errors = parent::validation($data, $files);

        $images = 0;
        $missing = [];
        foreach ($data['answerimage'] as $key => $itemid) {
            if (self::file_uploaded($itemid)) {
                $images++;
            } else {
                $missing[] = $key;
            }
        }

        if ($images < 2) {
            foreach ($missing as $key) {
                $errors["answerimage[$key]"] = get_string('formerror:answerimagecount', 'qtype_imagetimeline');
            }
        }

        $dropscount = 0;
        $choices = [];
        foreach ($data['drops'] as $key => $drop) {
            if ($drop['choice'] && $drop['timeline'] == '') {
                $errors["drops[$key]"] = get_string('formerror:timelinemissing', 'qtype_imagetimeline');
            }

            if ($drop['choice']) {
                $choicekey = $drop['choice'];

                if (in_array($choicekey-1, $missing)) {
                    $errors["drops[$key]"] = get_string('formerror:missingchoiceselected', 'qtype_imagetimeline');
                }

                if (in_array($choicekey, $choices)) {
                    $errors["drops[$key]"] = get_string('formerror:choicealreadyselected', 'qtype_imagetimeline');
                }

                $choices[] = $drop['choice'];
                $dropscount++;
            }
        }

        if ($dropscount < 2) {
            $errors["drops[0]"] = get_string('formerror:dropscount', 'qtype_imagetimeline');
        }


        // print_pre($data);
        // exit;

        return $errors;
    }

    /**
     * Checks to see if a file has been uploaded.
     *
     * @param string $draftitemid The draft id
     * @return bool True if files exist, false if not.
     */
    public static function file_uploaded($draftitemid) {
        $draftareafiles = file_get_drafarea_files($draftitemid);
        do {
            $draftareafile = array_shift($draftareafiles->list);
        } while ($draftareafile !== null && $draftareafile->filename == '.');
        if ($draftareafile === null) {
            return false;
        }
        return true;
    }
}
