<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * The questiontype class for the image timeline question type.
 *
 * @package    qtype_imagetimeline
 * @copyright  2024 LMSACE Dev Team <lmsace.com>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

defined('MOODLE_INTERNAL') || die();

// Multichoice question type question class.
require_once($CFG->dirroot . '/question/type/ddimageortext/questionbase.php');


/**
 * Image timeline question types options and method to fetch the answer images.
 * Used in the qtype_imagetimeline question instance.
 */
class qtype_imagetimeline_question extends qtype_ddtoimage_question_base {


    public $shuffleanswers;

    /**
     * Image timeline option for shuffle the answers.
     *
     * @var int
     */
    public $shufflechoices;

    /**
     * Image timeline option of entry animations.
     *
     * @var string
     */
    public $animation;

    /**
     * Image timeline option for duration of the feedback to force the users to listner.
     *
     * @var int
     */
    public $order;

    /**
     * Option to hide or disable the next button.
     *
     * @var bool
     */
    public $timeline;

    /**
     * Answer images.
     *
     * @var string
     */
    public $answerimage;

    /**
     * List of answers.
     *
     * @var text
     */
    public $answers;

    public $drops;

    public $showstandardinstruction;

    protected $step;

    protected $qa;

    /**
     * Fetch the answer image.
     *
     * @param integer $ansid
     * @return void
     */
    public function get_answer_image(int $ansid) {
        $fs = get_file_storage();

        $files = $fs->get_area_files($this->contextid, 'qtype_imagetimeline', 'answerimage', $ansid, "", false);

        if (!empty($files)) {
            $file = reset($files);

            $url = \moodle_url::make_pluginfile_url(
                $file->get_contextid(), $file->get_component(),
                $file->get_filearea(),
                $file->get_itemid(),
                $file->get_filepath(),
                $file->get_filename()
            );
        }

        return $url ?? false;
    }

        /**
     * Use the immediatefeedback behaviour as default feedback.
     *
     * @param question_attempt $qa
     * @param [type] $preferredbehaviour
     * @return void
     */
    public function make_behaviour(question_attempt $qa, $preferredbehaviour) {
        $this->qa = $qa;
        return parent::make_behaviour($qa, $preferredbehaviour);
    }

    /**
     * Helper method used by both {@link start_attempt()} and
     * {@link apply_attempt_state()}.
     * @param int $group the group number.
     * @param array $choiceorder the choices, in order.
     */
    public function set_choiceorder($group, $choiceorder) {
        parent::set_choiceorder($group, $choiceorder);
    }

    public function get_choiceorder($group) {
        return $this->choiceorder[$group] ?? [];
    }

    public function apply_attempt_state(question_attempt_step $step) {
        // print_pre($step);exit;
        $this->step = $step;
        foreach ($this->choices as $group => $choices) {
            $this->set_choiceorder($group, explode(',',
                    $step->get_qt_var('_choiceorder' . $group)));
        }
    }

    public function response_order(array $response) {

        foreach ($this->places as $placeno => $place) {
            if (array_key_exists($this->field($placeno), $response) && $response[$this->field($placeno)]) {
                $selected = $this->get_selected_choice($place->group, $response[$this->field($placeno)]);
                $order[] = $selected->no;
            }
        }

        return isset($order) ? $order : [];
    }

    public function get_random_guess_score() {
        if (empty($this->places)) {
            // Having no places would be nonsensical. However, it used to be possible
            // to create questions like that, so avoid errors in this case.
            return null;
        }

        $accum = 0;
        foreach ($this->places as $place) {
            $accum += 1 / count($this->choices[$place->group]);
        }

        return $accum / count($this->places);
    }


    /**
     * Get the choices in the user attempted order.
     *
     * @param [type] $group
     * @return void
     */
    public function get_attemptordered_choices($order, $group) {
        $choices = array();
        foreach ($order as $choicekey => $choiceid) {
            $choices[$choicekey] = $this->choices[$group][$choiceid];
        }
        return $choices;
    }

    public function get_right_choice_for($placeno) {
        return $this->rightchoices[$placeno] ?? false;
    }

    public function summarise_response(array $response) {

        $allblank = true;
        foreach ($this->places as $placeno => $place) {
            $summariseplace = $place->summarise();

            if (array_key_exists($this->field($placeno), $response) && $response[$this->field($placeno)]) {
                $selected = $this->get_selected_choice($place->group, $response[$this->field($placeno)]);
                if (isset($selected)) {
                    $summarisechoice = $selected->summarise();
                } else {
                    $summarisechoice = get_string('deletedchoice', 'qtype_ddimageortext');
                }
                $allblank = false;
            } else {
                $summarisechoice = '';
            }
            $choices[] = "$summariseplace -> {{$summarisechoice}}";
        }
        if ($allblank) {
            return null;
        }

        return implode(' ', $choices);
    }

    protected function get_selected_choice($group, $choiceno) {
        return isset($this->choices[$group][$choiceno]) ? $this->choices[$group][$choiceno] : null;
    }
}
