<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * The questiontype class for the image timeline question type.
 *
 * @package    qtype_imagetimeline
 * @copyright  2024 LMSACE <lmsace.com>
 * @copyright  Modifed version of qtype_multicoice by 1999 onwards Martin Dougiamas  {@link http://moodle.com}
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */


defined('MOODLE_INTERNAL') || die();

global $CFG;

require_once($CFG->libdir . '/questionlib.php');
require_once($CFG->dirroot . '/question/engine/lib.php');
require_once($CFG->dirroot . '/question/format/xml/format.php');
require_once($CFG->dirroot . '/question/type/gapselect/questiontype.php');
require_once($CFG->dirroot . '/question/type/ddimageortext/questiontypebase.php');

/**
 * The Image timeline question type.
 */
class qtype_imagetimeline extends qtype_ddtoimage_base {

    /**
     * Extra question fields.
     *
     * @return void
     */
    public function extra_question_fields() {
        return [
            'qtype_imagetimeline', 'shuffleanswers', 'animation',
            'correctfeedback', 'partiallycorrectfeedback', 'incorrectfeedback', 'showstandardinstruction',
        ];
    }

    /**
     * Return $answer, filling necessary fields for the question_answers table.
     *
     * The questions using question_answers table may want to overload this. Default code will work
     * for shortanswer and similar question types.
     * @param stdClass $answer Object to save data.
     * @param object $questiondata This holds the information from the question editing form or import.
     * @param int $key A key of the answer in question.
     * @param object $context needed for working with files.
     * @return $answer answer with filled data.
     */
    public function fill_answer_fields($answer, $questiondata, $key, $context) {

        $answerdata = $questiondata->answer[$key];

        $answer->answer   = $questiondata->answer[$key];
        $answer->fraction = 0; // $questiondata->fraction[$key];

        $answer->feedback = '';
        $answer->feedbackformat = 0;

        // Store the answer image.
        $draftitemid = $questiondata->answerimage[$key];

        if (!empty($draftitemid)) {
            // This is the normal case. We are safing the questions editing form.
            file_save_draft_area_files($draftitemid, $context->id, 'qtype_imagetimeline',
                    'answerimage', $answer->id, $this->fileoptions);
        }

        return $answer;
    }

    /**
     * Saves question-type specific options
     *
     * This is called by {@see save_question()} to save the question-type specific data
     *
     * @param object $question This holds the information from the editing form,
     *      it is not a standard question object.
     * @return bool|stdClass $result->error or $result->notice
     */
    public function save_question_options($question) {
        global $DB;

        $context = $question->context;

        $options = new stdClass();
        $options = $this->save_combined_feedback_helper($options, $question, $context, true);
        foreach ($options as $key => $value) {
            $question->$key = $value;
        }

        // Save the extra question fields.
        question_type::save_question_options($question);

        $this->save_hints($question, true);

        // Save the answers data.
        $this->save_question_answers($question);

        $DB->delete_records('qtype_imagetimeline_drops', array('questionid' => $question->id));
        $drops = [];

        foreach (array_keys($question->drops) as $dropno) {
            if (!isset($question->drops[$dropno]['choice'])
                || ($question->drops[$dropno]['choice'] && $question->drops[$dropno]['choice'] == 0)) {
                continue;
            }
            $drop = new stdClass();
            $drop->questionid = $question->id;
            $drop->no = $dropno + 1;
            $drop->choice = $question->drops[$dropno]['choice'];
            $drop->timeline = $question->drops[$dropno]['timeline'];

            $drops[] = $drop;
        }
        $DB->insert_records('qtype_imagetimeline_drops', $drops);
    }

    /**
     * Returns true is answer with the $key is empty in the question data and should not be saved in DB.
     *
     * The questions using question_answers table may want to overload this. Default code will work
     * for shortanswer and similar question types.
     * @param object $questiondata This holds the information from the question editing form or import.
     * @param int $key A key of the answer in question.
     * @return bool True if answer shouldn't be saved in DB.
     */
    protected function is_answer_empty($questiondata, $key) {
        return trim($questiondata->answer[$key]) == '' && $questiondata->answerimage[$key] == '';
    }

    /**
     * Get question options.
     *
     * @param [type] $question
     * @return void
     */
    public function get_question_options($question) {
        global $DB, $OUTPUT;

        $question->options = $DB->get_record('qtype_imagetimeline', ['questionid' => $question->id]);

        question_type::get_question_options($question);

        $question->options->drops = $DB->get_records('qtype_imagetimeline_drops', array('questionid' => $question->id), 'no ASC', '*');


        $no = 1;
        $question->options->answers = array_map(function($answer) use (&$no) {
            // Set the no for the answer.
            $answer->no = $no++;
            return $answer;
        }, $question->options->answers);
    }

    /**
     * Update the questions options to the question.
     *
     * @param question_definition $question
     * @param [type] $questiondata
     * @return void
     */
    protected function initialise_question_instance(question_definition $question, $questiondata) {

        question_type::initialise_question_instance($question, $questiondata);

        $question->shufflechoices = $questiondata->options->shuffleanswers;
        $question->animation = $questiondata->options->animation ?? '';

        $no = 1;

        $this->initialise_combined_feedback($question, $questiondata, true);

        $question->choices = array();
        $choiceindexmap = array();

        // Store the choices in arrays by group.
        $i = 1;

        foreach ($questiondata->options->answers as $k => $choicedata) {
            $choice = $this->make_choice($choicedata);
            $choicegroup = $choice->choice_group();

            if (array_key_exists($choicegroup, $question->choices)) {
                $question->choices[$choicegroup][$i] = $choice;
            } else {
                $question->choices[$choicegroup][1] = $choice;
            }

            end($question->choices[$choice->choice_group()]);
            $choiceindexmap[$i] = array($choicegroup, key($question->choices[$choicegroup]));

            $questiondata->options->answers[$k]->choice = $k;
            $questiondata->options->answers[$k]->no = $i;
            $questiondata->options->answers[$k]->group = 1;

            $i += 1;
        }

        $question->places = array();
        $question->rightchoices = array();
        // Break up the question text, and store the fragments, places and right answers.
        $i = 1;
        foreach ($questiondata->options->drops as $dropdata) {
            if (!array_key_exists($dropdata->choice, $choiceindexmap)) {
                continue;
            }
            list($group, $choiceindex) = $choiceindexmap[$dropdata->choice];
            $choice = $question->choices[$group][$dropdata->no];
            $dropdata->group = $group;
            $question->places[$dropdata->no] = $this->make_place($dropdata);
            $question->rightchoices[$dropdata->no] = $choiceindex;
        }

        $this->initialise_question_answers($question, $questiondata, false);

    }

    /**
     * Create the dropzone as place for drop.
     *
     * @param stdclass $dropzonedata
     * @return \qtype_imagetimeline_drop_zone
     */
    protected function make_place($dropzonedata) {
        return new qtype_imagetimeline_drop_zone(
            $dropzonedata->timeline, $dropzonedata->no, $dropzonedata->group
        );
    }

    /**
     * Make the answer as choice to draggable.
     *
     * @param stdclass $choicedata
     * @return \qtype_imagetimeline_drop_zone
     */
    protected function make_choice($choicedata) {
        return new qtype_timeline_choice(
            $choicedata->answer, 1, $choicedata->id, $choicedata->no);
    }

    /**
     * Delete the image timeline options data when the question is deleted.
     *
     * @param int $questionid
     * @param int $contextid
     * @return void
     */
    public function delete_question($questionid, $contextid) {
        global $DB;

        $DB->delete_records('qtype_imagetimeline', array('questionid' => $questionid));
        $DB->delete_records('qtype_imagetimeline_drops', array('questionid' => $questionid));

        question_type::delete_question($questionid, $contextid);
    }

    /**
     * Move the files
     *
     * @param int $questionid
     * @param int $oldcontextid
     * @param int $newcontextid
     * @return void
     */
    public function move_files($questionid, $oldcontextid, $newcontextid) {
        parent::move_files($questionid, $oldcontextid, $newcontextid);
        $this->move_files_in_answers($questionid, $oldcontextid, $newcontextid, true);
    }

     /**
     * Move all the files belonging to this question's answers when the question
     * is moved from one context to another.
     * @param int $questionid the question being moved.
     * @param int $oldcontextid the context it is moving from.
     * @param int $newcontextid the context it is moving to.
     * @param bool $answerstoo whether there is an 'answer' question area,
     *      as well as an 'answerfeedback' one. Default false.
     */
    protected function move_files_in_answers($questionid, $oldcontextid,
            $newcontextid, $answerstoo = false) {
        global $DB;
        $fs = get_file_storage();

        $answerids = $DB->get_records_menu('question_answers',
                array('question' => $questionid), 'id', 'id,1');
        foreach ($answerids as $answerid => $notused) {
            if ($answerstoo) {
                // Move the answer image files.
                $fs->move_area_files_to_new_context($oldcontextid,
                        $newcontextid, 'qtype_imagetimeline', 'answerimage', $answerid);
            }
        }
    }

    /**
     * Delete the files used in the question and answers.
     *
     * @param [type] $questionid
     * @param [type] $contextid
     * @return void
     */
    protected function delete_files($questionid, $contextid) {
        parent::delete_files($questionid, $contextid);

        $this->delete_files_in_answers($questionid, $contextid, true);
    }

    /**
     * Delete all the files belonging to this question's answers.
     * @param int $questionid the question being deleted.
     * @param int $contextid the context the question is in.
     * @param bool $answerstoo whether there is an 'answer' question area,as well as an 'answerfeedback' one. Default false.
     */
    protected function delete_files_in_answers($questionid, $contextid, $answerstoo = false) {
        global $DB;
        $fs = get_file_storage();

        $answerids = $DB->get_records_menu('question_answers', array('question' => $questionid), 'id', 'id,1');
        foreach ($answerids as $answerid => $notused) {
            if ($answerstoo) {
                // Delete the image timeline answer images.
                $fs->delete_area_files($contextid, 'qtype_imagetimeline', 'answerimage', $answerid);
            }
        }
    }

    /**
     * Export the question to XML format.
     *
     * @param question $question
     * @param qformat_xml $format
     * @param string $extra
     *
     * @return string
     */
    public function export_to_xml($question, qformat_xml $format, $extra = null) {
        $fs = get_file_storage();
        $contextid = $question->contextid;
        $output = '';

        $extraquestionfields = $this->extra_question_fields();
        if (!is_array($extraquestionfields)) {
            return false;
        }

        // Omit table name.
        array_shift($extraquestionfields);
        $expout='';
        $areafields = array('correctfeedback', 'partiallycorrectfeedback', 'incorrectfeedback');
        foreach ($extraquestionfields as $field) {
            $exportedvalue = $format->xml_escape($question->options->$field);
            if (in_array($field, $areafields)) {
                $formatfield = $field . 'format';
                $files = $fs->get_area_files($contextid, 'question', $field, $question->id);

                $expout .= "    <{$field} {$format->format($question->options->$formatfield)}>\n";
                $expout .= '      ' . $format->writetext($question->options->$field);
                $expout .= $format->write_files($files);
                $expout .= "    </{$field}>\n";
            } else {
                $expout .= "    <{$field}>{$exportedvalue}</{$field}>\n";
            }
        }

        foreach ($question->options->answers as $answer) {
            $files = $fs->get_area_files($contextid, 'qtype_imagetimeline', 'answerimage', $answer->id);
            $extra = $this->write_files($files, 3);
            $expout .= $format->write_answer($answer, $extra);
        }

        foreach ($question->options->drops as $drop) {
            $expout .= "    <drop>\n";
            $expout .= $format->writetext($drop->timeline, 3);
            $expout .= "      <no>{$drop->no}</no>\n";
            $expout .= "      <choice>{$drop->choice}</choice>\n";
            $expout .= "    </drop>\n";
        }

        return $expout;
    }

    /*
     * Imports question from the Moodle XML format
     *
     * Imports question using information from extra_question_fields function
     * If some of you fields contains id's you'll need to reimplement this
     */
    public function import_from_xml($data, $question, qformat_xml $format, $extra=null) {
        $question_type = $data['@']['type'];
        if ($question_type != $this->name()) {
            return false;
        }

        $extraquestionfields = $this->extra_question_fields();
        if (!is_array($extraquestionfields)) {
            return false;
        }

        // Omit table name.
        array_shift($extraquestionfields);
        $qo = $format->import_headers($data);
        $qo->qtype = $question_type;

        $areafields = array('correctfeedback', 'partiallycorrectfeedback', 'incorrectfeedback');
        foreach ($extraquestionfields as $field) {
            if (!in_array($field, $areafields)) {
                $qo->$field = $format->getpath($data, array('#', $field, 0, '#'), '');
            }
        }

        // Run through the answers.
        $answers = $data['#']['answer'];
        $acount = 0;
        $extraanswersfields = $this->extra_answer_fields();
        if (is_array($extraanswersfields)) {
            array_shift($extraanswersfields);
        }
        foreach ($answers as $answer) {
            $ans = $format->import_answer($answer);
            if (!$this->has_html_answers()) {
                $qo->answer[$acount] = $ans->answer['text'];
            } else {
                $qo->answer[$acount] = $ans->answer;
            }
            $qo->fraction[$acount] = $ans->fraction;
            $qo->feedback[$acount] = $ans->feedback;
            $filexml = $format->getpath($answer, array('#', 'file'), array());
            $qo->answerimage[$acount] = $format->import_files_as_draft($filexml);

            if (is_array($extraanswersfields)) {
                foreach ($extraanswersfields as $field) {
                    $qo->{$field}[$a_count] =
                        $format->getpath($answer, array('#', $field, 0, '#'), '');
                }
            }

            ++$acount;
        }

        $drops = $data['#']['drop'];
        $qo->drops = array();
        foreach ($drops as $dropxml) {
            $dropno = $format->getpath($dropxml, array('#', 'no', 0, '#'), 0);
            $dropindex = $dropno - 1;
            $qo->drops[$dropindex] = array();
            $qo->drops[$dropindex]['choice'] = $format->getpath($dropxml, array('#', 'choice', 0, '#'), 0);
            $qo->drops[$dropindex]['timeline'] = $format->getpath($dropxml, array('#', 'text', 0, '#'), '', true);
        }

        $format->import_combined_feedback($qo, $data, true);

        return $qo;
    }
}

/**
 * Represents one of the choices (select box option).
 *
 * @copyright  2009 The Open University
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class qtype_timeline_choice {
    /** @var string Text for the choice */
    public $text;

    /** @var int Selection group of the choice */
    public $selectgroup;

    /**
     * Answer id.
     *
     * @var string
     */
    public $ansid;

    /**
     * Name of the image, used as answer text.
     *
     * @var string
     */
    public $imagename;

    /**
     * Choice no in the list.
     *
     * @var int
     */
    public $no;

    /**
     * Creates summary text of for the drag item.
     *
     * @param string $text The text of the choice
     * @param int $selectgroup The select group of the choice
     */
    public function __construct($text, $selectgroup = 1, $ansid = 0, $no=0) {
        $this->text = $text;
        $this->selectgroup = $selectgroup;
        $this->ansid = $ansid ?? '';
        $this->no = $no;
    }

    /**
     * Returns the group of this item.
     *
     * @return int
     */
    public function choice_group() {
        return $this->selectgroup;
    }

    /**
     * Creates summary text of for the drag item.
     *
     * @return string
     */
    public function summarise() {
        if (trim($this->text) != '') {
            return get_string('summarisechoice', 'qtype_ddimageortext', $this);
        } else {
            return get_string('summarisechoiceno', 'qtype_ddimageortext', $this->no);
        }
    }
}

/**
 * Represents one of the places (drop zones).
 *
 * @copyright  2009 The Open University
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class qtype_imagetimeline_drop_zone {
    /** @var int Number of the item */
    public $no;

    /** @var string Alt text for the drop zone item */
    public $text;

    /** @var int Group of the item */
    public $group;


    /**
     * Create a drop zone object.
     *
     * @param string $alttextlabel The alt text of the drop zone
     * @param int $no Which number drop zone this is
     * @param int $group Group of the drop zone
     * @param int $x X location
     * @param int $y Y location
     */
    public function __construct($alttextlabel, $no, $group = 1) {
        $this->no = $no;
        $this->text = $alttextlabel;
        $this->group = $group;
    }

    /**
     * Creates summary text of for the drop zone
     *
     * @return string
     */
    public function summarise() {
        if (trim($this->text) != '') {
            $summariseplace =
                        get_string('summariseplace', 'qtype_ddimageortext', $this);
        } else {
            $summariseplace =
                    get_string('summariseplaceno', 'qtype_ddimageortext', $this->no);
        }
        return $summariseplace;
    }
}
