<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Multiple choice question renderer classes.
 *
 * @package    qtype_imagetimeline
 * @copyright  2024 LMSACE Dev Team <lmsace.com>
 * @copyright  Modified version of qtype_multichoice by 2009 The Open University
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

use mod_quiz\quiz_attempt;

defined('MOODLE_INTERNAL') || die();

require_once($CFG->dirroot . '/question/type/ddimageortext/rendererbase.php');


/**
 * Base class for generating the bits of output common to image timeline
 * single and multiple questions.
 */
class qtype_imagetimeline_renderer extends qtype_ddtoimage_renderer_base {


    /**
     * Return any HTML that needs to be included in the page's <head> when this
     * question is used.
     * @param $qa the question attempt that will be displayed on the page.
     * @return string HTML fragment.
     */
    public function head_code(question_attempt $qa) {
        global $PAGE, $CFG;

        $PAGE->add_body_class("qtype-imagetimeline-question-page");

        $PAGE->requires->css('/question/type/imagetimeline/animate.css');
    }

    /**
     * Formulation of the question text with answers.
     *
     * Prepare answers list and render from the template.
     *
     * @param question_attempt $qa
     * @param question_display_options $options
     * @return void
     */
    public function formulation_and_controls(question_attempt $qa, question_display_options $options) {
        global $SESSION;

        $ispreview = str_contains($this->page->url->get_path(), 'preview');
        $question = $qa->get_question();
        $usageid = $qa->get_usage_id();
        $response = $qa->get_last_qt_data();

        if ($response && $order = $question->response_order($response)) {
            // Some scenatiors need to merge the missing choices which is not configured fully.
            // Previous order.
            $previousorder = $question->get_choiceorder(1);
            $missing = array_diff($previousorder, $order);
            $neworder = array_merge($order, $missing);
            $question->set_choiceorder(1, array_values($neworder));
        }

        $inputname = $qa->get_qt_field_name('answer');
        $inputattributes = array(
            'type' => 'hidden',
            'name' => $inputname,
        );

        if ($options->readonly) {
            $inputattributes['disabled'] = 'disabled';
        }

        $choices = [];

        // Animation to reveal the answers and reveal the feedback.
        $entryanimation = $question->animation ?: 'fadeIn';
        $animation = $qa->get_state() != question_state::$todo ? 'animate__flipInY' : 'animate__'.$entryanimation;

        $orderedgroup = !empty($question->choices) ? $question->get_ordered_choices(1) : [];
        $placeinfoforjsandmobileapp = [];
        $places = [];
        $placeno = 1;
        foreach ($orderedgroup as $value => $choice) {

            $ansid = $choice->ansid;
            $ans = $question->answers[$ansid];

            $choicetext = $choice->text;
            $choiceimage = $question->get_answer_image($ansid);

            if (trim($choicetext) == '' && $choiceimage == '') {
                continue;
            }
            // If the question is answered and this answer is not selected one then no need to add animation.
            $noanimation = $qa->get_state() != question_state::$todo;

            $feedback = '';
            $feedbackimg = '';

            $varname = $question->field($placeno);
            [$fieldname, $placehtml] = $this->hidden_field_for_qt_var($qa, $varname, $choice->no,
                    ['placeinput', 'place' . $placeno, 'group1']);

            $places[] = $placehtml;
            $placeinfo = (object) (array) ($question->places[$placeno] ?? []);
            $placeinfo->fieldname = $fieldname;
            $placeinfoforjsandmobileapp[$placeno] = $placeinfo;

            $classes = [
                'r' . $value,
                'group1',
                'draghome',
                // 'user-select-none',
                'inplace'.$placeno,
                'placed',
                'choice' . $choice->no
            ];

            if ($options->correctness) {
                $response = $qa->get_last_qt_data();
                if (array_key_exists($varname, $response)) {
                    $fraction = (int) ($response[$varname] == $question->get_right_choice_for($placeno));
                    $classes[] = $this->feedback_class($fraction);
                }
            }

            $choices[] = [
                'placehtml' => $placehtml,
                'fieldname' => $fieldname,
                'choiceno' => $choice->no,
                'placeno' => $placeno,
                'place' => $placeinfo->text ?? '',
                'choicetext' => $choicetext,
                'choiceimage' => $choiceimage,
                'text' => $ans->answer,
                'inputattrs' => $inputattributes,
                'class' => implode(' ', $classes),
                'feedback' => $feedback,
                'feedbackstr' => isset($feedbackclass) ? get_string($feedbackclass, 'question') : '',
                'feedbackimg' => $feedbackimg,
                'ansid' => $ansid,
                'animation' => !$noanimation ? $animation : 'noanimation',
            ];

            $placeno++;
        }

        if ($question->showstandardinstruction == 1) {
            $legendclass = '';
            $questionnumber = $options->add_question_identifier_to_label(get_string('instructions', 'qtype_imagetimeline'), true, true);
        } else {
            $questionnumber = "";
            $legendclass = 'sr-only';
        }
        $legendattrs = [
            'class' => 'prompt h6 font-weight-normal ' . $legendclass,
        ];

        $data = [
            'questiontext' => $question->format_questiontext($qa),
            'legend' => html_writer::tag('legend', $questionnumber, $legendattrs),
            'instruction' => $question->showstandardinstruction,
            'choices' => $choices,
            'places' => $places,
            'fieldprefix' => $qa->get_field_prefix(),
            'revealfeedback' => $qa->get_state() != question_state::$todo && $options->feedback,
            'isanswered' => $qa->get_state() != question_state::$todo,
            'ispreview' => $ispreview,
        ];

        // JS options.
        $jsoptions = [
            'multianswer' => !$question->single,
            'forcefeedback' => $question->forcefeedback,
            'feedbackduration' => $question->feedbackduration,
            'forcenext' => $question->forcenext,
        ];

        // Load JS module for the question answers.
        if (!$options->readonly) {
            $this->page->requires->js_call_amd('qtype_imagetimeline/question', 'init',
                [$qa->get_outer_question_div_unique_id(), $options->readonly, $placeinfoforjsandmobileapp]);
        }

        $this->page->requires->js_call_amd('qtype_imagetimeline/imagetimeline', 'init',
            [$qa->get_outer_question_div_unique_id(), $question->contextid, $jsoptions]);

        if ($qa->get_state() == question_state::$invalid) {
            $data['validationerror'] = html_writer::nonempty_tag('div',
                $question->get_validation_error($qa->get_last_qt_data()), array('class' => 'validationerror'));
        }

        return $this->render_from_template('qtype_imagetimeline/question', $data);
    }
}
