<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Defines the editing form for the 'missingtype' question type.
 *
 * @package    qtype
 * @subpackage missingtype
 * @copyright  2024 LMSACE Dev Team <lmsace.com>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */


defined('MOODLE_INTERNAL') || die();


/**
 * This question renderer class is used when the actual question type of this
 * question cannot be found.
 *
 * @copyright  2024 LMSACE Dev Team <lmsace.com>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class qtype_multiorder_edit_form extends question_edit_form {

    public function qtype() {
        return 'multiorder';
    }

    /**
     * definition_inner adds all specific fields to the form.
     * @param object $mform (the form being built).
     */
    protected function definition_inner($mform) {
        global $CFG;

        $this->definition_answer_choice($mform);
        // Add the answer (choice) fields to the form.
        $this->definition_groups($mform);

        $this->add_combined_feedback_fields(true);

        $this->add_interactive_settings(true, true);
    }

    public function data_preprocessing($question) {
        $question = parent::data_preprocessing($question);
        $question = $this->data_preprocessing_combined_feedback($question, true);
        $question = $this->data_preprocessing_hints($question, true, true);
        $question = $this->data_preprocessing_answers($question, true);
        if (!empty($question->options->answers)) {
            $key = 0;
            foreach ($question->options->answers as $answer) {
                $question = $this->data_preprocessing_choice($question, $answer, $key);
                $key++;
            }
        }

        if (!empty($question->options->groups)) {
            $key = 0;
            foreach ($question->options->groups as $group) {
                $question = $this->data_preprocessing_group($question, $group, $key);
                $key++;
            }
        }
        return $question;
    }


    protected function data_preprocessing_group($question, $group, $key) {
        $question->groups[$key]['groupname'] = $group->name;
        $question->groups[$key]['selectoptions'] = $group->selectoption;
        return $question;
    }


    protected function data_preprocessing_choice($question, $answer, $key) {
        $question->choices[$key]['answer'] = $answer->answer;;
        $options = unserialize_object($answer->feedback);
        $question->choices[$key]['choicegroup'] = $options->draggroup ?? 1;
        return $question;
    }

    protected function add_combined_feedback_fields($withshownumpartscorrect = false) {
        $mform = $this->_form;

        $mform->addElement('header', 'combinedfeedbackhdr',
                get_string('combinedfeedback', 'question'));

        $fields = array('correctfeedback', 'partiallycorrectfeedback', 'incorrectfeedback');
        foreach ($fields as $feedbackname) {
            $element = $mform->addElement('editor', $feedbackname,
                    get_string($feedbackname, 'question'),
                    array('rows' => 5), $this->editoroptions);
            $mform->setType($feedbackname, PARAM_RAW);
            // Using setValue() as setDefault() does not work for the editor class.
            $element->setValue(array('text' => get_string($feedbackname .'default', 'question')));

            if ($withshownumpartscorrect && $feedbackname == 'partiallycorrectfeedback') {
                $mform->addElement('advcheckbox', 'shownumcorrect',
                        get_string('options', 'question'),
                        get_string('shownumpartscorrectwhenfinished', 'question'));
                $mform->setDefault('shownumcorrect', true);
            }

        }
    }

    public function definition_groups(&$mform) {
        $mform->addElement('header', 'groupshdr', get_string('groups', 'qtype_multiorder'));
        $mform->setExpanded('choicehdr', 1);

        if (!empty($this->question->options->answers)) {
            $repeatsatstart = count($this->question->options->answers);
        } else {
            $repeatsatstart = QUESTION_NUMANS_ADD * 2;
        }

        $repeats = $this->optional_param('noanswers', $repeatsatstart, PARAM_INT);
        $addfields = $this->optional_param('addanswers', '', PARAM_TEXT);
        if (!empty($addfields)){
            $repeats += QUESTION_NUMANS_ADD;
        }

        $grouparray = [];
        $grouparray[] = $mform->createElement('text', 'groupname', get_string('strname', 'qtype_multiorder'));
        $grouparray[] = $mform->createElement('hidden', 'groupid', 0);
        $grouparray[] = $mform->createElement('select', 'selectoptions', get_string('correctoptions', 'qtype_multiorder'),
            array_combine(range(1, $repeats), range(1, $repeats)));
        $textboxgroup = array();
        $textboxgroup[] = $mform->createElement('group', 'groups',
            get_string('groupx', 'qtype_multiorder'), $grouparray);

        $repeatsatstart = $this->get_initial_choice_group_number();

        if (!empty($this->question->options->groups)) {
            $repeatsatstart = count($this->question->options->groups);
        }

        $repeatoptions = $this->group_repeated_options($repeats);
        $this->group_repeat_elements($textboxgroup, $repeatsatstart, $repeatoptions,
            'nochoicegroups', 'addchoicegroups', 2, 'Add more groups');
    }

    /**
     * Returns an array for form repeat options.
     *
     * @return array Array of repeate options
     */
    protected function group_repeated_options($repeats) {
        $repeatedoptions = array();
        $repeatedoptions['groups[selectoptions]']['default'] = $repeats;
        $repeatedoptions['groups[groupname]']['type'] = PARAM_RAW;
        $repeatedoptions['groups[groupid]']['type'] = PARAM_INT;
        $repeatedoptions['groups[groupid]']['default'] = $repeats;
        return $repeatedoptions;
    }

    public function group_repeat_elements($elementobjs, $repeats, $options,
        $repeathiddenname, $addfieldsname, $addfieldsno = 5, $addstring = null) {
        global $DB;

        $repeats = $this->optional_param($repeathiddenname, $repeats, PARAM_INT);
        $addfields = $this->optional_param($addfieldsname, '', PARAM_TEXT);
        $oldrepeats = $repeats;
        if (!empty($addfields)){
            $repeats += $addfieldsno;
        }

        $mform =& $this->_form;
        $mform->registerNoSubmitButton($addfieldsname);
        $mform->addElement('hidden', $repeathiddenname, $repeats);
        $mform->setType($repeathiddenname, PARAM_INT);
        $mform->setConstants(array($repeathiddenname=>$repeats));

        $namecloned = array();
        $no = 1;
        for ($i = 0; $i < $repeats; $i++) {
            foreach ($elementobjs as $elementobj) {
                $elementclone = fullclone($elementobj);
                $this->group_repeat_elements_fix_clone($i, $elementclone, $namecloned, $no);
                if ($i >= $oldrepeats) {
                    $mform->note_new_repeat($elementclone->getName());
                }
                $mform->addElement($elementclone);
                $no++;
            }
        }
        $no = 1;
        for ($i=0; $i < $repeats; $i++) {

            foreach ($options as $elementname => $elementoptions) {
                $pos=strpos($elementname, '[');
                if ($pos!==FALSE){
                    $realelementname = substr($elementname, 0, $pos)."[$i]";
                    $realelementname .= substr($elementname, $pos);
                } else {
                    $realelementname = $elementname."[$i]";
                }
                foreach ($elementoptions as  $option => $params) {
                    switch ($option){
                        case 'default':
                            if ($elementname == 'groups[selectoptions]') {
                                if (!isset($this->question->id)) {
                                    $mform->setDefault($realelementname, str_replace('{no}', question_utils::int_to_letter($no), $params));
                                } else if (isset($this->question->id) && !$DB->record_exists('question_multiorder_groups',
                                    ['questionid' => $this->question->id, 'groupid' => $no])) {
                                        $mform->setDefault($realelementname, str_replace('{no}', question_utils::int_to_letter($no), $params));
                                }
                            } else if ($elementname == 'groups[groupid]') {
                                $mform->setDefault($realelementname, $no);
                            }
                            break;
                        case 'type':
                            $mform->setType($realelementname, $params);
                            break;
                        }
                }
            }
            $no++;
        }
        if ($repeats < $this->get_maximum_choice_group_number()) {
            $mform->addElement('submit', $addfieldsname, $addstring, [], false);
        }
    }

    /**
     * Helper used by {@link repeat_elements()}.
     *
     * @param int $i the index of this element.
     * @param HTML_QuickForm_element $elementclone
     * @param array $namecloned array of names
     */
    function group_repeat_elements_fix_clone($i, $elementclone, &$namecloned, $no) {
        $name = $elementclone->getName();
        $namecloned[] = $name;

        if (!empty($name)) {
            $elementclone->setName($name."[$i]");
        }

        $value=$elementclone->getLabel();
        $elementclone->setLabel(str_replace('{no}', question_utils::int_to_letter($no), $value));
    }


    public function validation($data, $files) {
        $errors = parent::validation($data, $files);
        foreach ($data['choices'] as $choices) {
            $choicegroup = $choices['choicegroup'];
            $groupkey = $choicegroup - 1;
            if (isset($data['groups'][$groupkey]))  {
                if (empty($data['groups'][$groupkey]['groupname']) && $groupkey > 0) { // More then one group condition is vaild.
                    $errors['groups['.$groupkey.']'] = get_string('missinggroupname', 'qtype_multiorder');
                }
            } else {
                // Missing gropus.
            }
        }
        return $errors;
    }

    /**
     * Defines form elements for answer choices.
     *
     * @param object $mform The Moodle form object being built
     */
    protected function definition_answer_choice(&$mform) {
        $mform->addElement('header', 'choicehdr', get_string('options', 'qtype_multiorder'));
        $mform->setExpanded('choicehdr', 1);

        $mform->addElement('static', 'choicehdrdescription', get_string('choicehdrdescription', 'qtype_multiorder'));

        $textboxgroup = array();
        $textboxgroup[] = $mform->createElement('group', 'choices',
                get_string('optionx', 'qtype_multiorder'), $this->choice_group($mform));

        if (!empty($this->question->options->answers)) {
            $repeatsatstart = count($this->question->options->answers);
        } else {
            $repeatsatstart = QUESTION_NUMANS_ADD * 2;
        }

        $repeatedoptions = $this->repeated_options();
        $mform->setType('answer', PARAM_RAW);
        $this->repeat_elements($textboxgroup, $repeatsatstart, $repeatedoptions,
                'noanswers', 'addanswers', QUESTION_NUMANS_ADD,
                get_string('addmorechoiceblanks', 'qtype_multiorder'), true);
    }

    /**
     * Creates an array with elements for a choice group.
     *
     * @param object $mform The Moodle form we are working with
     * @param int $maxgroup The number of max group generate element select.
     * @return array Array for form elements
     */
    protected function choice_group($mform) {
        $options = array();

        $repeats = $this->optional_param('nochoicegroups', $this->get_initial_choice_group_number(), PARAM_INT);
        $addfields = $this->optional_param('addchoicegroups', '', PARAM_TEXT);

        if (!empty($this->question->options->groups)) {
            $repeats = count($this->question->options->groups);
        }

        if (!empty($addfields)){
            $repeats += 2;
        }

        for ($i = 1; $i <= $repeats; $i += 1) {
            $options[$i] = question_utils::int_to_letter($i);
        }
        $grouparray = array();
        $grouparray[] = $mform->createElement('text', 'answer',
                get_string('value', 'qtype_multiorder'), array('size' => 30, 'class' => 'tweakcss'));
        $grouparray[] = $mform->createElement('select', 'choicegroup',
        get_string('group', 'qtype_multiorder'), $options);
        return $grouparray;
    }

    /**
     * Returns an array for form repeat options.
     *
     * @return array Array of repeate options
     */
    protected function repeated_options() {
        $repeatedoptions = array();
        $repeatedoptions['choicegroup']['default'] = '1';
        $repeatedoptions['choices[answer]']['type'] = PARAM_RAW;
        return $repeatedoptions;
    }

    function get_maximum_choice_group_number() {
        return 13;
    }

    function get_add_choice_group_number() {
        return 2;
    }
    function get_initial_choice_group_number() {
        return 3;
    }
}