<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Drag-and-drop words into sentences question definition class.
 *
 * @package   qtype_multiorder
 * @copyright 2024 LMSACE Dev Team <lmsace.com>
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

 require_once($CFG->dirroot . '/question/type/questionbase.php');

/**
 * Represents a drag-and-drop words into sentences question.
 *
 * @copyright  2024 LMSACE Dev Team <lmsace.com>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class qtype_multiorder_question extends question_graded_automatically_with_countback {


     /**
     * @var array of arrays. The outer keys are the choice group numbers.
     * The inner keys for most question types number sequentialy from 1. However
     * for ddimageortext questions it is strange (and difficult to change now).
     * the first item in each group gets numbered 1, and the other items get numbered
     * $choice->no. Be careful!
     * The values are arrays of qtype_multiorder_choice objects (or a subclass).
     */
    public $choices;

    /** @var string Feedback for any correct response. */
    public $correctfeedback;
    /** @var int format of $correctfeedback. */
    public $correctfeedbackformat;
    /** @var string Feedback for any partially correct response. */
    public $partiallycorrectfeedback;
    /** @var int format of $partiallycorrectfeedback. */
    public $partiallycorrectfeedbackformat;
    /** @var string Feedback for any incorrect response. */
    public $incorrectfeedback;
    /** @var int format of $incorrectfeedback. */
    public $incorrectfeedbackformat;

        /**
     * @var array place number => group number of the places in the question
     * text where choices can be put. Places are numbered from 1.
     */
    public $places;

    public $placesgroups;

    /**
     * @var array of strings, one longer than $places, which is achieved by
     * indexing from 0. The bits of question text that go between the placeholders.
     */
    public $textfragments;

    /** @var array index of the right choice for each stem. */
    public $rightchoices;

    /** @var array shuffled choice indexes. */
    protected $choiceorder;

     /** @var array choice groups. */
    public $choicegroups;


    public function get_expected_data() {
        // TODO.
        $vars = array();
        foreach ($this->places as $place => $notused) {
            $vars[$this->field($place)] = PARAM_INT;
        }
        return $vars;
    }

    public function start_attempt(question_attempt_step $step, $variant) {
        //TODO
        /* there are 9 occurrances of this method defined in files called question.php a new install of Moodle
        so you are probably going to have to define it */
        $ind = 1;
        foreach ($this->choices as $group => $choices) {
            $choiceorder = array_keys($choices);
            //TODO:  check the if suffle settings need to want or not.
            shuffle($choiceorder);
            $step->set_qt_var('_choiceorder' . $group, implode(',', $choiceorder));
            $this->set_choiceorder($group, $choiceorder, $ind);
        }
    }

    public function apply_attempt_state(question_attempt_step $step) {
        $ind = 1;
        foreach ($this->choices as $group => $choices) {
            $this->set_choiceorder($group, explode(',',
                    $step->get_qt_var('_choiceorder' . $group)), $ind);
        }
    }


    public function is_gradable_response(array $response) {
        foreach ($this->places as $place => $notused) {
            if (!empty($response[$this->field($place)])) {
                return true;
            }
        }
        return false;
    }

    /**
     * Helper method used by both {@link start_attempt()} and
     * {@link apply_attempt_state()}.
     * @param int $group the group number.
     * @param array $choiceorder the choices, in order.
     */
    protected function set_choiceorder($group, $choiceorder, &$ind) {
        foreach ($choiceorder as $key => $value) {
            $this->choiceorder[$group][$ind] = $value;
            $ind++;
        }
    }

    public function validate_can_regrade_with_other_version(question_definition $otherversion): ?string {
        $basemessage = parent::validate_can_regrade_with_other_version($otherversion);
        if ($basemessage) {
            return $basemessage;
        }

        if (count($this->choices) != count($otherversion->choices)) {
            return get_string('regradeissuenumgroupsschanged', 'qtype_multiorder');
        }

        foreach ($this->choices as $group => $choices) {
            if (count($this->choices[$group]) != count($otherversion->choices[$group])) {
                return get_string('regradeissuenumchoiceschanged', 'qtype_multiorder', $group);
            }
        }

        return null;
    }


    public function get_question_summary() {
        $question = $this->html_to_text($this->questiontext, $this->questiontextformat);
        $groups = array();
        foreach ($this->choices as $group => $choices) {
            $cs = array();
            foreach ($choices as $choice) {
                $cs[] = html_to_text($choice->text, 0, false);
            }
            $groups[] = '[[' . $group . ']] -> {' . implode(' / ', $cs) . '}';
        }
        return $question . '; ' . implode('; ', $groups);
    }

    public function field($place) {
        return 'p' . $place;
    }


    public function summarise_response(array $response) {
        // TODO.
        $matches = array();
        $allblank = true;
        $choices = [];
        $ind = 1;
        foreach ($this->placesgroups as $group => $places) {
            $groupinfo = $this->choicegroups[$group];
            foreach ($places as $place => $notused) {
                if (array_key_exists($this->field($ind), $response)) {
                    if (isset($response[$this->field($ind)]) && !empty($response[$this->field($ind)])) {
                        $choices[] = '{' . $this->summarise_choice(
                            $this->get_selected_choice($group, $response[$this->field($ind)])) . '}';
                        $allblank = false;
                        $ind++;
                    }
                } else {
                    $choices[] = '{}';
                }
            }
        }

        if ($allblank) {
            return null;
        }
        return implode(' ', $choices);
    }

    public function summarise_choice($choice) {
        return $this->html_to_text($choice->text, FORMAT_PLAIN);
    }

    public function get_random_guess_score() {
        $accum = 0;

        foreach ($this->places as $placegroup) {
            $accum += 1 / count($this->choices[$placegroup]);
        }

        return $accum / count($this->places);
    }

    protected function get_selected_choice($group, $shuffledchoicenumber) {
        $choiceno = $this->choiceorder[$group][$shuffledchoicenumber];
        return isset($this->choices[$group][$choiceno]) ? $this->choices[$group][$choiceno] : null;
    }

    public function is_complete_response(array $response) {
        // TODO.
        $complete = true;
        $countplaces = 0;
        $placeschecked = 0;
        foreach ($this->placesgroups as $group => $places) {
            $group = $this->choicegroups[$group];
            $countplaces += count($places);
            foreach ($places as $place => $notused) {
                if (!empty($response[$this->field($place)])) {
                    $placeschecked++;
                }
            }
        }

        if ($countplaces != $placeschecked) {
            return false;
        }
        return $complete;
    }

    public function get_validation_error(array $response) {
        if ($this->is_complete_response($response)) {
            return '';
        }
        return get_string('pleaseputananswerineachbox', 'qtype_multiorder');
    }

    public function is_same_response(array $prevresponse, array $newresponse) {
        // TODO.
        foreach ($this->places as $place => $notused) {
            $fieldname = $this->field($place);
            if (!question_utils::arrays_same_at_key_integer(
                    $prevresponse, $newresponse, $fieldname)) {
                return false;
            }
        }
        return true;
    }

    public function get_correct_response() {
        // TODO.
        $response = array();
        foreach ($this->places as $place => $notused) {
            $response[$this->field($place)] = $this->get_right_choice_for($place);
        }
        return $response;
    }

    public function get_ordered_choices($group) {
        $choices = array();
        foreach ($this->choiceorder[$group] as $choicekey => $choiceid) {
            $choices[$choicekey] = $this->choices[$group][$choiceid];
        }
        return $choices;
    }


    public function get_right_choice_for($place) {

        $group = $this->places[$place];
        foreach ($this->choiceorder[$group] as $choicekey => $choiceid) {
            if (isset($this->rightchoices[$group][$place])) {
                if ($this->rightchoices[$group][$place] == $choiceid) {
                    return $choicekey;
                }
            }
        }
    }

    /**
     * Given a response, reset the parts that are wrong. Relevent in
     * interactive with multiple tries
     * @param array $response a response
     * @return array a cleaned up response with the wrong bits reset.
     */
    public function clear_wrong_from_response(array $response) {
        foreach ($this->places as $place => $notused) {
            if (array_key_exists($this->field($place), $response) &&
                    $response[$this->field($place)] != $this->get_right_choice_for($place)) {
                $response[$this->field($place)] = '0';
            }
        }
        return $response;
    }

    public function check_file_access($qa, $options, $component, $filearea,
            $args, $forcedownload) {
        // TODO.
        if ($component == 'question' && in_array($filearea,
                array('correctfeedback', 'partiallycorrectfeedback', 'incorrectfeedback'))) {
            return $this->check_combined_feedback_file_access($qa, $options, $filearea, $args);

        } else if ($component == 'question' && $filearea == 'hint') {
            return $this->check_hint_file_access($qa, $options, $args);

        } else {
            return parent::check_file_access($qa, $options, $component, $filearea,
                    $args, $forcedownload);
        }
    }
    /**
     * @param array $response responses, as returned by
     *      {@link question_attempt_step::get_qt_data()}.
     * @return array (number, integer) the fraction, and the state.
     */
    public function grade_response(array $response) {
        // TODO.

        list($right, $total) = $this->get_num_parts_right($response);
        $fraction = $right / $total;
        return array($fraction, question_state::graded_state_for_fraction($fraction));
    }

    public function get_num_parts_right(array $response) {
        $numright = 0;
        $countplaces = 0;
        foreach ($this->placesgroups as $group => $places) {
            $group = $this->choicegroups[$group];
            $countplaces += count($places);
            foreach ($places as $place => $notused) {
                if (!array_key_exists($this->field($place), $response)) {
                    continue;
                }
                if ($response[$this->field($place)] == $this->get_right_choice_for($place)) {
                    $numright += 1;
                }
            }
        }
        return array($numright, $countplaces);
    }

    /**
     * Work out a final grade for this attempt, taking into account all the
     * tries the student made. Used in interactive behaviour once all
     * hints have been used.     * 
     * @param array $responses an array of arrays of the response for each try. 
     * Each element of this array is a response array, as would be 
     * passed to {@link grade_response()}. There may be between 1 and 
     * $totaltries responses. 
     * @param int $totaltries is the maximum number of tries allowed. Generally 
     * not used in the implementation.
     * @return numeric the fraction that should be awarded for this
     * sequence of response. 
     * 
     */
    public function compute_final_grade($responses, $totaltries) {
        $totalscore = 0;
        foreach ($this->places as $place => $notused) {
            $fieldname = $this->field($place);

            $lastwrongindex = -1;
            $finallyright = false;
            foreach ($responses as $i => $response) {
                if (!array_key_exists($fieldname, $response) ||
                        $response[$fieldname] != $this->get_right_choice_for($place)) {
                    $lastwrongindex = $i;
                    $finallyright = false;
                } else {
                    $finallyright = true;
                }
            }

            if ($finallyright) {
                $totalscore += max(0, 1 - ($lastwrongindex + 1) * $this->penalty);
            }
        }
        return $totalscore / count($this->places);
    }
}


/**
 * Represents one of the choices (draggable boxes).
 *
 * @copyright  2024 LMSACE Dev Team <lmsace.com>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class qtype_multiorder_choice {
    /** @var string Text for the choice */
    public $text;

    /** @var int Group of the choice */
    public $draggroup;

    public $id;

    /**
     * Initialize a choice object.
     *
     * @param string $text The text of the choice
     * @param int $draggroup Group of the drop choice
     * @param bool $infinite True if the item can be used an unlimited number of times
     */
    public function __construct($text, $id, $draggroup = 1,) {
        $this->text = $text;
        $this->id = $id;  // Unique identifier for the choice. Used for drag and drop.  // Unique identifier for the choice. Used for drag and drop.  // Unique identifier for the choice. Used for drag and drop.  // Unique identifier for the choice. Used for drag and drop.  // Unique identifier for the choice. Used for drag and drop.  // Unique identifier for the choice. Used for drag and drop.  // Unique identifier for the choice.
        $this->draggroup = $draggroup;
    }

     /**
     * Returns the group of this item.
     *
     * @return int
     */
    public function choice_id() {
        return $this->id;
    }

    /**
     * Returns the group of this item.
     *
     * @return int
     */
    public function choice_group() {
        return $this->draggroup;
    }
}