<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * The questiontype class for the multiple choice question type.
 *
 * @package    qtype
 * @subpackage multiorder
 * @copyright  2024 LMSACE Dev Team <lmsace.com>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */


defined('MOODLE_INTERNAL') || die();

global $CFG;
require_once($CFG->libdir . '/questionlib.php');


/**
 * The multiple choice question type.
 *
 * @copyright  2024 LMSACE Dev Team <lmsace.com>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class qtype_multiorder extends question_type {

    public function move_files($questionid, $oldcontextid, $newcontextid) {
        parent::move_files($questionid, $oldcontextid, $newcontextid);
        $this->move_files_in_combined_feedback($questionid, $oldcontextid, $newcontextid);
        $this->move_files_in_hints($questionid, $oldcontextid, $newcontextid);
    }

    protected function delete_files($questionid, $contextid) {
        parent::delete_files($questionid, $contextid);
        $this->delete_files_in_combined_feedback($questionid, $contextid);
        $this->delete_files_in_hints($questionid, $contextid);
    }

    public function save_question_options($question) {
        global $DB;
        $context = $question->context;

        // This question type needs the choices to be consecutively numbered, but
        // there is no reason why the question author should have done that,
        // so renumber if necessary.
        // Insert all the new answers.
        $nonblankchoices = [];
        $questiontext = $question->questiontext;
        foreach ($question->choices as $key => $choice) {
            if (trim($choice['answer']) == '') {
                continue;
            }
            $nonblankchoices[] = $choice;
        }

        $nonblankgroups = [];

        if (isset($question->groups)) {
            foreach ($question->groups as $key => $group) {
                if (trim($group['groupname']) == '' && $group['groupid'] != 1) {
                    continue;
                }
                $nonblankgroups[] = $group;
            }
        }
        $question->choices = $nonblankchoices;
        $question->groups = $nonblankgroups;

        if ($questiontext !== $question->questiontext) {
            $DB->set_field('question', 'questiontext', $questiontext,
                    ['id' => $question->id]);
            $question->questiontext = $questiontext;
        }

        $oldanswers = $DB->get_records('question_answers',
                array('question' => $question->id), 'id ASC');
        $groupcounts = [];
        // Insert all the new answers.
        foreach ($question->choices as $key => $choice) {
            $groupcounts[$choice['choicegroup']] = isset($groupcounts[$choice['choicegroup']]) ? $groupcounts[$choice['choicegroup']] + 1 : 1;
            // Answer guaranteed to be non-blank. See above.
            $feedback = $this->choice_options_to_feedback($choice);

            if ($answer = array_shift($oldanswers)) {
                $answer->answer = trim($choice['answer']);
                $answer->groupid = $choice['choicegroup'];
                $answer->feedback = $feedback;
                $DB->update_record('question_answers', $answer);

            } else {
                $answer = new stdClass();
                $answer->question = $question->id;
                $answer->answer = $choice['answer'];
                $answer->groupid = $choice['choicegroup'];
                $answer->answerformat = FORMAT_HTML;
                $answer->fraction = 0;
                $answer->feedback = $feedback;
                $answer->feedbackformat = 0;
                $DB->insert_record('question_answers', $answer);
            }
        }

        // Insert all the groups data.
        foreach ($question->groups as $key => $group) {
            $groupid = $group['groupid'];
            $groupname = empty($group['groupname']) ? get_string("strgroupname", "qtype_multiorder", question_utils::int_to_letter($groupid))
            : trim($group['groupname']);
            $groupcount = isset($groupcounts[$groupid]) ? $groupcounts[$groupid] : 0;
            $groupoption = ($group['selectoptions'] <= $groupcount) ? $group['selectoptions'] : $groupcount;
            if (!$DB->record_exists('question_multiorder_groups', ['id' => $groupid, 'questionid' => $question->id])) {
                $group = new stdClass;
                $group->groupid = $groupid;
                $group->questionid = $question->id;
                $group->name = $groupname;
                $group->selectoption = $groupoption;
                $group->timecreated = time();
                $DB->insert_record('question_multiorder_groups', $group);
            } else {
                $group = $DB->get_record('question_multiorder_groups', ['id' => $groupid, 'questionid' => $question->id]);
                if ($group->name != $groupname || $group->groupoption != $groupoption) {
                    $group->name = $groupname;
                    $group->selectoption = $groupoption;
                    $group->timemodified = time();
                    $DB->update_record('question_multiorder_groups', $group);
                }
            }
        }

        // Delete old answer records.
        foreach ($oldanswers as $oa) {
            $DB->delete_records('question_answers', array('id' => $oa->id));
        }

        $options = $DB->get_record('question_' . $this->name(),
                array('questionid' => $question->id));
        if (!$options) {
            $options = new stdClass();
            $options->questionid = $question->id;
            $options->correctfeedback = '';
            $options->partiallycorrectfeedback = '';
            $options->incorrectfeedback = '';
            $options->id = $DB->insert_record('question_' . $this->name(), $options);
        }

        $options = $this->save_combined_feedback_helper($options, $question, $context, true);
        $DB->update_record('question_' . $this->name(), $options);

        $this->save_hints($question, true);
    }

    protected function choice_options_to_feedback($choice) {
        $output = new stdClass();
        $output->draggroup = $choice['choicegroup'];
        return serialize($output);
    }

    public function get_question_options($question) {
        global $DB;
        $question->options = $DB->get_record('question_'.$this->name(),
                array('questionid' => $question->id), '*', MUST_EXIST);
        if ($DB->record_exists('question_'.$this->name(). '_groups', ['questionid' => $question->id])) {
            $question->options->groups = $DB->get_records('question_'.$this->name(). '_groups', ['questionid' => $question->id]);
        }
        parent::get_question_options($question);
    }

    public function delete_question($questionid, $contextid) {
        global $DB;
        $DB->delete_records('question_'.$this->name(), array('questionid' => $questionid));
        $DB->delete_records('question_multiorder_groups', ['questionid' => $questionid]);
        return parent::delete_question($questionid, $contextid);
    }

    protected function make_choice($choicedata, $id) {
        $options = $this->unserialize_feedback($choicedata->feedback);
        return new qtype_multiorder_choice(
                $choicedata->answer, $id, $options->draggroup);
    }

    /**
     * Safely convert given serialized feedback string into valid feedback object
     *
     * @param string $feedback
     * @return stdClass
     */
    protected function unserialize_feedback(string $feedback): stdClass {
        $feedbackobject = unserialize_object($feedback);

        return (object) [
            'draggroup' => $feedbackobject->draggroup ?? 1,
        ];
    }


    protected function initialise_question_instance(question_definition $question, $questiondata) {
        parent::initialise_question_instance($question, $questiondata);

        $this->initialise_combined_feedback($question, $questiondata, true);

        $question->choices = array();
        $choiceindexmap = array();

        // Store the choices in arrays by group.
        $i = 1;
        foreach ($questiondata->options->answers as $choicedata) {
            $choice = $this->make_choice($choicedata, $i);

            if (array_key_exists($choice->choice_group(), $question->choices)) {
                $question->choices[$choice->choice_group()][$i] = $choice;
            } else {
                $question->choices[$choice->choice_group()][$i] = $choice;
            }

            end($question->choices[$choice->choice_group()]);
            $choiceindexmap[$i] = array($choice->choice_group(),
                    key($question->choices[$choice->choice_group()]));
            $i += 1;
        }
        $question->choicegroups = [];

        if (isset($questiondata->options->groups)) {
            foreach ($questiondata->options->groups as $group) {
                if (!isset($question->choices[$group->groupid])) {
                    continue;
                }
                // If group option is higher then choices of group change the value to maximum.
                $group->selectoption = ($group->selectoption < count($question->choices[$group->groupid])) ? $group->selectoption : count($question->choices[$group->groupid]);
                $question->choicegroups[$group->groupid] = $group;
            }
        }

        $question->places = array();
        $question->placesgroups = array();
        $question->textfragments = array();
        $question->rightchoices = array();
        $question->textfragments[0] = $question->questiontext;

        $i = 1;
        $choiceorderind = 1;
        $gi = 0;

        foreach ($question->choices as $key => $choices) {
            $groupinfo = $question->choicegroups[$key];
            $groupchoices = count($choices);
            $groupind = 0;
            $sliceopitons = array_slice($choices, 0, $groupinfo->selectoption);

            for ($k = 1; $k <= $groupinfo->selectoption; $k++) {
                $question->places[$i] = $key;
                $question->placesgroups[$key][$i] = $key;
                $question->rightchoices[$key][$i] = $sliceopitons[$groupind]->id;
                $i++;
                $groupind++;
            }

            if ($groupchoices != $groupinfo->selectoption) {
                $gi += $groupchoices - $groupinfo->selectoption;
            }
            $choiceorderind++;
        }
    }


    public function import_from_xml($data, $question, qformat_xml $format, $extra=null) {

        if (!isset($data['@']['type']) || $data['@']['type'] != 'multiorder') {
            return false;
        }

        $question = $format->import_headers($data);
        $question->qtype = 'multiorder';
        $groups = $data['#']['group'];

        $question->groups = array();

        foreach ($groups as $groupxml) {

            $groupindex = $format->getpath($groupxml, array('#', 'id', 0, '#'), 0);
            $record = [];

            $record['id'] = $groupindex;
            $record['questionid'] = $format->getpath($groupxml, array('#', 'questionid', 0, '#'), 1);
            $record['groupid'] = $format->getpath($groupxml, array('#', 'groupid', 0, '#'), 1);
            $record['groupname'] = $format->getpath($groupxml, array('#', 'name', 0, '#'), true);
            $record['selectoptions'] = $format->getpath($groupxml, array('#', 'option', 0, '#'), 1);
            $record['timecreated'] = $format->getpath($groupxml, array('#', 'timecreated', 0, '#'), 1);
            $record['timemodified'] = $format->getpath($groupxml, array('#', 'timemodified', 0, '#'), 1);
            $question->groups[$groupindex] = $record;
        }

        if (!empty($data['#']['selectoption'])) {
            // Modern XML format.
            $selectoptions = $data['#']['selectoption'];
            $question->answer = array();
            $question->selectgroup = array();

            foreach ($data['#']['selectoption'] as $selectoptionxml) {
                $choicegroup = $format->getpath($selectoptionxml,
                array('#', 'group', 0, '#'), 1);
                $question->choices[] = array(
                    'answer'      => $format->getpath($selectoptionxml,
                                                      array('#', 'text', 0, '#'), '', true),
                    'choicegroup' => $choicegroup,
                );
            }

        } else {
            // Legacy format containing PHP serialisation.
            foreach ($data['#']['answer'] as $answerxml) {
                $ans = $format->import_answer($answerxml);
                $question->choices[] = array(
                    'answer' => $ans->answer,
                    'choicegroup' => $ans->feedback,
                );
            }
        }

        $format->import_combined_feedback($question, $data, true);
        $format->import_hints($question, $data, true, false,
                $format->get_format($question->questiontextformat));

        return $question;
    }

    public function export_to_xml($question, qformat_xml $format, $extra = null) {
        $output = '';
        $output .= $format->write_combined_feedback($question->options,
                                                    $question->id,
                                                    $question->contextid);

        foreach ($question->options->answers as $answer) {
            $options = $this->unserialize_feedback($answer->feedback);
            $output .= "    <selectoption>\n";
            $output .= $format->writetext($answer->answer, 3);
            $output .= "      <group>{$options->draggroup}</group>\n";
            $output .= "    </selectoption>\n";
        }

        foreach ($question->options->groups as $group) {
            $output .= "    <group>\n";
            $output .= "     <id>{$group->id}</id>\n";
            $output .= "     <questionid>{$group->questionid}</questionid>\n";
            $output .= "     <groupid>{$group->groupid}</groupid>\n";
            $output .= "     <name>{$group->name}</name>\n";
            $output .= "     <option>{$group->selectoption}</option>\n";
            $output .= "     <timecreated>{$group->timecreated}</timecreated>\n";
            $output .= "     <timemodified>{$group->timemodified}</timemodified>\n";
            $output .= "    </group>\n";
        }

        return $output;
    }
}
