<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Drag-and-drop words into sentences question renderer class.
 *
 * @package   qtype_multiorder
 * @copyright 2024 LMSACE Dev Team <lmsace.com>
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */


defined('MOODLE_INTERNAL') || die();

/**
 * Generates the output for drag-and-drop words into sentences questions.
 *
 * @copyright  2024 LMSACE Dev Team <lmsace.com>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class qtype_multiorder_renderer extends qtype_with_combined_feedback_renderer {

    public function formulation_and_controls(question_attempt $qa,
            question_display_options $options) {

        $question = $qa->get_question();

        $questiontext = '';
        $questionhead = '';
        $questionhead .= html_writer::start_div('que-multiorder-head');
        $questionhead .= html_writer::tag('p', $question->format_questiontext($qa));
        $questionhead .= html_writer::end_div();
        $quegroupchoices = "";

        $classes = "que-multi-order";
        $issinglegroup = true;
        if (count($question->choices) > 1) {
            $issinglegroup = false;
            $classes .= " group-based-order";
        }

        $result = html_writer::start_div($classes);
        $result .= $questionhead;
        $place = 1;
        $ind = 1;

        // spearte based on chocie groups.
        foreach ($question->choices as $groupid => $choices) {
            $quegroupchoices .= html_writer::start_div('choice-group');
            // Added the Group name.
            if (!$issinglegroup) {
                $groupname = isset($question->choicegroups[$groupid]->name) ? $question->choicegroups[$groupid]->name :
                    get_string("strgroupname", "qtype_multiorder", question_utils::int_to_letter($groupid));
                $quegroupchoices .= html_writer::tag('h3', $groupname);
            }
            $quegroupchoices .= $this->post_qtext_elements($qa, $options, $groupid, $ind);
            $quegroupchoices .= html_writer::start_div('qtext-block');
            $quegroupchoices .= html_writer::start_div('answer-order-block');
            $quegroupchoices .= html_writer::start_div('block-order-heading');
            $quegroupchoices .= html_writer::tag('div', '', ['class' => 'ind-block']);
            $quegroupchoices .= html_writer::tag('div', get_string('correctorder', 'qtype_multiorder'), ['class' => 'qtext-correct-order-block']);
            $quegroupchoices .= html_writer::tag('i', '', ['class' => 'icon fa fa-fw']);
            $quegroupchoices .= html_writer::end_div();
            if (!empty($question->choicegroups)) {
                for ($i = 1; $i <= $question->choicegroups[$groupid]->selectoption; $i++) {
                    $quegroupchoices .= html_writer::tag('div', $this->embedded_element($qa, $place, $options, $groupid, $i), array('class' => 'qtext'));
                    $place ++;
                }
            } else {
                for ($i = 1; $i <= count(current($question->choices)); $i++) {
                    $quegroupchoices .= html_writer::tag('div', $this->embedded_element($qa, $place, $options, 1, $i), array('class' => 'qtext'));
                    $place ++;
                }
            }
            $quegroupchoices .= html_writer::end_div();
            $quegroupchoices .= html_writer::end_div();
            $quegroupchoices .= html_writer::end_div();
            $ind++;
        }

        $result .= $quegroupchoices;
        // Info block.
        $result .= html_writer::start_div('qtext-info-block');
        $result .= html_writer::tag('i', '', array('class' => 'fa fa-info'));
        $result .= html_writer::tag('p', get_string('dragdropinfo', 'qtype_multiorder'));
        $result .= html_writer::end_div();


        if ($qa->get_state() == question_state::$invalid) {
            $result .= html_writer::nonempty_tag('div',
                    $question->get_validation_error($qa->get_last_qt_data()),
                    array('class' => 'validationerror'));
        }

        $result .= html_writer::end_div();

        $this->page->requires->js_call_amd('qtype_multiorder/multiorder', 'init',
        [$qa->get_outer_question_div_unique_id(), $options->readonly]);

        return $result;
    }

    public function embedded_element(question_attempt $qa, $place,
            question_display_options $options, $groupid, $i) {
        $question = $qa->get_question();
        $group = $groupid;
        $label = $options->add_question_identifier_to_label(get_string('blanknumber', 'qtype_multiorder', $place));
        $boxcontents = '&#160;' . html_writer::tag('span', $label, array('class' => 'accesshide'));

        $value = $qa->get_last_qt_var($question->field($place));

        $attributes = array(
            'class' => 'place' . $place . ' drop active group' . $group
        );

        if ($options->readonly) {
            $attributes['class'] .= ' readonly';
        } else {
            $attributes['tabindex'] = '0';
        }

        $feedbackimage = '';
        if ($options->correctness) {
            $response = $qa->get_last_qt_data();
            $fieldname = $question->field($place);
            if (array_key_exists($fieldname, $response)) {
                $fraction = (int) ($response[$fieldname] ==
                        $question->get_right_choice_for($place));
                $feedbackimage = $this->feedback_image($fraction);
            }
        }
        $content = html_writer::tag('div', $i, array('class' => 'ind-block'));
        $content .= html_writer::start_div('qorder-element');
        $content .= html_writer::tag('span', $boxcontents, $attributes) . ' ' . $feedbackimage;
        $content .= html_writer::end_div();
        return html_writer::tag('p', $content);
    }

    protected function post_qtext_elements(question_attempt $qa,
            question_display_options $options, $groupid, $place) {
        $result = '';
        $question = $qa->get_question();
        $dragboxs = '';

        $dragboxs .= $this->drag_boxes($qa, $groupid,
                $question->get_ordered_choices($groupid), $options, $place);


        $classes = array('answercontainer');
        if ($options->readonly) {
            $classes[] = 'readonly';
        }
        $result .= html_writer::tag('div', $dragboxs, array('class' => implode(' ', $classes)));

        // We abuse the clear_wrong method to output the hidden form fields we
        // want irrespective of whether we are actually clearing the wrong
        // bits of the response.
        if (!$options->clearwrong) {
            $result .= $this->clear_wrong($qa, false, $groupid);
        }
        return $result;
    }


    protected function drag_boxes($qa, $group, $choices, question_display_options $options, $place) {
        $boxes = '';
        $boxes .= html_writer::start_div('block-order-heading');
        $boxes .= html_writer::tag('span', get_string('options', 'qtype_multiorder'), [
            'class' => 'options-head']);
        $boxes .= html_writer::tag('div', '', ['class' => 'ind-block']);
        $boxes .= html_writer::end_div('');
        $choiceind = current(array_keys($qa->get_question()->placesgroups[$place]));
        foreach ($choices as $key => $choice) {
            // Bug 8632: long text entry causes bug in drag and drop field in IE.
            $content = str_replace('-', '&#x2011;', $choice->text);
            //$content = str_replace(' ', '&#160;', $content);
            $boxes .= html_writer::start_div('drag-info-block');
            $boxes .= html_writer::start_div('dragplaceholder-block');
            $boxes .= html_writer::tag('span', $content, [
                    'class' => 'draghome user-select-none choice' . $key . ' group' .
                            $choice->draggroup]) . ' ';
            $boxes .= html_writer::end_div('');
            $boxes .= html_writer::tag('div', '', ['class' => 'ind-block']);
            $boxes .= html_writer::end_div('');
            $choiceind++;
        }

        return html_writer::nonempty_tag('div', $boxes,
            ['class' => 'user-select-none draggrouphomes' . $choice->draggroup]);
    }

    /**
     * Find strings that we can use to glue the fragments with
     *
     * These strings have to be all different and neither of them can be present in the text
     *
     * @param array $fragments
     * @return array array with indexes from 1 to count($fragments)-1
     */
    protected function get_fragments_glue_placeholders($fragments) {
        $fragmentscount = count($fragments);
        if ($fragmentscount <= 1) {
            return [];
        }
        $prefix = '[[$';
        $postfix = ']]';
        $text = join('', $fragments);
        while (preg_match('/' . preg_quote($prefix, '/') . '\\d+' . preg_quote($postfix, '/') . '/', $text)) {
            $prefix .= '$';
        }
        $glues = [];
        for ($i = 1; $i < $fragmentscount; $i++) {
            $glues[$i] = $prefix . $i . $postfix;
        }
        return $glues;
    }

    protected function qtext_id($qa) {
        return str_replace(':', '_', $qa->get_qt_field_name(''));
    }

    protected function box_id(question_attempt $qa, $place) {
        return str_replace(':', '_', $qa->get_qt_field_name($place));
    }

    public function specific_feedback(question_attempt $qa) {
        return $this->combined_feedback($qa);
    }

    public function correct_response(question_attempt $qa) {
        $question = $qa->get_question();

        $correctanswer = '';
        $i = 1;
        $isgroupbase = count($question->choices) > 1 ? true : false;
        $groupcorrectanswer = '';
        foreach ($question->choices as $groupid => $choices) {
            $groupcorrectanswer .= html_writer::start_div('group-order-answer');
            $groupcorrectanswer .= html_writer::start_div('group-element-block');

            $groupname = $question->choicegroups[$groupid]->name;
            $groupcorrectanswer .= html_writer::tag('b', $groupname);

            $groupcorrectvalue  = '';
            $groupcorrectoptions = isset($question->choicegroups[$groupid]->selectoption) ?
            $question->choicegroups[$groupid]->selectoption : count(current($question->choices));
            $grouporderchoices = array_slice($choices, 0, $groupcorrectoptions, true);
            foreach ($grouporderchoices as $orderchoice) {
                $value = '[' . str_replace('-', '&#x2011;',
                $orderchoice->text) . ']';
                $correctanswer .= $value;
                $groupcorrectvalue .= $value . " ";
                $i++;
            }
            $groupcorrectanswer .= html_writer::tag('div', $groupcorrectvalue, array('class' => 'group-answer-block'));
            $groupcorrectanswer .= html_writer::end_div();
            $groupcorrectanswer .= html_writer::end_div();

        }

       if ($isgroupbase) {
            $correctanswer = $groupcorrectanswer;
       }

        if (!empty($correctanswer)) {
            return get_string('correctansweris', 'qtype_multiorder',
                    $question->format_text($correctanswer, $question->questiontextformat,
                            $qa, 'question', 'questiontext', $question->id));
        }
    }


    public function clear_wrong(question_attempt $qa, $reallyclear = true,  $groupid = 0) {
        $question = $qa->get_question();
        $response = $qa->get_last_qt_data();

        if (!empty($response) && $reallyclear) {
            $cleanresponse = $question->clear_wrong_from_response($response);
        } else {
            $cleanresponse = $response;
        }

        $output = '';
        foreach ($question->placesgroups[$groupid] as $place => $group) {
            $fieldname = $question->field($place);
            if (array_key_exists($fieldname, $response)) {
                $value = (string) $response[$fieldname];
            } else {
                $value = '0';
            }
            if (array_key_exists($fieldname, $cleanresponse)) {
                $cleanvalue = (string) $cleanresponse[$fieldname];
            } else {
                $cleanvalue = '0';
            }
            if ($cleanvalue === $value) {
                // Normal case: just one hidden input, to store the
                // current value and be the value submitted.
                $output .= html_writer::empty_tag('input', array(
                        'type' => 'hidden',
                        'id' => $this->box_id($qa, 'p' . $place),
                        'class' => 'placeinput place' . $place . ' group' . $groupid,
                        'name' => $qa->get_qt_field_name($fieldname),
                        'value' => s($value)));
            } else {
                // The case, which only happens when the question is read-only, where
                // we want to show the drag item in a given place (first hidden input),
                // but when submitted, we want it to go to a different place (second input).
                $output .= html_writer::empty_tag('input', array(
                        'type' => 'hidden',
                        'id' => $this->box_id($qa, 'p' . $place),
                        'class' => 'placeinput place' . $place . ' group' . $groupid,
                        'value' => s($value))) .
                        html_writer::empty_tag('input', array(
                        'type' => 'hidden',
                        'name' => $qa->get_qt_field_name($fieldname),
                        'value' => s($cleanvalue)));
            }
        }
        return $output;
    }

}
